@@ -1,5 +1,5 @@
 import sys
-from PyQt4 import Qt
+from taurus.qt import Qt
 from taurus.qt.qtgui.panel import TaurusForm
 
 app = Qt.QApplication(sys.argv)
@@ -8,5 +8,5 @@ panel = TaurusForm()
 props = [ 'state', 'status', 'position', 'velocity', 'acceleration' ]
 model = [ 'sys/taurustest/1/%s' % p for p in props ]
 panel.setModel(model)
-panel.setVisible(True)
+panel.show()
 sys.exit(app.exec_())
