@@ -1,17 +1,16 @@
 import sys
-from PyQt4 import Qt
 from taurus.qt.qtgui.panel import TaurusForm
 from taurus.qt.qtgui.display import TaurusValueLabel
-from taurus.qt.qtgui.input import TaurusWheelEdit
+from taurus.qt.qtgui.application import TaurusApplication
 
-app = Qt.QApplication(sys.argv)
+app = TaurusApplication(sys.argv)
 
 panel = TaurusForm()
 props = [ 'state', 'status', 'position', 'velocity', 'acceleration' ]
 model = [ 'sys/taurustest/1/%s' % p for p in props ]
 panel.setModel(model)
-panel.getItemByIndex(0).setReadWidgetClass(TaurusValueLabel)
-panel.getItemByIndex(2).setWriteWidgetClass(TaurusWheelEdit)
+panel[0].readWidgetClass = TaurusValueLabel
+panel[2].writeWidgetClass='TaurusWheelEdit'
 
-panel.setVisible(True)
+panel.show()
 sys.exit(app.exec_())
