@@ -27,25 +27,11 @@
 
 __docformat__ = "restructuredtext"
 
-import release as Release
-from .enums import *
-from .taurusbasetypes import *
-from .taurusexception import *
-from .taurusmodel import *
-from .tauruslistener import *
-from .taurusdevice import *
-from .taurusattribute import *
-from .taurusconfiguration import *
-from .taurusdatabase import *
-from .taurusfactory import *
-from .taurusmanager import *
-from .taurusoperation import *
-from .tauruspollingtimer import *
-from .taurusvalidator import *
+import taurus.tauruscustomsettings
 
-# enable compatibility code with tau V1 if tauv1 package is present
-try:
-    from .tauv1 import *
-except:
-    pass
+LIGHTWEIGHT_IMPORTS = getattr(taurus.tauruscustomsettings, 'LIGHTWEIGHT_IMPORTS', False)
 
+if LIGHTWEIGHT_IMPORTS:
+    from init_lightweight import *
+else:
+    from init_bkcomp import *
\ No newline at end of file
