@@ -32,15 +32,26 @@ __all__ = ['EvaluationFactory', 'EvaluationDatabase', 'EvaluationDevice',
 
 
 
-import os, time, re, weakref
+import time, re, weakref
 import numpy
 
+import taurus
 from taurus.core.taurusexception import TaurusException
-import taurus.core
-from taurus.core import OperationMode, MatchLevel, TaurusSWDevState, SubscriptionState, TaurusEventType
-from taurus.core.util import SafeEvaluator
+from taurus.core.tauruspollingtimer import TaurusPollingTimer
+from taurus.core.taurusbasetypes import MatchLevel, TaurusSWDevState, \
+    SubscriptionState, TaurusEventType, TaurusAttrValue, TaurusTimeVal, \
+    AttrQuality
+from taurus.core.util.log import Logger
+from taurus.core.util.singleton import Singleton
+from taurus.core.util.safeeval import SafeEvaluator
+from taurus.core.taurusfactory import TaurusFactory
+from taurus.core.taurusattribute import TaurusAttribute
+from taurus.core.taurusdevice import TaurusDevice
+from taurus.core.taurusdatabase import TaurusDatabase
+from taurus.core.taurusconfiguration import TaurusConfiguration
 
-class AbstractEvaluationNameValidator(taurus.core.util.Singleton):
+
+class AbstractEvaluationNameValidator(Singleton):
     #the object name class. *must* be implemented in subclasses
     name_pattern = '' #must be implemented in children classes
     # The following regexp pattern matches <variable>=<value> pairs    
@@ -55,37 +66,37 @@ class AbstractEvaluationNameValidator(taurus.core.util.Singleton):
         self.name_re = re.compile(self.name_pattern)
         self.kvsymbols_re = re.compile(self.kvsymbols_pattern)
         
-    def isValid(self,str, matchLevel = MatchLevel.ANY):
-        return self.name_re.match(str) is not None
+    def isValid(self,s, matchLevel = MatchLevel.ANY):
+        return self.name_re.match(s) is not None
         
-    def getParams(self, str):
-        m = self.attrname_re.match(str)
+    def getParams(self, s):
+        m = self.attrname_re.match(s)
         if m is None:
             return None
         return m.groupdict()
 
-    def getNames(self, str, factory=None):
+    def getNames(self, s, factory=None):
         """Returns the full, normal and simple names for this object, or None if there is no match'''
         """
-        raise RunTimeError('Not Allowed to call this method from subclasses')
+        raise RuntimeError('Not Allowed to call this method from subclasses')
     
-    def getDeviceName(self, str, full=True):
+    def getDeviceName(self, s, full=True):
         '''
         returns the device name for the given attribute name. 
         If full=True, the returned name includes the DB name
         '''
-        m = self.name_re.match(str)
+        m = self.name_re.match(s)
         if m is None:
             return None
         devname = m.group('devname') or EvaluationFactory.DEFAULT_DEVICE
         if full:
-            return '%s;dev=%s'%(self.getDBName(str),devname)
+            return '%s;dev=%s'%(self.getDBName(s),devname)
         else:
             return 'eval://dev=%s'%devname
     
-    def getDBName(self,str):
+    def getDBName(self,s):
         '''returns the full data base name for the given attribute name'''
-        m = self.name_re.match(str)
+        m = self.name_re.match(s)
         if m is None:
             return None
         dbname = m.group('dbname') or EvaluationFactory.DEFAULT_DATABASE
@@ -93,7 +104,7 @@ class AbstractEvaluationNameValidator(taurus.core.util.Singleton):
     
 
 class EvaluationAttributeNameValidator(AbstractEvaluationNameValidator):
- # The groups in a match object using the regexp below are:
+    # The groups in a match object using the regexp below are:
     #    1: scheme; named as 'scheme'
     #    2: 
     #    3: database name; optional; named as 'dbname'
@@ -113,8 +124,8 @@ class EvaluationAttributeNameValidator(AbstractEvaluationNameValidator):
     #                 1                             2   3                     4    5                      6                    7                    8                  9 A                                                        
     name_pattern = r'^(?P<scheme>eval|evaluation)://(db=(?P<dbname>[^?#;]+);)?(dev=(?P<devname>[^?#;]+);)?(?P<attrname>[^?#;]+)(\?(?!configuration=)(?P<subst>[^#?]*))?(#(?P<fragment>.*))?$'
         
-    def isValid(self,str, matchLevel = MatchLevel.ANY):
-        m = self.name_re.match(str)
+    def isValid(self,s, matchLevel = MatchLevel.ANY):
+        m = self.name_re.match(s)
         if m is None: 
             return False
         elif matchLevel == MatchLevel.COMPLETE:
@@ -122,7 +133,7 @@ class EvaluationAttributeNameValidator(AbstractEvaluationNameValidator):
         else:
             return True
 
-    def getNames(self, str, factory=None):
+    def getNames(self, s, factory=None):
         """Returns the complete, normal and short names.
         
         For example::
@@ -131,19 +142,19 @@ class EvaluationAttributeNameValidator(AbstractEvaluationNameValidator):
             >>> ("eval://db=_DefaultEvalDB;dev=foo;123*{a/b/c/d}", "eval://dev=foo;bar*blah", "bar*blah")
         
         """
-        m = self.name_re.match(str)
+        m = self.name_re.match(s)
         if m is None:
             return None
         #The following comments are for an example name like: "eval://dev=foo;bar*blah?bar=123;blah={a/b/c/d}#[1:-3]"
         attr_name = m.group('attrname') # attr_name = "bar*blah"
-        normal_name = "%s;%s"%(self.getDeviceName(str, full=False),attr_name) #normal_name = "eval://dev=foo;bar*blah"
-        expanded_attr_name = self.getExpandedTransformation(str)
-        fullname = "%s;%s"%(self.getDeviceName(str, full=True),expanded_attr_name) #fullname = "eval://db=_DefaultEvalDB;dev=foo;123*{a/b/c/d}"
+        normal_name = "%s;%s"%(self.getDeviceName(s, full=False),attr_name) #normal_name = "eval://dev=foo;bar*blah"
+        expanded_attr_name = self.getExpandedTransformation(s)
+        fullname = "%s;%s"%(self.getDeviceName(s, full=True),expanded_attr_name) #fullname = "eval://db=_DefaultEvalDB;dev=foo;123*{a/b/c/d}"
         return fullname, normal_name, attr_name
     
-    def getExpandedTransformation(self, str):
+    def getExpandedTransformation(self, s):
         'expands the attribute name by substituting all symbols'
-        m = self.name_re.match(str)
+        m = self.name_re.match(s)
         if m is None:
             return None
         transf = m.group('attrname')
@@ -167,23 +178,22 @@ class EvaluationDeviceNameValidator(AbstractEvaluationNameValidator):
     #                 1                             2   3                     4    5                    6                    7                                     
     name_pattern = r'^(?P<scheme>eval|evaluation)://(db=(?P<dbname>[^?#;]+);)?(dev=(?P<devname>[^?#;]+))(\?(?!configuration=)(?P<subst>[^#?]*))?$'
     
-    def getNames(self, str, factory=None):
+    def getNames(self, s, factory=None):
         """Returns the complete, normal and short names. (note: complete=normal)
         
-        :param str: (str) input string describing the device
+        :param s: (str) input string describing the device
         :param factory: (TaurusFactory) [Unused]
         
         :return: (tuple<str,str,str> or None) A tuple of complete, normal and
-                 short names, or None if str is an invalid device name
+                 short names, or None if s is an invalid device name
         """
-        m = self.name_re.match(str)
+        m = self.name_re.match(s)
         if m is None:
             return None
-        gdict = m.groupdict() 
         #The following comments are for a name of the type: eval://dev=foo?bar=123;blah={a/b/c/d} 
         devname = m.group('devname')  # foo
-        normal_name = self.getDeviceName(str, full=False) #eval://dev=foo
-        full_name = self.getDeviceName(str, full=True) #eval://db=_DefaultEvalDB;dev=foo
+        normal_name = self.getDeviceName(s, full=False) #eval://dev=foo
+        full_name = self.getDeviceName(s, full=True) #eval://db=_DefaultEvalDB;dev=foo
         return full_name, normal_name, devname
 
 
@@ -205,8 +215,8 @@ class EvaluationConfigurationNameValidator(AbstractEvaluationNameValidator):
     #                 1                             2   3                     4    5                      6                    7                    8                  9                 A                    
     name_pattern = r'^(?P<scheme>eval|evaluation)://(db=(?P<dbname>[^?#;]+);)?(dev=(?P<devname>[^?#;]+);)?(?P<attrname>[^?#;]+)(\?(?!configuration=)(?P<subst>[^#?]*))?(\?configuration=?(?P<cfgkey>[^#?]*))$'
         
-    def isValid(self,str, matchLevel = MatchLevel.ANY):
-        m = self.name_re.match(str)
+    def isValid(self,s, matchLevel = MatchLevel.ANY):
+        m = self.name_re.match(s)
         if m is None: 
             return False
         elif matchLevel == MatchLevel.COMPLETE:
@@ -214,27 +224,27 @@ class EvaluationConfigurationNameValidator(AbstractEvaluationNameValidator):
         else:
             return True
 
-    def getNames(self, str, factory=None):
+    def getNames(self, s, factory=None):
         """Returns the complete, normal and short names"""
-        m = self.name_re.match(str)
+        m = self.name_re.match(s)
         if m is None:
             return None
         #The following comments are for an example name like: "eval://dev=foo;bar*blah?bar=123;blah={a/b/c/d}?configuration=label"
         cfg_key = m.group('cfgkey') # cfg_key = "label"
         attr_name = m.group('attrname')
-        normal_name = "%s;%s?configuration"%(self.getDeviceName(str, full=False),attr_name) #normal_name = "eval://dev=foo;bar*blah?configuration"
-        expanded_attr_name = self.getExpandedTransformation(str)
-        fullname = "%s;%s?configuration"%(self.getDeviceName(str, full=True),expanded_attr_name) #fullname = "eval://db=_DefaultEvalDB;dev=foo;123*{a/b/c/d}?configuration"
+        normal_name = "%s;%s?configuration"%(self.getDeviceName(s, full=False),attr_name) #normal_name = "eval://dev=foo;bar*blah?configuration"
+        expanded_attr_name = self.getExpandedTransformation(s)
+        fullname = "%s;%s?configuration"%(self.getDeviceName(s, full=True),expanded_attr_name) #fullname = "eval://db=_DefaultEvalDB;dev=foo;123*{a/b/c/d}?configuration"
         return fullname, normal_name, cfg_key
     
-    def getAttrName(self, str):
-        names = self.getNames(str)
+    def getAttrName(self, s):
+        names = self.getNames(s)
         if names is None: return None
         return names[0][:-len('?configuration')] #remove the "?configuration" substring from the fullname
         
-    def getExpandedTransformation(self, str):
+    def getExpandedTransformation(self, s):
         'expands the attribute name by substituting all symbols'
-        m = self.name_re.match(str)
+        m = self.name_re.match(s)
         if m is None:
             return None
         transf = m.group('attrname')
@@ -243,7 +253,7 @@ class EvaluationConfigurationNameValidator(AbstractEvaluationNameValidator):
             transf = re.sub(k,v, transf)
         return transf
 
-class EvaluationDatabase(taurus.core.TaurusDatabase):
+class EvaluationDatabase(TaurusDatabase):
     '''
     Dummy database class for Evaluation (the Database concept is not used in the Evaluation scheme)
     
@@ -257,7 +267,7 @@ class EvaluationDatabase(taurus.core.TaurusDatabase):
         return "EvaluationDatabase object calling %s" % name
 
 
-class EvaluationDevice(taurus.core.TaurusDevice, SafeEvaluator):
+class EvaluationDevice(TaurusDevice, SafeEvaluator):
     '''
     The evaluator object. It is a :class:`TaurusDevice` and is used as the
     parent of :class:`EvaluationAttribute` objects for which it performs the
@@ -272,7 +282,7 @@ class EvaluationDevice(taurus.core.TaurusDevice, SafeEvaluator):
     
     def __init__(self, name, **kw):
         """Object initialization."""
-        self.call__init__(taurus.core.TaurusDevice, name, **kw)
+        self.call__init__(TaurusDevice, name, **kw)
         safedict = {}
         for s in self._symbols:
             if hasattr(self,s):
@@ -313,12 +323,12 @@ class EvaluationDevice(taurus.core.TaurusDevice, SafeEvaluator):
         else:
             self.info("Unexpected value to decode: %s" % str(event_value))
             new_sw_state = TaurusSWDevState.Crash
-        value = taurus.core.TaurusAttrValue() 
+        value = TaurusAttrValue() 
         value.value = new_sw_state
         return value
     
 
-class EvaluationAttribute(taurus.core.TaurusAttribute):
+class EvaluationAttribute(TaurusAttribute):
     '''
     A :class:`TaurusAttribute` that can be used to perform mathematical
     operations involving other arbitrary Taurus attributes. The mathematical
@@ -336,15 +346,15 @@ class EvaluationAttribute(taurus.core.TaurusAttribute):
     cref_RegExp = re.compile("\{(.+?)\}") #regexp for references to other taurus models within operation model names
 
     def __init__(self, name, parent, storeCallback = None):
-        self.call__init__(taurus.core.TaurusAttribute, name, parent, storeCallback=storeCallback)
+        self.call__init__(TaurusAttribute, name, parent, storeCallback=storeCallback)
         
-        self._value = taurus.core.TaurusAttrValue()
+        self._value = TaurusAttrValue()
         self._value.config.writable = False #Evaluation Attributes are always read-only (at least for now)
         self._references = [] 
         self._validator= self.getNameValidator()
         self._transformation = None
         # reference to the configuration object
-        self.__attr_config = None#taurus.core.TaurusConfiguration()
+        self.__attr_config = None#taurus.core.configuration.TaurusConfiguration()
         self.__subscription_state = SubscriptionState.Unsubscribed
 
         
@@ -476,11 +486,11 @@ class EvaluationAttribute(taurus.core.TaurusAttribute):
         try:
             evaluator = self.getParentObj() 
             self._value.value = evaluator.eval(self._transformation)
-            self._value.time = taurus.core.TaurusTimeVal.now()
-            self._value.quality = taurus.core.AttrQuality.ATTR_VALID
+            self._value.time = TaurusTimeVal.now()
+            self._value.quality = AttrQuality.ATTR_VALID
             self._value.config.data_format = len(numpy.shape(self._value.value))
         except Exception, e:
-            self._value.quality = taurus.core.AttrQuality.ATTR_INVALID
+            self._value.quality = AttrQuality.ATTR_INVALID
             self.warning("the function '%s' could not be evaluated. Reason: %s"%(self._transformation, repr(e)))
             #self.traceback(taurus.Warning)
             
@@ -567,7 +577,7 @@ class EvaluationAttribute(taurus.core.TaurusAttribute):
         
         initial_subscription_state = self.__subscription_state
         
-        ret = taurus.core.TaurusAttribute.addListener(self, listener)
+        ret = TaurusAttribute.addListener(self, listener)
 
         if not ret:
             return ret
@@ -587,7 +597,7 @@ class EvaluationAttribute(taurus.core.TaurusAttribute):
         """ Remove a TaurusListener from the listeners list. If polling enabled 
             and it is the last element then stop the polling timer.
             If the listener is not registered nothing happens."""
-        ret = taurus.core.TaurusAttribute.removeListener(self, listener)
+        ret = TaurusAttribute.removeListener(self, listener)
 
         cfg = self._getRealConfig()
         cfg.removeListener(listener)
@@ -598,7 +608,7 @@ class EvaluationAttribute(taurus.core.TaurusAttribute):
         return ret
     
 
-class EvaluationConfiguration(taurus.core.TaurusConfiguration):
+class EvaluationConfiguration(TaurusConfiguration):
     '''
     A :class:`TaurusConfiguration` 
     
@@ -608,7 +618,7 @@ class EvaluationConfiguration(taurus.core.TaurusConfiguration):
                  Instead it should be done via the :meth:`EvaluationFactory.getConfig`
     '''
     def __init__(self, name, parent, storeCallback = None):
-        self.call__init__(taurus.core.TaurusConfiguration, name, parent, storeCallback=storeCallback)
+        self.call__init__(TaurusConfiguration, name, parent, storeCallback=storeCallback)
         
         #fill the attr info
         i = parent.read().config
@@ -644,7 +654,7 @@ class EvaluationConfiguration(taurus.core.TaurusConfiguration):
         """ Returns the current configuration for the attribute."""
         return self._attr_info   
     
-class EvaluationFactory(taurus.core.util.Singleton, taurus.core.TaurusFactory, taurus.core.util.Logger):
+class EvaluationFactory(Singleton, TaurusFactory, Logger):
     """
     A Singleton class that provides Evaluation related objects.
     """
@@ -659,8 +669,8 @@ class EvaluationFactory(taurus.core.util.Singleton, taurus.core.TaurusFactory, t
     def init(self, *args, **kwargs):
         """Singleton instance initialization."""
         name = self.__class__.__name__
-        self.call__init__(taurus.core.util.Logger, name)
-        self.call__init__(taurus.core.TaurusFactory)
+        self.call__init__(Logger, name)
+        self.call__init__(TaurusFactory)
         self.eval_attrs = weakref.WeakValueDictionary()
         self.eval_devs = weakref.WeakValueDictionary()
         self.eval_configs = weakref.WeakValueDictionary()
@@ -753,15 +763,15 @@ class EvaluationFactory(taurus.core.util.Singleton, taurus.core.TaurusFactory, t
         return a
 
     def getConfiguration(self, param):
-        """getConfiguration(param) -> taurus.core.TaurusConfiguration
+        """getConfiguration(param) -> taurus.core.taurusconfiguration.TaurusConfiguration
 
         Obtain the object corresponding to the given attribute or full name.
         If the corresponding configuration already exists, the existing instance
         is returned. Otherwise a new instance is stored and returned.
 
-        @param[in] param taurus.core.TaurusAttribute object or full configuration name
+        @param[in] param taurus.core.taurusattribute.TaurusAttribute object or full configuration name
            
-        @return a taurus.core.TaurusAttribute object
+        @return a taurus.core.taurusattribute.TaurusAttribute object
         @throws TaurusException if the given name is invalid.
         """
         if isinstance(param, str):
@@ -784,7 +794,7 @@ class EvaluationFactory(taurus.core.util.Singleton, taurus.core.TaurusFactory, t
         
     def _getConfigurationFromAttribute(self, attr):
         cfg = attr.getConfig()
-        cfg_name = attrObj.getFullName() + "?configuration"
+        cfg_name = attr.getFullName() + "?configuration"
         self.eval_configs[cfg_name] = cfg
         return cfg
     
@@ -794,10 +804,10 @@ class EvaluationFactory(taurus.core.util.Singleton, taurus.core.TaurusFactory, t
         if exists is not None:
             if exists == dev: 
                 self.debug("%s has already been registered before" % name)
-                raise taurus.core.DoubleRegistration
+                raise DoubleRegistration
             else:
                 self.debug("%s has already been registered before with a different object!" % name)
-                raise taurus.core.DoubleRegistration
+                raise DoubleRegistration
         self.eval_devs[name] = dev
     
     def _storeAttr(self, attr):
@@ -806,10 +816,10 @@ class EvaluationFactory(taurus.core.util.Singleton, taurus.core.TaurusFactory, t
         if exists is not None:
             if exists == attr: 
                 self.debug("%s has already been registered before" % name)
-                raise taurus.core.DoubleRegistration
+                raise DoubleRegistration
             else:
                 self.debug("%s has already been registered before with a different object!" % name)
-                raise taurus.core.DoubleRegistration
+                raise DoubleRegistration
         self.eval_attrs[name] = attr
         
     def _storeConfig(self, fullname, config):
@@ -819,10 +829,10 @@ class EvaluationFactory(taurus.core.util.Singleton, taurus.core.TaurusFactory, t
         if exists is not None:
             if exists == config: 
                 self.debug("%s has already been registered before" % name)
-                raise taurus.core.DoubleRegistration
+                raise DoubleRegistration
             else:
                 self.debug("%s has already been registered before with a different object!" % name)
-                raise taurus.core.DoubleRegistration
+                raise DoubleRegistration
         self.eval_configs[name] = config
         
     def addAttributeToPolling(self, attribute, period, unsubscribe_evts = False):
@@ -833,7 +843,7 @@ class EvaluationFactory(taurus.core.util.Singleton, taurus.core.TaurusFactory, t
            :param period: (float) polling period (in seconds)
            :param unsubscribe_evts: (bool) whether or not to unsubscribe from events
         """
-        tmr = self.polling_timers.get(period,taurus.core.TaurusPollingTimer(period))
+        tmr = self.polling_timers.get(period, TaurusPollingTimer(period))
         self.polling_timers[period] = tmr
         tmr.addAttribute(attribute, self.isPollingEnabled())
         
@@ -876,7 +886,6 @@ def test1():
     
 
 def test2():
-    import taurus.core
     a=taurus.Attribute('eval://[{sys/tg_test/1/short_scalar},{sys/tg_test/1/double_scalar}, {sys/tg_test/1/short_scalar}+{sys/tg_test/1/double_scalar}]')
     #a=taurus.Attribute('eval://2*{sys/tg_test/1/short_scalar}+rand()')  
     class Dummy:
@@ -896,8 +905,8 @@ def test3():
     import sys
     from taurus.qt.qtgui.application import TaurusApplication
     from taurus.qt.qtgui.panel import TaurusForm
-    from taurus.qt.qtgui.plot import TaurusTrend
-    from taurus.qt.qtgui.display import TaurusLabel
+    #from taurus.qt.qtgui.plot import TaurusTrend
+    #from taurus.qt.qtgui.display import TaurusLabel
     app = TaurusApplication()
     
     w = TaurusForm()
