@@ -3,21 +3,21 @@
 #############################################################################
 ##
 ## This file is part of Taurus, a Tango User Interface Library
-## 
+##
 ## http://www.tango-controls.org/static/taurus/latest/doc/html/index.html
 ##
 ## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
-## 
+##
 ## Taurus is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU Lesser General Public License as published by
 ## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
-## 
+##
 ## Taurus is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU Lesser General Public License for more details.
-## 
+##
 ## You should have received a copy of the GNU Lesser General Public License
 ## along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
 ##
@@ -39,21 +39,21 @@ __all__ = ["BaseSardanaElement", "BaseSardanaElementContainer",
 
 __docformat__ = 'restructuredtext'
 
-import socket
-
-import time
 import PyTango
 
-from taurus.core.util import Enumeration, Singleton, Logger, CaselessDict, \
-    CodecFactory
+from taurus.core.util.enumeration import Enumeration
+from taurus.core.util.log import Logger
+from taurus.core.util.singleton import Singleton
+from taurus.core.util.containers import CaselessDict
+from taurus.core.util.codecs import CodecFactory
 
 PoolElementType = Enumeration("PoolElementType",
-    ("0D", "1D", "2D", "Communication", "CounterTimer", "IORegister", 
+    ("0D", "1D", "2D", "Communication", "CounterTimer", "IORegister",
       "Motor","PseudoCounter", "PseudoMotor"))
 
-ChannelView = Enumeration("ChannelView", 
-    ("Channel", "Enabled", "Output", "PlotType", "PlotAxes", "Timer", 
-     "Monitor", "Trigger", "Conditioning", "Normalization","NXPath", 
+ChannelView = Enumeration("ChannelView",
+    ("Channel", "Enabled", "Output", "PlotType", "PlotAxes", "Timer",
+     "Monitor", "Trigger", "Conditioning", "Normalization","NXPath",
      "Shape", "DataType",
      "Unknown"))
 
@@ -64,7 +64,7 @@ Normalization = Enumeration("Normalization", ("No", "Avg", "Integ"))
 #: an enumeration describing all possible acquisition trigger types
 AcqTriggerType = Enumeration("AcqTriggerType", ( \
     "Software", # channel triggered by software - start and stop by software
-    "Gate",     # channel triggered by HW - start and stop by external 
+    "Gate",     # channel triggered by HW - start and stop by external
     "Unknown") )
 
 #: an enumeration describing all possible acquisition mode types
@@ -76,113 +76,113 @@ AcqMode = Enumeration("AcqMode", ( \
 
 class BaseSardanaElement(object):
     """Generic sardana element"""
-    
+
     def __init__(self, *args, **kwargs):
         self._manager = kwargs.pop('manager')
         self.__dict__.update(kwargs)
         self._data = kwargs
         self._object = None
-    
+
     def __repr__(self):
         return "{0}({1})".format(self.type, self.full_name)
-    
+
     def __str__(self):
         return self.name
-    
+
     def __getattr__(self, name):
         return getattr(self.getObj(), name)
-    
+
     def __cmp__(self, elem):
         return cmp(self.name, elem.name)
-    
+
     def getData(self):
         return self._data
-    
+
     def getName(self):
         return self.name
-    
+
     def getId(self):
         return self.full_name
-    
+
     def getType(self):
         return self.getTypes()[0]
-    
+
     def getTypes(self):
         elem_types = self.type
         if isinstance(elem_types, (str, unicode)):
             return [elem_types]
         return elem_types
-    
+
     def serialize(self, *args, **kwargs):
         kwargs.update(self._data)
         return kwargs
-    
+
     def str(self, *args, **kwargs):
         #TODO change and check which is the active protocol to serialize
         #acordingly
         return CodecFactory().encode(('json', self.serialize(*args, **kwargs)))
-    
+
     def getObj(self):
         obj = self._object
         if obj is None:
             self._object = obj = self._manager.getObject(self)
         return obj
-    
+
 
 
 class BaseSardanaElementContainer:
-    
+
     def __init__(self):
         # dict<str, dict> where key is the type and value is:
-        #     dict<str, MacroServerElement> where key is the element alias and
-        #                                   value is the Element object
+        #     dict<str, MacroServerElement> where key is the element full name
+        #                                   and value is the Element object
         self._type_elems_dict = CaselessDict()
-        
+
         # dict<str, container> where key is the interface and value is the set
         # of elements which implement that interface
         self._interfaces_dict = {}
-    
+
     def addElement(self, elem):
         elem_type = elem.getType()
-        elem_name = elem.getName()
-        
+        elem_full_name = elem.full_name
+
         #update type_elems
         type_elems = self._type_elems_dict.get(elem_type)
         if type_elems is None:
             self._type_elems_dict[elem_type] = type_elems = CaselessDict()
-        type_elems[elem_name] = elem
-        
+        type_elems[elem_full_name] = elem
+
         # update interfaces
         for interface in elem.interfaces:
             interface_elems = self._interfaces_dict.get(interface)
             if interface_elems is None:
                 self._interfaces_dict[interface] = interface_elems = CaselessDict()
-            interface_elems[elem_name] = elem
-    
+            interface_elems[elem_full_name] = elem
+
     def removeElement(self, e):
-        type = e.getType()
-        
+        elem_type = e.getType()
+
         # update type_elems
-        type_elems = self._type_elems_dict.get(type)
+        type_elems = self._type_elems_dict.get(elem_type)
         if type_elems:
-            del type_elems[e.name]
-        
+            del type_elems[e.full_name]
+
         # update interfaces
         for interface in e.interfaces:
             interface_elems = self._interfaces_dict.get(interface)
-            del interface_elems[e.name]
-    
+            del interface_elems[e.full_name]
+
     def removeElementsOfType(self, t):
         for elem in self.getElementsOfType(t):
             self.removeElement(elem)
-    
+
     def getElementsOfType(self, t):
         elems = self._type_elems_dict.get(t, {})
         return elems
-    
+
     def getElementNamesOfType(self, t):
-        return [ e.name for e in self.getElementsOfType(t).values() ]
-    
+        return [e.name for e in self.getElementsOfType(t).values()]
+
     def getElementsWithInterface(self, interface):
         elems = self._interfaces_dict.get(interface, {})
         return elems
@@ -194,32 +194,42 @@ class BaseSardanaElementContainer:
         return ret
 
     def getElementNamesWithInterface(self, interface):
-        return [ e.name for e in self.getElementsWithInterface(interface).values() ]
-    
+        return [e.name for e in self.getElementsWithInterface(interface).values()]
+
     def hasElementName(self, elem_name):
         return self.getElement(elem_name) != None
-    
+
     def getElement(self, elem_name):
+        elem_name = elem_name.lower()
         for elems in self._type_elems_dict.values():
-            elem = elems.get(elem_name)
+            elem = elems.get(elem_name) # full_name?
             if elem is not None:
                 return elem
-    
+            for elem in elems.values():
+                if elem.name.lower() == elem_name:
+                    return elem
+
     def getElementWithInterface(self, elem_name, interface):
-        return self._interfaces_dict.get(interface, {}).get(elem_name)
-    
+        elem_name = elem_name.lower()
+        elems = self._interfaces_dict.get(interface, {})
+        if elem_name in elems:
+            return elems[elem_name]
+        for elem in elems.values():
+            if elem.name.lower() == elem_name:
+                return elem
+
     def getElements(self):
         ret = set()
         for elems in self._type_elems_dict.values():
             ret.update(elems.values())
         return ret
-        
+
     def getInterfaces(self):
         return self._interfaces_dict
-    
+
     def getTypes(self):
         return self._type_elems_dict
-    
+
 #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 # T E M P O R A R Y   I M P L E M E N T A T I O N
 #
@@ -231,19 +241,19 @@ class PropertyInfo():
         self._type = type
         self._format = format
         self._default_value=default_value
-        
+
     def get_name(self):
         return self._name
 
     def get_type(self):
         return self._type
-    
+
     def get_format(self):
         return self._format
-    
+
     def get_default_value(self):
         return self._default_value
-    
+
 
 #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 # T E M P O R A R Y   I M P L E M E N T A T I O N
@@ -251,32 +261,32 @@ class PropertyInfo():
 # THIS IS USED FOR TEST PURPOSES ONLY. DO NOT USE IT OUTSIDE SARDANA TESTS
 #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 class ControllerClassInfo(object):
-    
+
     def __init__(self, name, type, library):
         self._name = name
         self._type = type
         self._library = library
-        
+
     def get_max_elements(self):
         return 20
-    
+
     def get_name(self):
         return self._name
-    
+
     def get_model(self):
         # fake data ###############
-        return "Model of "+ self._name 
-    
+        return "Model of "+ self._name
+
     def get_icon(self):
         # fake data ###############
         import taurus.qt.qtgui.resource
-        
+
         return taurus.qt.qtgui.resource.getIcon(":/designer/extra_motor.png")
-    
+
     def get_organization(self):
         # fake data ###############
-        return "Organization of "+ self._name 
-    
+        return "Organization of "+ self._name
+
     def get_description(self):
         #fake data############
         descr="This is description of "
@@ -284,19 +294,19 @@ class ControllerClassInfo(object):
             descr=descr + " and " +self._name
         ####################
         return descr
-    
+
     def get_family(self):
         # fake data ###############
-        return "Family of "+ self._name 
-    
+        return "Family of "+ self._name
+
     def get_properties(self):
         properties = []
         # fake data ######################
         properties.append(PropertyInfo("my parameter", "string", "0D", "deviceName"))
         properties.append(PropertyInfo("asdsadasd", "integer", "0D", 5))
         properties.append(PropertyInfo("boollll0", "boolean", "0D", False))
-        properties.append(PropertyInfo("boollll0", "boolean", "0D", True)) 
-        properties.append(PropertyInfo("boollll0", "boolean", "0D", False))     
+        properties.append(PropertyInfo("boollll0", "boolean", "0D", True))
+        properties.append(PropertyInfo("boollll0", "boolean", "0D", False))
         properties.append(PropertyInfo("number1", "float", "0D", 3.5))
         properties.append(PropertyInfo("string2", "string", "0D", "hehe"))
         properties.append(PropertyInfo("tableIntegerD1", "integer", "1D", [1,2,3]))
@@ -308,9 +318,9 @@ class ControllerClassInfo(object):
         properties.append(PropertyInfo("tableinteger2", "integer", "2D",[ [1,2,3],[11,22,33],[-10,-20,-30] ]))
         properties.append(PropertyInfo("tablefloatD2", "float", "2D",[ [0.5,0.6,0.8],[0.4,0.0,0.333333],[-0.1111,1,123123.6] ]))
         properties.append(PropertyInfo("tablestringD2", "string", "2D",[ ["aaaa","bbb","ccc"],["aaaa2","bbb2","ccc2"],["aaaa3","bbb3","ccc3"] ]))
-    
+
         return properties
-    
+
     def get_controller_type(self):
         return self._type
 
@@ -321,7 +331,7 @@ class ControllerClassInfo(object):
 # THIS IS USED FOR TEST PURPOSES ONLY. DO NOT USE IT OUTSIDE SARDANA TESTS
 #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 class ControllerInfo(object):
-    
+
     def __init__(self, name, ctrl_class_info):
         self._name = name
         self._ctrl_class_info = ctrl_class_info
@@ -331,13 +341,13 @@ class ControllerInfo(object):
 
     def get_controller_type(self):
         return self._ctrl_class_info.get_controller_type()
-    
+
     def get_name(self):
         return self._name
-    
+
     def get_max_elements(self):
         return self._ctrl_class_info.get_max_elements()
-    
+
     def is_axis_free(self, axis):
         #fake data
         if axis == 3:
@@ -354,8 +364,6 @@ class ControllerInfo(object):
 
     def get_icon(self):
         return self._ctrl_class_info.get_icon()
-    
-        return properties
 
 #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 # T E M P O R A R Y   I M P L E M E N T A T I O N
@@ -363,7 +371,7 @@ class ControllerInfo(object):
 # THIS IS USED FOR TEST PURPOSES ONLY. DO NOT USE IT OUTSIDE SARDANA TESTS
 #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 class Pool(object):
-    
+
     def __init__(self, sardana, name, poolpath, version, alias=None, device_name=None):
         self._sardana = sardana
         self._name = name
@@ -371,16 +379,16 @@ class Pool(object):
         self._version = version
         self._alias = alias
         self._device_name = device_name
-        
+
     def starter_run(self, host, level=1):
         return True
-    
+
     def get_name(self):
         return self._name
-    
+
     def local_run(self):
         return True
-    
+
     def get_element_types(self):
         return sorted(PoolElementType.keys())
 
@@ -391,7 +399,7 @@ class Pool(object):
             data.append(ControllerClassInfo("motorController"+str(i), PoolElementType.Motor, None))
         for i in range(5):
             data.append(ControllerClassInfo("counterTimerController"+str(i), PoolElementType.CounterTimer, None))
-            
+
         return data
 
     def get_controller_infos(self):
@@ -402,10 +410,10 @@ class Pool(object):
         for i in range(2):
             data.append(ControllerInfo("My_ct_ctrl_"+str(i), ctrl_classes[i+5]))
         return data
- 
+
     def create_controller(self,controller_class_info, name, properties ):
         pass
-    
+
     def create_element(self, controller_name, name, axis):
         pass
 
@@ -416,7 +424,7 @@ class Pool(object):
 # THIS IS USED FOR TEST PURPOSES ONLY. DO NOT USE IT OUTSIDE SARDANA TESTS
 #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 class MacroServer(object):
-    
+
     def __init__(self, sardana, name, macropath, pool_names, version, alias=None, device_name=None):
         self._sardana = sardana
         self._name = name
@@ -426,7 +434,7 @@ class MacroServer(object):
         self._alias = alias
         self._device_name = device_name
         self._doors = []
-    
+
     def create_door(self, alias, device_name):
         try:
             return self._create_door(alias, device_name)
@@ -434,7 +442,7 @@ class MacroServer(object):
             db = self.get_database()
             db.delete_device(device_name)
             raise
-        
+
     def _create_door(self, alias, device_name):
         db = self.get_database()
         info = PyTango.DbDevInfo()
@@ -447,16 +455,16 @@ class MacroServer(object):
         door = Door(alias=alias, device_name=device_name)
         self._doors.append(door)
         return door
-    
+
     def remove_door(self, device_name):
         pass
-    
+
     def starter_run(self, host, level=1):
         return True
-        
+
     def local_run(self):
         return True
-    
+
     def get_database(self):
         return self._sardana.get_database()
 
@@ -466,7 +474,7 @@ class MacroServer(object):
 # THIS IS USED FOR TEST PURPOSES ONLY. DO NOT USE IT OUTSIDE SARDANA TESTS
 #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 class Door(object):
-    
+
     def __init__(self, alias=None, device_name=None):
         self._name = alias
         self._device_name = device_name
@@ -478,7 +486,7 @@ class Door(object):
 # THIS IS USED FOR TEST PURPOSES ONLY. DO NOT USE IT OUTSIDE SARDANA TESTS
 #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 class Sardana(object):
-    
+
     def __init__(self, sardana_db , name, device_name=None):
         self._sardana_db = sardana_db
         self._name = name
@@ -486,7 +494,7 @@ class Sardana(object):
         self._pools = []
         self._macroservers = []
         self._init()
-        
+
     def _init(self):
         if not self._device_name:
             return
@@ -516,23 +524,23 @@ class Sardana(object):
                 pool_alias = pool_dev_info.alias()
                 pool = Pool(self, pool_name, pool_props.get("poolpath"), pool_props.get("version"), pool_alias, pool_dev_name)
                 self._pools.append(pool)
-    
+
     def get_name(self):
         return self._name
-    
+
     def set_device_name(self, device_name):
         self._device_name = device_name
         self._init()
-    
+
     def get_device_name(self):
         return self._device_name
-        
+
     def get_pools(self):
         return self._pools
-    
+
     def get_macro_servers(self):
         return self._macro_servers
-        
+
     def create_pool(self, name, poolpath, version, alias=None, device_name=None):
         try:
             return self._create_pool(name, poolpath, version, alias=alias, device_name=device_name)
@@ -540,7 +548,7 @@ class Sardana(object):
             db = self.get_database()
             db.delete_device(device_name)
             raise
-        
+
     def _create_pool(self, name, poolpath, version, alias=None, device_name=None):
         db = self.get_database()
         info = PyTango.DbDevInfo()
@@ -550,13 +558,13 @@ class Sardana(object):
         db.add_device(info)
         if alias:
             db.put_device_alias(device_name, alias)
-            
+
         db.put_device_property(device_name,{"PoolPath" : poolpath, "Version": version} )
         pool = Pool(self, name, poolpath, version, alias=alias, device_name=device_name)
         self._pools.append(pool)
         db.cache().refresh()
         return pool
-    
+
     def create_macroserver(self, name, macropath, pool_names, version, alias=None, device_name=None):
         try:
             return self._create_macroserver(name, macropath, pool_names, version, alias=alias, device_name=device_name)
@@ -564,7 +572,7 @@ class Sardana(object):
             db = self.get_database()
             db.delete_device(device_name)
             raise
-     
+
     def _create_macroserver(self, name, macropath, pool_names, version, alias=None, device_name=None):
         db = self.get_database()
         info = PyTango.DbDevInfo()
@@ -574,16 +582,16 @@ class Sardana(object):
         db.add_device(info)
         if alias:
             db.put_device_alias(device_name, alias)
-            
+
         db.put_device_property(device_name,{"MacroPath" : macropath, "Version": version, "PoolNames":pool_names} )
         ms = MacroServer(self, name, macropath, pool_names, version, alias=alias, device_name=device_name)
         self._macroservers.append(ms)
         db.cache().refresh()
         return ms
-        
+
     def remove_pool(self):
         pass
-        
+
     def remove_macroserver(self):
         pass
 
@@ -597,12 +605,12 @@ class Sardana(object):
 #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 class DatabaseSardana(object):
     """A class containning all sardanas for a single database"""
-    
+
     def __init__(self, db):
         assert(db is not None)
         self._db = db
         self.refresh()
-        
+
     def refresh(self):
         self._sardanas = sardanas = {}
         services = self._db.get_service_list("Sardana/.*")
@@ -612,7 +620,7 @@ class DatabaseSardana(object):
                 sardanas[service_instance] = Sardana(self, service_instance, dev)
             except:
                 pass
-    
+
     def create_sardana(self, name, device_name):
         if self._sardanas.has_key(name):
             raise Exception("Sardana '%s' already exists" % name)
@@ -620,20 +628,20 @@ class DatabaseSardana(object):
         sardana = Sardana(self, name)
         self._sardanas[name] = sardana
         return sardana
-        
+
     def remove_sardana(self, name):
         try:
-            sardana = self._sardanas.pop(name)
+            self._sardanas.pop(name)
         except KeyError:
             raise Exception("Sardana '%s' does NOT exist" % name)
         self._db.unregister_service("Sardana", name)
 
     def get_sardanas(self):
         return self._sardanas
-    
+
     def get_sardana(self, name):
         return self._sardanas[name]
-    
+
     def get_database(self):
         return self._db
 
@@ -643,7 +651,7 @@ class DatabaseSardana(object):
 # THIS IS USED FOR TEST PURPOSES ONLY. DO NOT USE IT OUTSIDE SARDANA TESTS
 #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 class SardanaManager(Singleton, Logger):
-    
+
     def __init__(self):
         """ Initialization. Nothing to be done here for now."""
         pass
@@ -654,9 +662,10 @@ class SardanaManager(Singleton, Logger):
         name = self.__class__.__name__
         self.call__init__(Logger, name)
         self._db_sardanas = {}
-        
+
     def _get_db_sardana(self, db=None):
         if db is None:
+            import taurus
             db = taurus.Database()
         db_sardana = self._db_sardanas.get(db)
         if db_sardana is None:
@@ -668,36 +677,37 @@ class SardanaManager(Singleton, Logger):
 
     def remove_sardana(self, name, db=None):
         self._get_db_sardana(db).remove_sardana(name)
-        
+
     def get_sardanas(self, db=None):
         return self._get_db_sardana(db).get_sardanas()
-        
+
     def get_sardana(self, name, db=None):
         return self._get_db_sardana(db).get_sardana(name)
-    
+
     def get_hosts(self):
         return ["localhost"] + ["controls%02d" % i for i in range(5)]
-    
+
     def get_level_range(self):
         return 1, 200
-    
+
     def has_localhost_starter(self):
+        import socket
         return socket.gethostname() in self.get_hosts()
-    
+
     @classmethod
     def get_default_pool_path(cls):
         pathList = []
         pathList.append("/homelocal/sicilia/lib/poolcontrollers")
         pathList.append("/homelocal/sicilia/lib/python/site-packages/poolcontrollers")
         return pathList
-    
+
     @classmethod
     def get_default_ms_path(cls):
         pathList = []
         pathList.append("/homelocal/sicilia/lib/python/site-packages/macroserver/macros")
         return pathList
-    
-    
-    
-        
-    
\ No newline at end of file
+
+
+
+
+
