@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 
 #############################################################################
 ##
@@ -29,13 +30,9 @@ __all__ = ["TaurusConfigurationProxy", "TaurusConfiguration"]
 
 __docformat__ = "restructuredtext"
 
-# -*- coding: utf-8 -*-
-import time
-import weakref
-import numpy
+from .taurusbasetypes import AttrAccess, TaurusElementType
+from .taurusmodel import TaurusModel
 
-from enums import TaurusEventType, AttrAccess
-import taurusmodel
 
 class TaurusConfigurationProxy(object):
     """
@@ -54,7 +51,7 @@ class TaurusConfigurationProxy(object):
         return getattr(self.__parent._getRealConfig(), name)
 
 
-class TaurusConfiguration(taurusmodel.TaurusModel):
+class TaurusConfiguration(TaurusModel):
     
     no_cfg_value = '-----'
     no_unit = 'No unit'
@@ -83,7 +80,7 @@ class TaurusConfiguration(taurusmodel.TaurusModel):
         # the configuration event identifier
         self._cfg_evt_id = None
 
-        self.call__init__(taurusmodel.TaurusModel, name, parent)
+        self.call__init__(TaurusModel, name, parent)
         
         # Everything went ok so now we are sure we can store the object
         if not storeCallback is None:
@@ -92,6 +89,9 @@ class TaurusConfiguration(taurusmodel.TaurusModel):
         self._dev_hw_obj = self._getDev().getHWObj()
             
         self._subscribeEvents()
+
+    def __str__(self):
+        return self.getFullName()
         
     def _subscribeEvents(self):
         pass
@@ -101,15 +101,14 @@ class TaurusConfiguration(taurusmodel.TaurusModel):
         self._unsubscribeEvents()
         self._attr_info = None
         self._dev_hw_obj = None
-        taurusmodel.TaurusModel.cleanUp(self)        
+        TaurusModel.cleanUp(self)        
         
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
     # TaurusModel implementation
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-    
 
     def getTaurusElementType(self):
-        import taurus.core
-        return taurus.core.TaurusElementType.Configuration
+        return TaurusElementType.Configuration
 
     @classmethod
     def buildModelName(cls, parent_model, relative_name):
@@ -172,7 +171,7 @@ class TaurusConfiguration(taurusmodel.TaurusModel):
             if cache is set to True (default) and the the configuration has 
             events active then it will return the local cached value. Otherwise
             it will read from the tango layer."""
-        raise RuntimeException("May not be called in abstract TaurusConfiguration")
+        raise RuntimeError("May not be called in abstract TaurusConfiguration")
     
     def getDisplayValue(self,cache=True):
         confvalue = self.getValueObj(cache=cache)
@@ -189,8 +188,8 @@ class TaurusConfiguration(taurusmodel.TaurusModel):
             return [('name', self.getLabel(cache=cache))]
         return attrObj.getDisplayDescrObj(cache=cache)
     
-    def isWritable(self):
-        return True
+#    def isWritable(self):
+#        return True
     
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
     # API for listeners
@@ -199,7 +198,7 @@ class TaurusConfiguration(taurusmodel.TaurusModel):
 #    def addListener(self, listener):
 #        """ Add a TaurusListener object in the listeners list.
 #            If the listener is already registered nothing happens."""
-#        ret = taurusmodel.TaurusModel.addListener(self, listener)
+#        ret = TaurusModel.addListener(self, listener)
 #        if not ret:
 #            return ret
         
@@ -393,7 +392,7 @@ class TaurusConfiguration(taurusmodel.TaurusModel):
     def setDescription(self,descr):
         config = self.getValueObj()
         if config:
-            config.description = description
+            config.description = descr
             self._applyConfig()
 
     def setLabel(self,lbl):
@@ -463,8 +462,23 @@ class TaurusConfiguration(taurusmodel.TaurusModel):
         attr = self.getParentObj()
         if attr is None: return False
         v = attr.read(cache=cache)
+        import numpy
         return numpy.isscalar(v.value)
     
+    def isSpectrum(self, cache=True):
+        attr = self.getParentObj()
+        if attr is None: return False
+        v = attr.read(cache=cache)
+        import numpy
+        return not numpy.isscalar(v.value) and numpy.array(v.value).ndim == 1
+    
+    def isImage(self, cache=True):
+        attr = self.getParentObj()
+        if attr is None: return False
+        v = attr.read(cache=cache)
+        import numpy
+        return not numpy.isscalar(v.value) and numpy.array(v.value).ndim == 2
+    
     def isWrite(self, cache=True):
         return self.getWritable(cache) == AttrAccess.WRITE
     
@@ -473,3 +487,6 @@ class TaurusConfiguration(taurusmodel.TaurusModel):
 
     def isReadWrite(self, cache=True):
         return self.getWritable(cache) == AttrAccess.READ_WRITE
+
+#del AttrAccess
+#del TaurusModel
