@@ -38,7 +38,7 @@ sequence of codecs. This way you can have codecs 'inside' codecs.
 
 Example::
 
-    >>> from taurus.core.util import CodecFactory
+    >>> from taurus.core.util.codecs import CodecFactory
     >>> cf = CodecFactory()
     >>> json_codec = cf.getCodec('json')
     >>> bz2_json_codec = cf.getCodec('bz2_json')
@@ -55,7 +55,7 @@ A Taurus related example::
 
     >>> # this example shows how to automatically get the data from a DEV_ENCODED attribute
     >>> import taurus
-    >>> from taurus.core.util import CodecFactory
+    >>> from taurus.core.util.codecs import CodecFactory
     >>> cf = CodecFactory()
     >>> devenc_attr = taurus.Attribute('a/b/c/devenc_attr')
     >>> v = devenc_attr.read()
@@ -72,6 +72,10 @@ import copy
 import operator
 import types
 
+#need by VideoImageCodec
+import struct
+import numpy
+
 from singleton import Singleton
 from log import Logger, DebugIt
 from containers import CaselessDict
@@ -137,7 +141,7 @@ class ZIPCodec(Codec):
     
     Example::
     
-        >>> from taurus.core.util import CodecFactory
+        >>> from taurus.core.util.codecs import CodecFactory
         
         >>> # first encode something
         >>> data = 100 * "Hello world\\n"
@@ -179,7 +183,7 @@ class BZ2Codec(Codec):
 
     Example::
     
-        >>> from taurus.core.util import CodecFactory
+        >>> from taurus.core.util.codecs import CodecFactory
         
         >>> # first encode something
         >>> data = 100 * "Hello world\\n"
@@ -222,7 +226,7 @@ class PickleCodec(Codec):
     
     Example::
         
-        >>> from taurus.core.util import CodecFactory
+        >>> from taurus.core.util.codecs import CodecFactory
         
         >>> cf = CodecFactory()
         >>> codec = cf.getCodec('pickle')
@@ -277,7 +281,7 @@ class JSONCodec(Codec):
     
     Example::
         
-        >>> from taurus.core.util import CodecFactory
+        >>> from taurus.core.util.codecs import CodecFactory
         
         >>> cf = CodecFactory()
         >>> codec = cf.getCodec('json')
@@ -360,7 +364,7 @@ class BSONCodec(Codec):
     
     Example::
         
-        >>> from taurus.core.util import CodecFactory
+        >>> from taurus.core.util.codecs import CodecFactory
         
         >>> cf = CodecFactory()
         >>> codec = cf.getCodec('bson')
@@ -437,7 +441,9 @@ class FunctionCodec(Codec):
         self._func_name = func_name
     
     def encode(self, data, *args, **kwargs):
-        return self._func_name, { 'type' : self._func_name, 'data' : data }
+        format = self._func_name
+        if len(data[0]): format += '_%s' % data[0]
+        return format, { 'type' : self._func_name, 'data' : data[1] }
     
     def decode(self, data, *args, **kwargs):
         if not data[0].startswith(self._func_name):
@@ -452,12 +458,170 @@ class PlotCodec(FunctionCodec):
         FunctionCodec.__init__(self, 'plot')
 
 
+class VideoImageCodec(Codec):
+    """A codec able to encode/decode to/from LImA video_image format.
+    
+    Example::
+    
+        >>> from taurus.core.util.codecs import CodecFactory
+        >>> import PyTango
+        
+        >>> #first get an image from a LImA device to decode
+        >>> data = PyTango.DeviceProxy(ccdName).read_attribute('video_last_image').value
+        >>> cf = CodecFactory()
+        >>> codec = cf.getCodec('VIDEO_IMAGE')
+        >>> format,decoded_data = codec.decode(data)
+        >>> # encode it again to check
+        >>> format, encoded_data = codec.encode(("",decoded_data))
+        >>> #compare images excluding the header:
+        >>> data[1][32:] == encoded_data[32:]
+        >>> #The headers can be different in the frameNumber
+        >>> struct.unpack('!IHHqiiHHHH',data[1][:32])
+        (1447314767, 1, 0, 6868, 1294, 964, 0, 32, 0, 0)
+        >>> struct.unpack('!IHHqiiHHHH',encoded_data[:32])
+        (1447314767, 1, 0, -1, 1294, 964, 0, 32, 0, 0)
+    """
+    
+    VIDEO_HEADER_FORMAT = '!IHHqiiHHHH'
+    
+    def encode(self, data, *args, **kwargs):
+        """encodes the given data to a LImA's video_image. The given data **must** be an numpy.array
+        
+        :param data: (sequence[str, obj]) a sequence of two elements where the first item is the encoding format of the second item object
+        
+        :return: (sequence[str, obj]) a sequence of two elements where the first item is the encoding format of the second item object"""
+
+        format = 'VIDEO_IMAGE'
+        if len(data[0]): format += '_%s' % data[0]
+        #imgMode depends on numpy.array dtype
+        imgMode = self.__getModeId(str(data[1].dtype))
+        #frameNumber, unknown then -1
+        height,width = data[1].shape
+        header = self.__packHeader(imgMode,-1,width,height)
+        img2D = data[1]
+        img1D = img2D.flatten()
+        buffer = struct.pack(self.__getFormatId(imgMode)*img1D.size,*img1D)
+        return format,header+buffer
+    
+    def decode(self, data, *args, **kwargs):
+        """decodes the given data from a LImA's video_image.
+            
+        :param data: (sequence[str, obj]) a sequence of two elements where the first item is the encoding format of the second item object
+        
+        :return: (sequence[str, obj]) a sequence of two elements where the first item is the encoding format of the second item object"""
+
+        if not data[0] == 'VIDEO_IMAGE':
+            return data
+        header = self.__unpackHeader(data[1][:struct.calcsize(self.VIDEO_HEADER_FORMAT)])
+        
+        imgBuffer = data[1][struct.calcsize(self.VIDEO_HEADER_FORMAT):]
+        fmt = self.__getFormatId(header['imageMode'])
+        img1D = numpy.array(struct.unpack(fmt*(len(imgBuffer)/struct.calcsize(fmt)),
+                                          imgBuffer),
+                            dtype=self.__getDtypeId(header['imageMode']))
+        img2D = img1D.reshape(header['height'],header['width'])
+        return '',img2D
+
+    def __unpackHeader(self,header):
+        h = struct.unpack(self.VIDEO_HEADER_FORMAT,header)
+        headerDict={}
+        headerDict['magic']         = h[0]
+        headerDict['headerVersion'] = h[1]
+        headerDict['imageMode']     = h[2]
+        headerDict['frameNumber']   = h[3]
+        headerDict['width']         = h[4]
+        headerDict['height']        = h[5]
+        headerDict['endianness']    = h[6]
+        headerDict['headerSize']    = h[7]
+        headerDict['padding']       = h[7:]
+        return headerDict
+    
+    def __packHeader(self,imgMode,frameNumber,width,height):
+        magic = 0x5644454f
+        version = 1
+        endian = ord(struct.pack('=H',1)[-1])
+        hsize = struct.calcsize(self.VIDEO_HEADER_FORMAT)
+        return struct.pack(self.VIDEO_HEADER_FORMAT,
+                           magic,
+                           version,
+                           imgMode,
+                           frameNumber,
+                           width,
+                           height,
+                           endian,
+                           hsize,
+                           0,0)#padding
+
+    def __getModeId(self,mode):
+        return {#when encode
+                'uint8'      : 0,#Core.Y8,
+                'uint16'     : 1,#Core.Y16,
+                'uint32'     : 2,#Core.Y32,
+                'uint64'     : 3,#Core.Y64,
+                #when decode
+                'Y8'         : 0,#Core.Y8,
+                'Y16'        : 1,#Core.Y16,
+                'Y32'        : 2,#Core.Y32,
+                'Y64'        : 3,#Core.Y64,
+                #TODO: other modes
+                #'RGB555'     : Core.RGB555,
+                #'RGB565'     : Core.RGB565,
+                #'RGB24'      : Core.RGB24,
+                #'RGB32'      : Core.RGB32,
+                #'BGR24'      : Core.BGR24,
+                #'BGR32'      : Core.BGR32,
+                #'BAYER RG8'  : Core.BAYER_RG8,
+                #'BAYER RG16' : Core.BAYER_RG16,
+                #'I420'       : Core.I420,
+                #'YUV411'     : Core.YUV411,
+                #'YUV422'     : Core.YUV422,
+                #'YUV444'     : Core.YUV444
+               }[mode]
+
+    def __getFormatId(self,mode):
+        return {0      : 'B',
+                1      : 'H',
+                2      : 'I',
+                3      : 'L',
+                #'RGB555'     : Core.RGB555,
+                #'RGB565'     : Core.RGB565,
+                #'RGB24'      : Core.RGB24,
+                #'RGB32'      : Core.RGB32,
+                #'BGR24'      : Core.BGR24,
+                #'BGR32'      : Core.BGR32,
+                #'BAYER RG8'  : Core.BAYER_RG8,
+                #'BAYER RG16' : Core.BAYER_RG16,
+                #'I420'       : Core.I420,
+                #'YUV411'     : Core.YUV411,
+                #'YUV422'     : Core.YUV422,
+                #'YUV444'     : Core.YUV444
+               }[mode]
+
+    def __getDtypeId(self,mode):
+        return {0       : 'uint8',
+                1      : 'uint16',
+                2      : 'uint32',
+                3      : 'uint64',
+                #'RGB555'     : Core.RGB555,
+                #'RGB565'     : Core.RGB565,
+                #'RGB24'      : Core.RGB24,
+                #'RGB32'      : Core.RGB32,
+                #'BGR24'      : Core.BGR24,
+                #'BGR32'      : Core.BGR32,
+                #'BAYER RG8'  : Core.BAYER_RG8,
+                #'BAYER RG16' : Core.BAYER_RG16,
+                #'I420'       : Core.I420,
+                #'YUV411'     : Core.YUV411,
+                #'YUV422'     : Core.YUV422,
+                #'YUV444'     : Core.YUV444
+               }[mode]
+
 class CodecPipeline(Codec, list):
     """The codec class used when encoding/decoding data with multiple encoders
 
     Example usage::
         
-        >>> from taurus.core.util import CodecPipeline
+        >>> from taurus.core.util.codecs import CodecPipeline
         
         >>> data = range(100000)
         >>> codec = CodecPipeline('bz2_json')
@@ -512,7 +676,7 @@ class CodecFactory(Singleton, Logger):
     
     To get the singleton object do::
     
-        from taurus.core.util import CodecFactory
+        from taurus.core.util.codecs import CodecFactory
         f = CodecFactory()
         
     The :class:`CodecFactory` class allows you to get a codec object for a given 
@@ -522,7 +686,7 @@ class CodecFactory(Singleton, Logger):
 
     Example::
 
-        >>> from taurus.core.util import CodecFactory
+        >>> from taurus.core.util.codecs import CodecFactory
         >>> cf = CodecFactory()
         >>> json_codec = cf.getCodec('json')
         >>> bz2_json_codec = cf.getCodec('bz2_json')
@@ -539,7 +703,7 @@ class CodecFactory(Singleton, Logger):
 
         >>> # this example shows how to automatically get the data from a DEV_ENCODED attribute
         >>> import taurus
-        >>> from taurus.core.util import CodecFactory
+        >>> from taurus.core.util.codecs import CodecFactory
         >>> cf = CodecFactory()
         >>> devenc_attr = taurus.Attribute('a/b/c/devenc_attr')
         >>> v = devenc_attr.read()
@@ -555,6 +719,7 @@ class CodecFactory(Singleton, Logger):
         'zip'    : ZIPCodec,
         'pickle' : PickleCodec,
         'plot'   : PlotCodec,
+        'VIDEO_IMAGE' : VideoImageCodec,
         'null'   : NullCodec,
         'none'   : NullCodec,
         ''       : NullCodec })
@@ -632,4 +797,4 @@ class CodecFactory(Singleton, Logger):
         
     def encode(self, format, data, *args, **kwargs):
         return self.getCodec(format).encode(data, *args, **kwargs)
-    
\ No newline at end of file
+    
