@@ -381,7 +381,7 @@ class EventListener(object):
         with this object's lock before calling this method and always unlock it 
         afterward, of course::
         
-            from taurus.core.util import EventListener
+            from taurus.core.util.event import EventListener
             
             class MyEvtListener(EventListener):
                 # Your specific listener code here
@@ -460,7 +460,7 @@ class ListEventGenerator(EventGenerator):
 
 
 class AttributeEventWait(object):
-    """Class designed to connect to a :class:`taurus.core.TaurusAttribute` and
+    """Class designed to connect to a :class:`taurus.core.taurusattribute.TaurusAttribute` and
     fire events or wait for a certain event."""
 
     def __init__(self, attr=None):
@@ -474,7 +474,7 @@ class AttributeEventWait(object):
     def connect(self, attr):
         """Connect to the given attribute
         :param attr: the attribute to connect to
-        :type  attr: taurus.core.TaurusAttribute"""
+        :type  attr: taurus.core.taurusattribute.TaurusAttribute"""
         needAdd = True
         if self._attr is not None:
             if attr == self._attr:
@@ -521,9 +521,9 @@ class AttributeEventWait(object):
         """Event listener method for the underlying attribute. Do not call this
         method. It will be called internally when the attribute generates
         an event."""
-        if t == taurus.core.TaurusEventType.Config:
+        if t == taurus.core.taurusbasetypes.TaurusEventType.Config:
             return
-        elif t == taurus.core.TaurusEventType.Error:
+        elif t == taurus.core.taurusbasetypes.TaurusEventType.Error:
             self.fireEvent(None)
         else:
             self.fireEvent(v.value)
@@ -662,7 +662,7 @@ class AttributeEventIterator(object):
                   "locked by thread %s" % (curr_th.name, th.name)
     
     def eventReceived(self, s, t, v):
-        if t not in (taurus.core.TaurusEventType.Change, taurus.core.TaurusEventType.Periodic):
+        if t not in (taurus.core.taurusbasetypes.TaurusEventType.Change, taurus.core.taurusbasetypes.TaurusEventType.Periodic):
             return
         self.fireEvent(s, v.value)
     
