@@ -3,32 +3,33 @@
 #############################################################################
 ##
 ## This file is part of Taurus, a Tango User Interface Library
-## 
+##
 ## http://www.tango-controls.org/static/taurus/latest/doc/html/index.html
 ##
 ## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
-## 
+##
 ## Taurus is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU Lesser General Public License as published by
 ## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
-## 
+##
 ## Taurus is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU Lesser General Public License for more details.
-## 
+##
 ## You should have received a copy of the GNU Lesser General Public License
 ## along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
 ##
 #############################################################################
 
-"""This module contains a set of useful logging elements based on python's 
+"""This module contains a set of useful logging elements based on python's
 :mod:`logging` system."""
 
 __all__ = ["LogIt", "TraceIt", "DebugIt", "InfoIt", "WarnIt", "ErrorIt",
            "CriticalIt", "MemoryLogHandler", "LogExceptHook", "Logger",
-           "LogFilter"]
+           "LogFilter",
+           "_log", "trace", "debug", "info", "warning", "error", "critical"]
 
 __docformat__ = "restructuredtext"
 
@@ -39,77 +40,101 @@ import weakref
 import warnings
 import traceback
 import inspect
-import functools
 import threading
 
 from object import Object
+from wrap import wraps
 from excepthook import BaseExceptHook
 
 TRACE = 5
 logging.addLevelName(TRACE, "TRACE")
 
+#
+# _srcfile is used when walking the stack to check when we've got the first
+# caller stack frame.
+#
+if hasattr(sys, 'frozen'): #support for py2exe
+    _srcfile = "logging%s__init__%s" % (os.sep, __file__[-4:])
+elif __file__[-4:].lower() in ['.pyc', '.pyo']:
+    _srcfile = __file__[:-4] + '.py'
+else:
+    _srcfile = __file__
+_srcfile = os.path.normcase(_srcfile)
+
+# next bit filched from 1.5.2's inspect.py
+def currentframe():
+    """Return the frame object for the caller's stack frame."""
+    try:
+        raise Exception
+    except:
+        return sys.exc_info()[2].tb_frame.f_back
+
+if hasattr(sys, '_getframe'): currentframe = lambda: sys._getframe(3)
+# done filching
+
+
 class LogIt(object):
     """A function designed to be a decorator of any method of a Logger subclass.
     The idea is to log the entrance and exit of any decorated method of a Logger
     subclass.
     Example::
-    
-        from taurus.core.util import *
-        
+
+        from taurus.core.util.log import Logger, LogIt
+
         class Example(Logger):
-            
+
             @LogIt(Logger.Debug)
             def go(self):
                 print "Hello world "
-    
+
     This will generate two log messages of Debug level, one before the function
     go is called and another when go finishes. Example output::
-        
+
         MainThread     DEBUG    2010-11-15 15:36:11,440 Example: -> go
         Hello world of mine
         MainThread     DEBUG    2010-11-15 15:36:11,441 Example: <- go
-    
+
     This decorator can receive two optional arguments **showargs** and **showret**
     which are set to False by default. Enabling them will had verbose infomation
     about the parameters and return value. The following example::
 
-        from taurus.core.util import *
-        
+        from taurus.core.uti.log import Logger, LogIt
+
         class Example(Logger):
-            
+
             @LogIt(Logger.Debug, showargs=True, showret=True)
             def go(self, msg):
                 msg = "Hello world",msg
                 print msg
                 return msg
-                 
+
     would generate an output like::
 
         MainThread     DEBUG    2010-11-15 15:42:02,353 Example: -> go('of mine',)
         Hello world of mine
         MainThread     DEBUG    2010-11-15 15:42:02,353 Example: <- go = Hello world of mine
-    
-    .. note:: 
+
+    .. note::
         it may happen that in these examples that the output of the method
         appears before or after the log messages. This is because log
         messages are, by default, written to the *stardard error* while the print
         message inside the go method outputs to the *standard ouput*. On many
         systems these two targets are not synchronized.
     """
-    
+
     def __init__(self, level=logging.DEBUG, showargs=False, showret=False, col_limit=0):
         self._level = level
         self._showargs = showargs
         self._showret = showret
         self._col_limit = col_limit
-        
+
     def __call__(self, f):
-        @functools.wraps(f)
+        @wraps(f)
         def wrapper(*args, **kwargs):
             f_self = args[0]
             if f_self.log_level > self._level:
                 return f(*args, **kwargs)
-            
+
             has_log = hasattr(f_self, "log")
             fname = f.func_name
             log_obj = f_self
@@ -146,14 +171,14 @@ class LogIt(object):
 class TraceIt(LogIt):
     """Specialization of LogIt for trace level messages.
     Example::
-    
-        from taurus.core.util import TraceIt
+
+        from taurus.core.util.log import Logger, TraceIt
         class Example(Logger):
-            
+
             @TraceIt()
             def go(self):
                 print "Hello world"
-    
+
     .. seealso:: :class:`LogIt`"""
     def __init__(self, showargs=False, showret=False):
         LogIt.__init__(self, level=TRACE, showargs=showargs, showret=showret)
@@ -162,14 +187,14 @@ class TraceIt(LogIt):
 class DebugIt(LogIt):
     """Specialization of LogIt for debug level messages.
     Example::
-    
-        from taurus.core.util import DebugIt
+
+        from taurus.core.util.log import Logger, DebugIt
         class Example(Logger):
-            
+
             @DebugIt()
             def go(self):
                 print "Hello world"
-                
+
     .. seealso:: :class:`LogIt`"""
     def __init__(self, showargs=False, showret=False):
         LogIt.__init__(self, level=logging.DEBUG, showargs=showargs, showret=showret)
@@ -178,14 +203,14 @@ class DebugIt(LogIt):
 class InfoIt(LogIt):
     """Specialization of LogIt for info level messages.
     Example::
-    
-        from taurus.core.util import InfoIt
+
+        from taurus.core.util.log import Logger, InfoIt
         class Example(Logger):
-            
+
             @InfoIt()
             def go(self):
                 print "Hello world"
-    
+
     .. seealso:: :class:`LogIt`"""
     def __init__(self, showargs=False, showret=False):
         LogIt.__init__(self, level=logging.INFO, showargs=showargs, showret=showret)
@@ -194,14 +219,14 @@ class InfoIt(LogIt):
 class WarnIt(LogIt):
     """Specialization of LogIt for warn level messages.
     Example::
-    
-        from taurus.core.util import WarnIt
+
+        from taurus.core.util.log import Logger, WarnIt
         class Example(Logger):
-            
+
             @WarnIt()
             def go(self):
                 print "Hello world"
-    
+
     .. seealso:: :class:`LogIt`"""
     def __init__(self, showargs=False, showret=False):
         LogIt.__init__(self, level=logging.WARN, showargs=showargs, showret=showret)
@@ -210,14 +235,14 @@ class WarnIt(LogIt):
 class ErrorIt(LogIt):
     """Specialization of LogIt for error level messages.
     Example::
-    
-        from taurus.core.util import ErrorIt
+
+        from taurus.core.util.log import Logger, ErrorIt
         class Example(Logger):
-            
+
             @ErrorIt()
             def go(self):
                 print "Hello world"
-    
+
     .. seealso:: :class:`LogIt`"""
     def __init__(self, showargs=False, showret=False):
         LogIt.__init__(self, level=logging.ERROR, showargs=showargs, showret=showret)
@@ -226,14 +251,14 @@ class ErrorIt(LogIt):
 class CriticalIt(LogIt):
     """Specialization of LogIt for critical level messages.
     Example::
-    
-        from taurus.core.util import CriticalIt
+
+        from taurus.core.util.log import Logger, CriticalIt
         class Example(Logger):
-            
+
             @CriticalIt()
             def go(self):
                 print "Hello world"
-    
+
     .. seealso:: :class:`LogIt`"""
     def __init__(self, showargs=False, showret=False):
         LogIt.__init__(self, level=logging.CRITICAL, showargs=showargs, showret=showret)
@@ -242,22 +267,22 @@ class CriticalIt(LogIt):
 class MemoryLogHandler(list, logging.handlers.BufferingHandler):
     """An experimental log handler that stores temporary records in memory.
        When flushed it passes the records to another handler"""
-       
+
     def __init__(self, capacity=1000):
         list.__init__(self)
         logging.handlers.BufferingHandler.__init__(self, capacity=capacity)
         self._handler_list_changed = False
-    
+
     def shouldFlush(self, record):
         """Determines if the given record should trigger the flush
-        
+
            :param record: (logging.LogRecord) a log record
            :return: (bool) wheter or not the handler should be flushed
         """
         return (len(self.buffer) >= self.capacity) or \
                 (record.levelno >= Logger.getLogLevel()) or \
                 self._handler_list_changed
-    
+
     def flush(self):
         """Flushes this handler"""
         for record in self.buffer:
@@ -269,13 +294,13 @@ class MemoryLogHandler(list, logging.handlers.BufferingHandler):
         """Closes this handler"""
         self.flush()
         del self[:]
-        BufferingHandler.close(self)
+        logging.handlers.BufferingHandler.close(self)
 
 
 class LogExceptHook(BaseExceptHook):
     """A callable class that acts as an excepthook that logs the exception in
     the python logging system.
-    
+
     :param hook_to: callable excepthook that will be called at the end of
                     this hook handling [default: None]
     :type hook_to: callable
@@ -283,13 +308,13 @@ class LogExceptHook(BaseExceptHook):
     :type name: str
     :param level: log level [default: logging.ERROR]
     :type level: int"""
-    
+
     def __init__(self, hook_to=None, name=None, level=logging.ERROR):
         BaseExceptHook.__init__(self, hook_to=hook_to)
         name = name or self.__class__.__name__
         self._level = level
         self._log = Logger(name=name)
-        
+
     def report(self, *exc_info):
         text = "".join(traceback.format_exception(*exc_info))
         if text[-1] == '\n':
@@ -297,63 +322,90 @@ class LogExceptHook(BaseExceptHook):
         self._log.log(self._level, "Unhandled exception:\n%s", text)
 
 
+class _Logger(logging.Logger):
+    
+    def findCaller(self):
+        """
+        Find the stack frame of the caller so that we can note the source
+        file name, line number and function name.
+        """
+        f = currentframe()
+        #On some versions of IronPython, currentframe() returns None if
+        #IronPython isn't run with -X:Frames.
+        if f is not None:
+            f = f.f_back
+        rv = "(unknown file)", 0, "(unknown function)"
+        while hasattr(f, "f_code"):
+            co = f.f_code
+            filename = os.path.normcase(co.co_filename)
+            if filename in (_srcfile, logging._srcfile):
+                f = f.f_back
+                continue
+            rv = (co.co_filename, f.f_lineno, co.co_name)
+            break
+        return rv
+   
+
 class Logger(Object):
     """The taurus logger class. All taurus pertinent classes should inherit
     directly or indirectly from this class if they need taurus logging
     facilities."""
-    
+
     #: Internal usage
     root_inited    = False
-    
+
     #: Internal usage
     root_init_lock = threading.Lock()
-    
+
     #: Critical message level (constant)
     Critical = logging.CRITICAL
-    
+
     #: Error message level (constant)
     Error    = logging.ERROR
-    
+
     #: Warning message level (constant)
     Warning  = logging.WARNING
-    
+
     #: Info message level (constant)
     Info     = logging.INFO
-    
+
     #: Debug message level (constant)
     Debug    = logging.DEBUG
-    
+
     #: Trace message level (constant)
     Trace    = TRACE
-    
+
     #: Default log level (constant)
     DftLogLevel = Info
-    
+
+    #: Default log message format (constant)
+    DftLogMessageFormat = '%(threadName)-14s %(levelname)-8s %(asctime)s %(name)s: %(message)s'
+
     #: Default log format (constant)
-    DftLogFormat = logging.Formatter('%(threadName)-14s %(levelname)-8s %(asctime)s %(name)s: %(message)s')
-    
+    DftLogFormat = logging.Formatter(DftLogMessageFormat)
+
     #: Current global log level
     log_level = DftLogLevel
-    
+
     #: Default log message format
     log_format = DftLogFormat
-    
+
     #: the main stream handler
     stream_handler = None
-    
-    
+
+
     def __init__(self, name='', parent=None, format=None):
         """The Logger constructor
-        
+
         :param name: (str) the logger name (default is empty string)
         :param parent: (Logger) the parent logger or None if no parent exists (default is None)
         :param format: (str) the log message format or None to use the default log format (default is None)
         """
         self.call__init__(Object)
-        
+
         if format: self.log_format = format
         Logger.initRoot()
-        
+
         if name is None or len(name) == 0:
             name = self.__class__.__name__
         self.log_name = name
@@ -361,8 +413,8 @@ class Logger(Object):
             self.log_full_name = '%s.%s' % (parent.log_full_name, name)
         else:
             self.log_full_name = name
-
-        self.log_obj = logging.getLogger(self.log_full_name)
+        
+        self.log_obj = self._getLogger(self.log_full_name)
         self.log_handlers = []
 
         self.log_parent = None
@@ -373,8 +425,7 @@ class Logger(Object):
 
     def cleanUp(self):
         """The cleanUp. Default implementation does nothing
-           Overwrite when necessary
-        """
+           Overwrite when necessary"""
         pass
 
     @classmethod
@@ -383,16 +434,16 @@ class Logger(Object):
            method directly in your code
         """
         if cls.root_inited:
-            return cls._getRootLog()
-        
+            return cls._getLogger()
+
         try:
             cls.root_init_lock.acquire()
-            root_logger = cls._getRootLog()
+            root_logger = cls._getLogger()
             logging.addLevelName(cls.Trace, "TRACE")
             cls.stream_handler = logging.StreamHandler(sys.__stderr__)
             cls.stream_handler.setFormatter(cls.log_format)
             root_logger.addHandler(cls.stream_handler)
-            
+
             console_log_level = os.environ.get("TAURUSLOGLEVEL", None)
             if console_log_level is not None:
                 console_log_level = console_log_level.capitalize()
@@ -407,16 +458,16 @@ class Logger(Object):
     @classmethod
     def addRootLogHandler(cls, h):
         """Adds a new handler to the root logger
-        
+
            :param h: (logging.Handler) the new log handler
         """
         h.setFormatter(cls.getLogFormat())
         cls.initRoot().addHandler(h)
-        
+
     @classmethod
     def removeRootLogHandler(cls, h):
         """Removes the given handler from the root logger
-        
+
            :param h: (logging.Handler) the handler to be removed
         """
         cls.initRoot().removeHandler(h)
@@ -438,24 +489,24 @@ class Logger(Object):
     @classmethod
     def setLogLevel(cls,level):
         """sets the new log level (the root log level)
-        
+
            :param level: (int) the new log level
         """
         cls.log_level = level
         cls.initRoot().setLevel(level)
-    
+
     @classmethod
     def getLogLevel(cls):
         """Retuns the current log level (the root log level)
-        
+
            :return: (int) a number representing the log level
         """
         return cls.log_level
-    
+
     @classmethod
     def setLogFormat(cls,format):
         """sets the new log message format
-        
+
            :param level: (str) the new log message format
         """
         cls.log_format = logging.Formatter(format)
@@ -466,11 +517,11 @@ class Logger(Object):
     @classmethod
     def getLogFormat(cls):
         """Retuns the current log message format (the root log format)
-        
+
            :return: (str) the log message format
         """
         return cls.log_format
-    
+
     @classmethod
     def resetLogLevel(cls):
         """Resets the log level (the root log level)"""
@@ -480,11 +531,11 @@ class Logger(Object):
     def resetLogFormat(cls):
         """Resets the log message format (the root log format)"""
         cls.setLogFormat(cls.DftLogFormat)
-            
+
     @classmethod
     def addLevelName(cls, level_no, level_name):
         """Registers a new log level
-           
+
            :param level_no: (int) the level number
            :param level_name: (str) the corresponding name
         """
@@ -492,30 +543,40 @@ class Logger(Object):
         level_name = level_name.capitalize()
         if not hasattr(cls, level_name):
             setattr(cls, level_name, level_no)
-    
+
     @classmethod
     def getRootLog(cls):
         """Retuns the root logger
-        
+
            :return: (logging.Logger) the root logger
         """
-        cls.initRoot()
-        return cls._getRootLog()
+        return cls.initRoot()
+
+    @staticmethod
+    def _getLogger(name=None):
+        orig_logger_class = logging.getLoggerClass()
+        try:
+            logging.setLoggerClass(_Logger)
+            ret = logging.getLogger(name)
+            return ret
+        finally:
+            logging.setLoggerClass(orig_logger_class)
 
     @classmethod
-    def _getRootLog(cls):
-        return logging.getLogger()
-    
+    def getLogger(cls, name=None):
+        cls.initRoot()
+        return cls._getLogger(name=name)
+
     def getLogObj(self):
         """Returns the log object for this object
-        
+
            :return: (logging.Logger) the log object
         """
         return self.log_obj
-    
+
     def getParent(self):
         """Returns the log parent for this object or None if no parent exists
-        
+
            :return: (logging.Logger or None) the log parent for this object
         """
         if self.log_parent is None:
@@ -524,7 +585,7 @@ class Logger(Object):
 
     def getChildren(self):
         """Returns the log children for this object
-           
+
            :return: (sequence<logging.Logger) the list of log children
         """
         children = []
@@ -533,10 +594,10 @@ class Logger(Object):
             if child is not None:
                 children.append(child)
         return children
-        
+
     def addChild(self, child):
         """Adds a new logging child
-           
+
            :param child: (logging.Logger) the new child
         """
         if not self.log_children.get(id(child)):
@@ -544,7 +605,7 @@ class Logger(Object):
 
     def addLogHandler(self, handler):
         """Registers a new handler in this object's logger
-        
+
            :param handler: (logging.Handler) the new handler to be added
         """
         self.log_obj.addHandler(handler)
@@ -552,54 +613,54 @@ class Logger(Object):
 
     def copyLogHandlers(self, other):
         """Copies the log handlers of other object to this object
-           
+
            :param other: (object) object which contains 'log_handlers'
         """
         for handler in other.log_handlers:
             self.addLogHandler(handler)
-            
+
     def trace(self, msg, *args, **kw):
         """Record a trace message in this object's logger. Accepted *args* and
            *kwargs* are the same as :meth:`logging.Logger.log`.
-        
+
            :param msg: (str) the message to be recorded
            :param args: list of arguments
            :param kw: list of keyword arguments
         """
         self.log_obj.log(self.Trace, msg, *args, **kw)
-    
+
     def traceback(self, level=Trace, extended=True):
         """Log the usual traceback information, followed by a listing of all the
            local variables in each frame.
-        
+
            :param level: (int) the log level assigned to the traceback record
            :param extended: (bool) if True, the log record message will have multiple lines
-           
+
            :return: (str) The traceback string representation
         """
         out = traceback.format_exc()
         if extended:
             out += "\n"
             out += self._format_trace()
-            
+
         self.log_obj.log(level, out)
         return out
 
     def stack(self, target=Trace):
         """Log the usual stack information, followed by a listing of all the
            local variables in each frame.
-           
+
            :param target: (int) the log level assigned to the record
-           
+
            :return: (str) The stack string representation
         """
         out = self._format_stack()
         self.log_obj.log(target, out)
         return out
-    
+
     def _format_trace(self):
         return self._format_stack(inspect.trace)
-    
+
     def _format_stack(self, stack_func=inspect.stack):
         line_count = 3
         stack = stack_func(line_count)
@@ -629,7 +690,7 @@ class Logger(Object):
                         cut = False
                         v = str(v)
                         i = v.find('\n')
-                        if i == -1: 
+                        if i == -1:
                             i = 80
                         else:
                             i = min(i, 80)
@@ -640,22 +701,22 @@ class Logger(Object):
                     except:
                         out += '<could not find suitable string representation>'
         return out
-    
+
     def log(self, level, msg, *args, **kw):
         """Record a log message in this object's logger. Accepted *args* and
            *kwargs* are the same as :meth:`logging.Logger.log`.
-        
+
            :param level: (int) the record level
            :param msg: (str) the message to be recorded
            :param args: list of arguments
            :param kw: list of keyword arguments
         """
         self.log_obj.log(level, msg, *args, **kw)
-    
+
     def debug(self, msg, *args, **kw):
         """Record a debug message in this object's logger. Accepted *args* and
            *kwargs* are the same as :meth:`logging.Logger.debug`.
-        
+
            :param msg: (str) the message to be recorded
            :param args: list of arguments
            :param kw: list of keyword arguments
@@ -665,7 +726,7 @@ class Logger(Object):
     def info(self, msg, *args, **kw):
         """Record an info message in this object's logger. Accepted *args* and
            *kwargs* are the same as :meth:`logging.Logger.info`.
-        
+
            :param msg: (str) the message to be recorded
            :param args: list of arguments
            :param kw: list of keyword arguments
@@ -675,7 +736,7 @@ class Logger(Object):
     def warning(self, msg, *args, **kw):
         """Record a warning message in this object's logger. Accepted *args* and
            *kwargs* are the same as :meth:`logging.Logger.warning`.
-        
+
            :param msg: (str) the message to be recorded
            :param args: list of arguments
            :param kw: list of keyword arguments
@@ -685,7 +746,7 @@ class Logger(Object):
     def deprecated(self, msg, *args, **kw):
         """Record a deprecated warning message in this object's logger. Accepted *args* and
            *kwargs* are the same as :meth:`logging.Logger.warning`.
-        
+
            :param msg: (str) the message to be recorded
            :param args: list of arguments
            :param kw: list of keyword arguments
@@ -697,7 +758,7 @@ class Logger(Object):
     def error(self, msg, *args, **kw):
         """Record an error message in this object's logger. Accepted *args* and
            *kwargs* are the same as :meth:`logging.Logger.error`.
-        
+
            :param msg: (str) the message to be recorded
            :param args: list of arguments
            :param kw: list of keyword arguments
@@ -707,23 +768,23 @@ class Logger(Object):
     def critical(self, msg, *args, **kw):
         """Record a critical message in this object's logger. Accepted *args* and
            *kwargs* are the same as :meth:`logging.Logger.critical`.
-        
+
            :param msg: (str) the message to be recorded
            :param args: list of arguments
            :param kw: list of keyword arguments
         """
         self.log_obj.critical(msg, *args, **kw)
-    
+
     def exception(self, msg, *args):
-        """Log a message with severity 'ERROR' on the root logger, with 
-           exception information.. Accepted *args* are the same as 
+        """Log a message with severity 'ERROR' on the root logger, with
+           exception information.. Accepted *args* are the same as
            :meth:`logging.Logger.exception`.
-        
+
            :param msg: (str) the message to be recorded
            :param args: list of arguments
         """
         self.log_obj.exception(msg, *args)
-    
+
     def flushOutput(self):
         """Flushes the log output"""
         self.syncLog()
@@ -743,44 +804,44 @@ class Logger(Object):
                 sync()
                 synced.append(handler)
             logger = logger.getParent()
-        
+
     def getLogName(self):
         """Gets the log name for this object
-        
+
            :return: (str) the log name
         """
         return self.log_name
-    
+
     def getLogFullName(self):
         """Gets the full log name for this object
-        
+
            :return: (str) the full log name
         """
         return self.log_full_name
 
     def changeLogName(self,name):
         """Change the log name for this object.
-        
+
            :param name: (str) the new log name
-        """ 
+        """
         self.log_name = name
         p = self.getParent()
         if p is not None:
             self.log_full_name = '%s.%s' % (p.log_full_name, name)
         else:
             self.log_full_name = name
-        
+
         self.log_obj = logging.getLogger(self.log_full_name)
         for handler in self.log_handlers:
             self.log_obj.addHandler(handler)
-        
+
         for child in self.getChildren():
             child.changeLogName(child.log_name)
 
 
 class LogFilter(logging.Filter):
     """Experimental log filter"""
-    
+
     def __init__(self, level):
         self.filter_level = level
         logging.Filter.__init__(self)
@@ -788,3 +849,29 @@ class LogFilter(logging.Filter):
     def filter(self, record):
         ok = (record.levelno == self.filter_level)
         return ok
+
+def __getrootlogger():
+    return Logger.getLogger("TaurusRootLogger")
+    
+# cannot export log because upper package taurus.core.util imports this 'log' 
+# module and it would itself be overwritten by this log function
+def _log(level, msg, *args, **kw):
+    return __getrootlogger().log(level, msg, *args, **kw)
+
+def trace(msg, *args, **kw):
+    return _log(Logger.Trace, msg, *args, **kw)
+
+def debug(msg, *args, **kw):
+    return __getrootlogger().debug(msg, *args, **kw)
+
+def info(msg, *args, **kw):
+    return __getrootlogger().info(msg, *args, **kw)
+
+def warning(msg, *args, **kw):
+    return __getrootlogger().warning(msg, *args, **kw)
+
+def error(msg, *args, **kw):
+    return __getrootlogger().error(msg, *args, **kw)
+
+def critical(msg, *args, **kw):
+    return __getrootlogger().critical(msg, *args, **kw)
