@@ -50,12 +50,14 @@ class configurableProperty:
         else:
             result = self.fget() 
         return result
+    
     def applyConfig(self, value, depth=-1):
         '''calls the fset function for this property with the given value. The depth parameter is ignored'''
         if isinstance(self.fget, basestring):# fget is not a method but a method name...
             getattr(self._obj, self.fset)(value)
         else:
             self.fset(value)
+            
     def objectName(self):
         '''returns the name of this property'''
         return self.name
@@ -122,8 +124,11 @@ class BaseConfigurableClass:
     implementing "perspectives" in your application.
     
     '''
+    
+    defaultConfigRecursionDepth = -1
+    _supportedConfigVersions = ("__UNVERSIONED__",)#the latest element of this list is considered the current version
+    
     def __init__(self):
-        self._supportedConfigVersions = ["__UNVERSIONED__"] #the latest element of this list is considered the current version
         self.resetConfigurableItems()
     
     @staticmethod
@@ -185,7 +190,7 @@ class BaseConfigurableClass:
         configdict["__orderedConfigNames__"] = self.__configurableItemNames
         return configdict
     
-    def applyConfig(self, configdict, depth=-1):
+    def applyConfig(self, configdict, depth=None):
         """applies the settings stored in a configdict to the current object.
         
         In most usual situations, using :meth:`registerConfigProperty` and 
@@ -198,12 +203,17 @@ class BaseConfigurableClass:
                       for this object, and not for any other object registered
                       via :meth:`registerConfigurableItem`. If depth > 0,
                       applyConfig will be called recursively as many times as
-                      the depth value. If depth < 0 (default), no limit is
-                      imposed to recursion (i.e., it will recurse for as deep as
-                      there are registered items)
+                      the depth value. If depth < 0 (default, see note), no
+                      limit is imposed to recursion (i.e., it will recurse for
+                      as deep as there are registered items).
+                      
+        .. note:: the default recursion depth can be tweaked in derived classes
+                  by changing the class property `defaultConfigRecursionDepth`
         
         .. seealso:: :meth:`createConfig`
         """
+        if depth is None:
+            depth = self.defaultConfigRecursionDepth
         if not self.checkConfigVersion(configdict): 
             raise ValueError('the given configuration is of unsupported version')
         #delegate restoring the configuration of any registered configurable item
@@ -318,6 +328,9 @@ class BaseConfigurableClass:
         :param item: (object or str) The object that should be unregistered.
                      Alternatively, the name under which the object was registered
                      can be passed as a python string.
+        :param raiseOnError: (bool) If True (default), it raises a KeyError 
+                             exception if item was not registered. If False, it 
+                             just logs a debug message 
         
         .. seealso:: :meth:`registerConfigProperty`, :meth:`registerConfigDelegate`
         '''
@@ -330,7 +343,7 @@ class BaseConfigurableClass:
         elif raiseOnError:
             raise KeyError('"%s" was not registered.'%name)
         else:
-            self.info('"%s" was not registered. Skipping'%name)
+            self.debug('"%s" was not registered. Skipping'%name)
             return False
 
       
@@ -399,8 +412,8 @@ class BaseConfigurableClass:
         import cPickle as pickle
         if ofile is None:
             from taurus.qt import Qt
-            ofile = Qt.QFileDialog.getSaveFileName( self, 'Save Configuration', '%s.pck'%self.__class__.__name__, 'Configuration File (*.pck)')
-            if ofile.isEmpty(): return
+            ofile = unicode(Qt.QFileDialog.getSaveFileName( self, 'Save Configuration', '%s.pck'%self.__class__.__name__, 'Configuration File (*.pck)'))
+            if not ofile: return
         if not isinstance(ofile,file): ofile=open(ofile,'w')
         configdict=self.createConfig(allowUnpickable=False)
         self.info("Saving current settings in '%s'"%ofile.name)
@@ -417,8 +430,8 @@ class BaseConfigurableClass:
         import cPickle as pickle
         if ifile is None:
             from taurus.qt import Qt
-            ifile = Qt.QFileDialog.getOpenFileName( self, 'Load Configuration', '', 'Configuration File (*.pck)')
-            if ifile.isEmpty(): return
+            ifile = unicode(Qt.QFileDialog.getOpenFileName( self, 'Load Configuration', '', 'Configuration File (*.pck)'))
+            if not ifile: return
         if not isinstance(ifile,file): ifile=open(ifile,'r')
         configdict=pickle.load(ifile)
         self.applyConfig(configdict)
