@@ -3,28 +3,30 @@
 #############################################################################
 ##
 ## This file is part of Taurus, a Tango User Interface Library
-## 
+##
 ## http://www.tango-controls.org/static/taurus/latest/doc/html/index.html
 ##
 ## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
-## 
+##
 ## Taurus is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU Lesser General Public License as published by
 ## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
-## 
+##
 ## Taurus is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU Lesser General Public License for more details.
-## 
+##
 ## You should have received a copy of the GNU Lesser General Public License
 ## along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
 ##
 #############################################################################
 
-"""This module provides the base :class:`taurus.qt.qtgui.application.TaurusApplication` 
-class."""
+"""This module provides the base
+:class:`taurus.qt.qtgui.application.TaurusApplication` class."""
+
+from __future__ import with_statement
 
 __all__ = ["TaurusApplication"]
 
@@ -34,6 +36,7 @@ import os
 import sys
 import logging
 import optparse
+import threading
 
 from taurus.qt import Qt
 
@@ -42,11 +45,13 @@ import taurus.core.util.argparse
 
 
 class STD(Logger):
-    
+
+    FlushWaterMark = 1000
+
     def __init__(self, name='', parent=None, format=None, std=None,
                  pure_text=True):
         """The STD Logger constructor
-        
+
         :param name: (str) the logger name (default is empty string)
         :param parent: (Logger) the parent logger or None if no parent exists
                        (default is None)
@@ -54,27 +59,31 @@ class STD(Logger):
                        log format (default is None)
         :param std: std to forward write
         :param pure_text: if True, writes the 'message' parameter of the log
-                          message in a separate line preserving the indentation
+                          message in a separate line preserving the
+                          indentation
         """
         Logger.__init__(self, name=name, parent=parent, format=format)
         self.buffer = ''
         self.log_obj.propagate = False
         self.std = std
-    
+
     def addLogHandler(self, handler):
-        """When called, set to use a private handler and DON'T send messages to
-           parent loggers (basically will act as an independent logging system
-           by itself)
+        """When called, set to use a private handler and DON'T send messages
+        to parent loggers (basically will act as an independent logging system
+        by itself)
 
         :param handler: new handler"""
         Logger.addLogHandler(self, handler)
         self.log_obj.propagate = not len(self.log_handlers)
-    
+
     def write(self, msg):
         try:
             self.buffer += msg
             # while there is no new line, just accumulate the buffer
-            if msg and (msg[-1] == '\n' or msg.index('\n') >= 0):
+            msgl = len(msg)
+            if msgl > 0 and \
+               (msg[-1] == '\n' or msg.index('\n') >= 0 or \
+                msgl >= self.FlushWaterMark):
                 self.flush()
         except ValueError:
             pass
@@ -85,7 +94,7 @@ class STD(Logger):
                 except:
                     pass
             pass
-        
+
     def flush(self):
         try:
             buff = self.buffer
@@ -93,7 +102,8 @@ class STD(Logger):
                 return
             #take the '\n' because the output is a list of strings, each to be
             #interpreted as a separate line in the client
-            if buff[-1] == '\n': buff = buff[:-1]
+            if buff[-1] == '\n':
+                buff = buff[:-1]
             if self.log_handlers:
                 self.log(Logger.Console, '\n' + buff)
             self.buffer = ""
@@ -108,75 +118,82 @@ class STD(Logger):
 
 class TaurusApplication(Qt.QApplication, Logger):
     """A QApplication that additionally parses the command line looking
-    for taurus options. This is done using the :mod:`taurus.core.util.argparse`.
+    for taurus options. This is done using the
+    :mod:`taurus.core.util.argparse`.
     To create a TaurusApplication object you should use the same parameters
-    as in QApplication. 
-    
+    as in QApplication.
+
     The optional keyword parameters:
         - app_name: (str) application name
         - app_version: (str) application version
         - org_name: (str) organization name
         - org_domain: (str) organization domain
-    
-    ...And at last the 'cmd_line_parser' which should be an instance of 
+
+    ...And at last the 'cmd_line_parser' which should be an instance of
     :class:`optparse.OptionParser`. Simple example::
 
         import sys
         import taurus.qt.qtgui.application
         import taurus.qt.qtgui.display
-                
+
         app = taurus.qt.qtgui.application.TaurusApplication()
-        
+
         w = taurus.qt.qtgui.display.TaurusLabel()
         w.model = 'sys/tg_test/1/double_scalar'
         w.show()
 
         sys.exit(app.exec_())
-    
+
     A more complex example showing how to add options and a usage help::
-    
+
         import sys
         import taurus.core.util.argparse
         import taurus.qt.qtgui.application
         import taurus.qt.qtgui.display
-                
+
         parser = taurus.core.util.argparse.get_taurus_parser()
         parser.usage = "%prog [options] <model>"
         parser.add_option("--hello")
-        
+
         app = taurus.qt.qtgui.application.TaurusApplication(cmd_line_parser=parser)
         args = app.get_command_line_args()
         if len(args) < 1:
             sys.stderr.write("Need to supply model attribute")
             sys.exit(1)
-        
+
         w = taurus.qt.qtgui.display.TaurusLabel()
         w.model = args[1]
         w.show()
 
         sys.exit(app.exec_())
-        
-    For more details on taurus command line parsing check 
+
+    For more details on taurus command line parsing check
     :mod:`taurus.core.util.argparse`.
     """
-    
+
     def __init__(self, *args, **kwargs):
         """The constructor. Parameters are the same as QApplication plus a
         keyword parameter: 'cmd_line_parser' which should be an instance of
         :class:`optparse.OptionParser`"""
+
+        # lock to safely get singleton elements (like IPython taurus
+        # console app)
+        self._lock = threading.Lock()
+        
         if len(args) == 0:
-            args = ([],)
+            args = getattr(sys, 'argv', []),
+
         parser=None
         app_name, app_version, org_name, org_domain = None, None, None, None
-        if kwargs.has_key('app_name'):
+        if 'app_name' in kwargs:
             app_name = kwargs.pop('app_name')
-        if kwargs.has_key('app_version'):
+        if 'app_version' in kwargs:
             app_version = kwargs.pop('app_version')
-        if kwargs.has_key('org_name'):
+        if 'org_name' in kwargs:
             org_name = kwargs.pop('org_name')
-        if kwargs.has_key('org_domain'):
+        if 'org_domain' in kwargs:
             org_domain = kwargs.pop('org_domain')
-        if kwargs.has_key('cmd_line_parser'):
+        if 'cmd_line_parser' in kwargs:
             parser = kwargs.pop('cmd_line_parser')
 
         try:
@@ -186,6 +203,9 @@ class TaurusApplication(Qt.QApplication, Logger):
 
         Logger.__init__(self)
 
+        self._out = None
+        self._err = None
+
         if app_name is not None:
             self.setApplicationName(app_name)
         if app_version is not None:
@@ -195,9 +215,6 @@ class TaurusApplication(Qt.QApplication, Logger):
         if org_domain is not None:
             self.setOrganizationDomain(org_domain)
 
-        self._out = None
-        self._err = None
-        
         # if the constructor was called without a parser or with a parser that
         # doesn't contain version information and with an application
         # name and/or version, then add the --version capability
@@ -211,19 +228,20 @@ class TaurusApplication(Qt.QApplication, Logger):
                 parser.version = v
                 parser._add_version_option()
 
-        p, opt, args = taurus.core.util.argparse.init_taurus_args(parser=parser)
-        
+        p, opt, args = \
+            taurus.core.util.argparse.init_taurus_args(parser=parser, args=args[0][1:])
+
         self._cmd_line_parser = p
         self._cmd_line_options = opt
         self._cmd_line_args = args
         self.__registerQtLogger()
         self.__registerExtensions()
         self.__redirect_std()
-    
+
     def __registerQtLogger(self):
         import taurus.qt.qtcore.util
         taurus.qt.qtcore.util.initTaurusQtLogger()
-    
+
     def __registerExtensions(self):
         """Registers taurus Qt extensions"""
         try:
@@ -246,8 +264,10 @@ class TaurusApplication(Qt.QApplication, Logger):
             sys.stdout = self._out
             self._err = STD(name="ERR", std=sys.stderr)
             sys.stderr = self._err
-            
-    def __buildLogFileName(self, prefix="/tmp", name=None):
+
+    def __buildLogFileName(self, prefix=None, name=None):
+        if prefix is None:
+            prefix = os.path.expanduser('~/tmp')
         appName = str(self.applicationName())
         if not appName:
             appName = os.path.splitext(os.path.basename(sys.argv[0]))[0]
@@ -258,42 +278,42 @@ class TaurusApplication(Qt.QApplication, Logger):
             name = appName + '.log'
         fileName = os.path.join(dirName, name)
         return fileName
-    
+
     def get_command_line_parser(self):
-        """Returns the :class:`optparse.OptionParser` used to parse the command
-        line parameters.
-        
+        """Returns the :class:`optparse.OptionParser` used to parse the
+        command line parameters.
+
         :return: the parser used in the command line
         :rtype: :class:`optparse.OptionParser`"""
         return self._cmd_line_parser
-        
+
     def get_command_line_options(self):
         """Returns the :class:`optparse.Option` that resulted from parsing the
         command line parameters.
-        
+
         :return: the command line options
         :rtype: :class:`optparse.Option`"""
         return self._cmd_line_options
-        
+
     def get_command_line_args(self):
         """Returns the list of arguments that resulted from parsing the
         command line parameters.
-        
+
         :return: the command line arguments
         :rtype: list of strings"""
         return self._cmd_line_args
-    
+
     def setTaurusStyle(self, styleName):
         """Sets taurus application style to the given style name
-        
+
         :param styleName: the new style name to be applied
         :type styleName: str"""
         import taurus.qt.qtgui.style
         taurus.qt.qtgui.style.setTaurusStyle(styleName)
-    
+
     def basicConfig(self, log_file_name=None, maxBytes=1E7, backupCount=5,
                     with_gui_exc_handler=True):
-        
+
         # prepare exception handler to report exceptions as error log messages
         # and to taurus message box (if with_gui is set)
         hook_to = None
@@ -301,19 +321,18 @@ class TaurusApplication(Qt.QApplication, Logger):
             import taurus.qt.qtgui.dialog
             hook_to = taurus.qt.qtgui.dialog.TaurusExceptHookMessageBox()
         sys.excepthook = LogExceptHook(hook_to=hook_to)
-        
+
         # create a file log handler
         try:
             if log_file_name is None:
                 log_file_name = self.__buildLogFileName()
             f_h = logging.handlers.RotatingFileHandler(log_file_name,
-                                                       maxBytes=maxBytes,
-                                                       backupCount=backupCount)
+                maxBytes=maxBytes, backupCount=backupCount)
             Logger.addRootLogHandler(f_h)
-            if self._out is not None:            
+            if self._out is not None:
                 self._out.std = sys.__stdout__
                 self._out.addLogHandler(f_h)
-            if self._out is not None:            
+            if self._out is not None:
                 self._err.std = sys.__stderr__
                 self._err.addLogHandler(f_h)
             self.info("Logs will be saved in %s", log_file_name)
@@ -321,4 +340,4 @@ class TaurusApplication(Qt.QApplication, Logger):
             self.warning("'%s' could not be created. Logs will not be stored",
                          log_file_name)
             self.debug("Error description", exc_info=1)
-        
+
