@@ -37,7 +37,7 @@ import weakref
 
 from taurus.qt import Qt
 
-from taurus.core import DataFormat, TaurusEventType
+from taurus.core.taurusbasetypes import DataFormat, TaurusEventType
 
 from taurus.qt.qtgui.util import QT_ATTRIBUTE_QUALITY_PALETTE
 from taurus.qt.qtgui.util import QT_DEVICE_STATE_PALETTE
@@ -105,24 +105,31 @@ class TaurusBaseController(object):
         return self.widget().getDisplayValue()
     
     def handleEvent(self, evt_src, evt_type, evt_value):
-        if evt_type == TaurusEventType.Change or evt_type == TaurusEventType.Periodic:
-            self._last_value = evt_value
-        elif evt_type == TaurusEventType.Config:
-            self._last_config_value = evt_value
-        else:
-            self._last_error_value = evt_value
-            #In case of error, modify the last_value as well
-            try:
-                self._last_value = self.modelObj().getValueObj()
-            except:
-                pass
+        if evt_src == self.modelObj(): #update the "_last" values only if the event source is the model (it could be the background...) 
+            if evt_type == TaurusEventType.Change or evt_type == TaurusEventType.Periodic:
+                self._last_value = evt_value
+            elif evt_type == TaurusEventType.Config:
+                self._last_config_value = evt_value
+            else:
+                self._last_error_value = evt_value
+                #In case of error, modify the last_value as well
+                try:
+                    self._last_value = self.modelObj().getValueObj()
+                except:
+                    pass
         self.update()
 
     def eventReceived(self, evt_src, evt_type, evt_value):
         # should handle the state event here. Because this is invoked by a random
         # thread, we pass it to the widget, which will forward to the proper
         # thread
-        self.widget().eventReceived(evt_src, evt_type, evt_value)
+        
+        #@todo: sometimes we get this method called but self.widget() is None. Check why. 
+        #       For the moment I just protect it by substituting the following line by the ones after it
+        #self.widget().eventReceived(evt_src, evt_type, evt_value)
+        w = self.widget()
+        if w is not None:            
+            w.eventReceived(evt_src, evt_type, evt_value)
     
     def update(self):
         widget = self.widget()
@@ -250,7 +257,9 @@ class TaurusConfigurationControllerHelper(object):
         if self._configParam is None:
             model = self.widget().model
             try:
-                self._configParam = model[model.rfind('=')+1:].lower() #@todo: !!This is assuming tango names.A regexp or a call to a validator should be used
+                #@todo: This works for tango, eval and epics configuration names but is not general.
+                #@todo: This should be done calling to the ConfigurationNameValidator
+                self._configParam = model[model.rfind('?configuration=')+15:].lower() 
             except:
                 self._configParam = ''
         return self._configParam
@@ -269,14 +278,29 @@ class TaurusConfigurationControllerHelper(object):
                     val = widget.getNoneValue()
             except:
                 pass
-        except:
+        except AttributeError:
+            if param:
+                val = str(param)
+                attr = self.attrObj()
+                if attr is not None:
+                    val = val.replace('<label>', attr.label or '---')      
+                    val = val.replace('<attr_name>',attr.name or '---')
+                    val = val.replace('<attr_fullname>',attr.getFullName() or '---')
+                    val = val.replace('<dev_alias>',attr.dev_alias or '---')
+                    val = val.replace('<dev_name>',attr.dev_name or '---')
+                dev = self.deviceObj()   
+                if dev is not None:     
+                    val = val.replace('<dev_fullname>',dev.getFullName() or '---')
+            else:
+                val = widget.getNoneValue()
+        except:    
             widget.debug("Invalid configuration parameter '%s'" % param)
             val = widget.getNoneValue()
         if val is None:
             val = widget.getNoneValue()
         return val
-
-
+ 
+ 
 StyleSheetTemplate = """border-style: outset; border-width: 2px; border-color: {0}; {1}"""
 
 def _updatePaletteColors(widget, bgBrush, fgBrush, frameBrush):
