@@ -33,12 +33,13 @@ __docformat__ = 'restructuredtext'
 import PyTango
 
 from taurus.qt import Qt
-from taurus.core import TaurusDevice, LockStatus, TaurusLockInfo
+from taurus.core.taurusbasetypes import LockStatus, TaurusLockInfo
+from taurus.core.taurusdevice import TaurusDevice
 from taurus.qt.qtgui.base import TaurusBaseWidget
 from taurus.core.util import eventfilters
-from taurus.core.util import Enumeration
+from taurus.core.util.enumeration import Enumeration
 from taurus.qt.qtgui.resource import getIcon
-from taurus.qt.qtgui.dialog import TaurusMessageBox
+from taurus.qt.qtgui.dialog import ProtectTaurusMessageBox
 
 class _ButtonDialog(Qt.QDialog):
     _widget = None
@@ -47,6 +48,7 @@ class _ButtonDialog(Qt.QDialog):
         Qt.QDialog.__init__(self, parent, Qt.Qt.WindowTitleHint)
         l = Qt.QVBoxLayout()
         self.setLayout(l)
+        self.previousWidgetConfig = None
         
     def setWidget(self, widget):
         oldWidget = self.widget()
@@ -67,6 +69,10 @@ class _ButtonDialog(Qt.QDialog):
         
     def closeEvent(self, event):
         if self.deleteWidgetOnClose:
+            try:
+                self.previousWidgetConfig = self.widget().createConfig()
+            except:
+                self.previousWidgetConfig = None
             self.setWidget(None)
         Qt.QDialog.closeEvent(self, event)
 
@@ -122,7 +128,13 @@ class TaurusLauncherButton(Qt.QPushButton, TaurusBaseWidget):
         self._text = text
         self.setText(self.getDisplayValue())
         self._dialog = _ButtonDialog(self)
-        self.setWidget(widget)
+        if widget is None:
+            pass
+        elif isinstance(widget, Qt.QWidget):
+            self._deleteWidgetOnClose = False # we cannot be sure on recreating the same widget again
+            self.setWidget(widget)
+        else:
+            self.setWidgetClassName(widget)
         self.connect(self, Qt.SIGNAL('clicked()'), self.onClicked)
         self.setDefault(False)
         self.setAutoDefault(False)
@@ -159,6 +171,11 @@ class TaurusLauncherButton(Qt.QPushButton, TaurusBaseWidget):
         klass = TaurusWidgetFactory().getWidgetClass(self._widgetClassName)
         widget = klass(*self._args, **self._kwargs)
         self.setWidget(widget)
+        if self._dialog.previousWidgetConfig is not None:
+            try:
+                widget.applyConfig(self._dialog.previousWidgetConfig)
+            except Exception,e:
+                self.warning('Cannot apply previous configuration to widget. Reason: %s',repr(e))
     
     def widget(self):
         return self._dialog.widget()
@@ -169,7 +186,7 @@ class TaurusLauncherButton(Qt.QPushButton, TaurusBaseWidget):
         :param widget: (Qt.QWidget)
         '''
         self._dialog.setWidget(widget)
-        if self._text is None and self.self.widget() is not None:
+        if self._text is None and self.widget() is not None:
             self._text = self.widget().__class__.__name__
         
     def displayValue(self,v):
@@ -185,9 +202,9 @@ class TaurusLauncherButton(Qt.QPushButton, TaurusBaseWidget):
             return '---'
         config = modelobj.getConfig()
         if config.isSpectrum():
-            return 'Spect[%i]' % modelobj.read().dim_x
+            return 'Spect[%i]' % modelobj.read().value.shape[0]
         elif config.isImage():
-            return 'Imag[%ix%i]'%(modelobj.read().dim_x,modelobj.read().dim_y)
+            return 'Imag[%ix%i]'%(modelobj.read().value.shape)
         else:
             self.debug('Unknown model type for TaurusLauncherButton')
             return '---'
@@ -276,10 +293,11 @@ class TaurusCommandButton(Qt.QPushButton, TaurusBaseWidget):
             return '---'
         return self._command
     
-    import taurus.qt.qtgui.dialog
-    
-    @taurus.qt.qtgui.dialog.ProtectTaurusMessageBox(title="Unexpected error when executing command")
+    @ProtectTaurusMessageBox(title="Unexpected error when executing command")
     def onClicked(self):
+        return self._onClicked()
+
+    def _onClicked(self):
         '''Slot called when the button is clicked. It executes the command with
         parameters. It may issue a warning if the command is flagged as
         dangerous. On successful execution, it returns the command result and it
@@ -323,7 +341,7 @@ class TaurusCommandButton(Qt.QPushButton, TaurusBaseWidget):
                            passed, the currently set parameters are used.
         :param command: (str) the command name. If None is passed, the currently
                         set command is used.
-        :param dev: (taurus.core.TaurusDevice) the device on which the command is
+        :param dev: (taurus.core.taurusdevice.TaurusDevice) the device on which the command is
                     executed. If None is passed, the current model is used.
         
         :return: (sequence or scalar) a sequence of parameters (or a scalar if only one parameter)
@@ -381,10 +399,19 @@ class TaurusCommandButton(Qt.QPushButton, TaurusBaseWidget):
                            elements of the sequence are not of the right type
                            required for the parameter, an automatic conversion
                            will be attempted on execution time. As a special
-                           case, if parameters is a string, it will be splitted
-                           on whitespace to obtain a sequence of parameters.
+                           case, if parameters is a string not starting and
+                           ending in quote characters, it will be splitted on
+                           whitespace to obtain a sequence of parameters. If
+                           it is a string starting and ending with quotes, the
+                           quotes will be removed and the quoted text will not
+                           be splitted.
         '''
-        if isinstance(parameters,(basestring, Qt.QString)): parameters = str(parameters).split()
+        if isinstance(parameters,(basestring, Qt.QString)): 
+            parameters = str(parameters).strip()
+            if parameters[0] in ('"',"'") and parameters[0] == parameters[-1]:
+                parameters = [parameters[1:-1]]
+            else:
+                parameters = parameters.split()
         self._parameters = parameters
     
     def getParameters(self):
@@ -507,6 +534,7 @@ class TaurusLockButton(Qt.QPushButton, TaurusBaseWidget):
             self._on_toggle(down)
         except:
             import sys
+            from taurus.qt.qtgui.dialog import TaurusMessageBox
             msgbox = TaurusMessageBox(*sys.exc_info())
             msgbox.setWindowTitle("Error locking device")
             if self.update_button().status == LockStatus.Locked:
@@ -515,41 +543,9 @@ class TaurusLockButton(Qt.QPushButton, TaurusBaseWidget):
         
     model = Qt.pyqtProperty("QString", TaurusBaseWidget.getModel, setModel,
                             TaurusBaseWidget.resetModel)
-  
-#if __name__ == "__main__":
-#    import sys
-#    from taurus.widget.qwt import TaurusPlot, TaurusArrayEditor
-#    #from taurus.widget import TaurusAttrForm
-#    import taurus.widget.resources.qrc_taurusdesigner_icons
-#    
-#    app = Qt.QApplication(sys.argv)
-#    
-#    ##Uncomment the following for testing TaurusLauncherButton
-#    w=TaurusArrayEditor()
-#    form = TaurusLauncherButton(parent=None, designMode=False, widget = w, icon=':/taurus.png', text = 'show')
-#    form.setModel('bl97/pc/dummy-03/waveform')
-#    #form.setModel('bl97/pc/dummy-03')
-#    #form.setModel('bl97/pyattributeprocessor/1/a|bl97/pyattributeprocessor/1/b')
-#    
-#    ##Uncomment the following for testing TaurusCommandButton
-#    ##form = TaurusCommandButton(parent=None, designMode=False, command = 'Exec', parameters=['2+2'], icon=':/taurus.png', text = None)
-#    #form = TaurusCommandButton(parent=None, designMode=False, command = 'Status', parameters=None, icon=':/taurus.png', text = None)
-#    #form.setModel('bl97/pc/dummy-03')
-#    #form.setDangerMessage('Booo scary command!!\n Maybe you should think twice!')
-#    #def f(*a):print a
-#    #form.connect(form, Qt.SIGNAL('commandExecuted'),f)
-#    form.show()
-#    
-#    sys.exit(app.exec_())
-#
-#
 
-def demo():
-    lock_button = TaurusLockButton()
-    lock_button.model = "sys/tg_test/1"
-    return lock_button
 
-def main():
+def lockButtonMain():
     import sys
     import taurus.qt.qtgui.application
     Application = taurus.qt.qtgui.application.TaurusApplication
@@ -586,5 +582,58 @@ def main():
     else:
         return w
 
+
+def commandButtonMain():
+    import sys
+    from taurus.qt.qtgui.application import TaurusApplication
+    
+    app = TaurusApplication()
+    form = TaurusCommandButton(parent=None, designMode=False, command = 'DevBoolean', parameters=[123], icon=':/taurus.png', text = 'launch: DevBoolean 123')
+    form.setModel('sys/tg_test/1')
+    form.setDangerMessage('Booo scary command!!\n Maybe you should think twice!')
+    def f(*a):print a
+    form.connect(form, Qt.SIGNAL('commandExecuted'),f)
+    form.show()
+    sys.exit(app.exec_())
+
+
+def launcherButtonMain():
+    import sys
+    from taurus.qt.qtgui.application import TaurusApplication
+    
+    app = TaurusApplication()
+    
+    #Creating button giving the widget
+#    from taurus.qt.qtgui.plot import TaurusPlot
+#    w=TaurusPlot()
+#    form = TaurusLauncherButton(parent=None, designMode=False, widget = w, icon=':/taurus.png', text = 'show')
+#    
+    #Creating button giving the widget class name
+#    form = TaurusLauncherButton(parent=None, designMode=False, widget = 'TaurusPlot', icon=':/taurus.png', text = 'show')
+    
+    #Creating button using a derived class with the name widget class hardcoded
+    class MyButton(TaurusLauncherButton):
+        _widgetClassName = 'TaurusPlot'   
+        _icon = ':/taurus.png'
+        _text = 'show'
+    form = MyButton()
+    
+    form.setModel('sys/tg_test/1/wave')
+    form.show()
+    sys.exit(app.exec_())
+
+
+def main():
+    lockButtonMain()
+    
+    
+def demo():
+    lock_button = TaurusLockButton()
+    lock_button.model = "sys/tg_test/1"
+    return lock_button
+
+
 if __name__ == '__main__':
-    main()
+    #lockButtonMain()
+    launcherButtonMain()
+    #commandButtonMain()
