@@ -28,7 +28,10 @@
 __docformat__ = 'restructuredtext'
 
 try:
-    from .taurusconsole import *
-except:
-    from taurus.qt.qtgui.display import create_taurus_fallback as __create
-    TaurusConsole = __create("TaurusConsole")
+    from .taurusconsole import TaurusConsole
+except Exception,e:
+    from taurus.qt.qtgui.display import TaurusFallBackWidget
+
+    class TaurusConsole(TaurusFallBackWidget):
+        pass
+
