@@ -42,7 +42,6 @@ from taurus.qt.qtgui.util import ExternalAppAction
 from taurus.qt.qtgui.resource import getIcon, getThemeIcon
 from taurus.qt.qtgui.dialog import protectTaurusMessageBox
 
-import cPickle as pickle
 
 class CommandArgsLineEdit(Qt.QLineEdit):
     ''' An specialized QLineEdit that can transform its text from/to command argument lists'''
@@ -125,33 +124,51 @@ class Rpdb2WaitDialog(Qt.QMessageBox):
         
 
 class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
-    '''A Taurus-aware QMainWindow with several customizations:
+    '''
+    A Taurus-aware QMainWindow with several customizations:
     
         - It takes care of (re)storing its geometry and state (see :meth:`loadSettings`)
-        - It provides a splashScreen (which can be disabled)
-        - It provides a statusBar (@TODO)
-        - It provides basic Taurus menus and actions:
-            The following Menus are already Provided:
-                - Help (self.helpMenu)
-                - About
-                - View (self.viewMenu)
-                - Taurus (self.taurusMenu)
-            The following actions are already provided:
-                -Help-->About
-        - It incorporates a TaurusLogo (@TODO)
+        - Supports perspectives (programmatic access and, optionally, 
+          accessible by user), and allows defining a set of "factory settings" 
+        - It provides a customizable splashScreen (optional)
+        - Supports spawning remote consoles and remote debugging
+        - Supports full-screen mode toggling
+        - Supports adding launchers to external applications
+        - It provides a statusBar with an optional heart-beat LED
+        - The following Menus are optionally provided and populated with basic actions:
+            - File  (accessible by derived classes  as `self.fileMenu`)
+            - View (accessible by derived classes  as `self.viewMenu`)
+            - Taurus (accessible by derived classes  as `self.taurusMenu`)
+            - Tools (accessible by derived classes  as `self.toolsMenu`)
+            - Help (accessible by derived classes  as `self.helpMenu`)
+                
     '''
     __pyqtSignals__ = ("modelChanged(const QString &)",)
-        
-    def __init__(self, parent = None, designMode = False, splash=True):
+    
+    #customization options:
+    _heartbeat = 1500 #blinking semi-period in ms. Set to None for not showing the Heart beat LED 
+    _showFileMenu = True
+    _showViewMenu = True
+    _showTaurusMenu = True
+    _showToolsMenu = True
+    _showHelpMenu = True
+    _supportUserPerspectives = True #Allows the user to change/create/delete perspectives
+    _showLogger = True
+    _splashLogo = ":/TaurusSplash.png" #set to None for disabling splash screen
+    _splashMessage = "Initializing Main window..."
+    
+    def __init__(self, parent = None, designMode = False, splash=None):
         name = self.__class__.__name__
         self.call__init__wo_kw(Qt.QMainWindow, parent)
         self.call__init__(TaurusBaseContainer, name, designMode=designMode)
+        if splash is None:  
+            splash = bool(self._splashLogo)
         
         self.__splashScreen = None
         if splash and not designMode:
-            self.__splashScreen = Qt.QSplashScreen(Qt.QPixmap(":/logo.png"))
+            self.__splashScreen = Qt.QSplashScreen(Qt.QPixmap(self._splashLogo))
             self.__splashScreen.show()
-            self.__splashScreen.showMessage("Initializing Main window...")
+            self.__splashScreen.showMessage(self._splashMessage)
         
         self.__tangoHost = ""
         self.__settings = None
@@ -162,6 +179,14 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         self.helpManualBrowser = None
         self.resetHelpManualURI()
         
+        #Heartbeat
+        if self._heartbeat is not None:
+            from taurus.qt.qtgui.display import QLed
+            self.heartbeatLed = QLed()
+            self.heartbeatLed.setToolTip('Heartbeat: if it does not blink, the application is hung')
+            self.statusBar().addPermanentWidget(self.heartbeatLed)
+            self.resetHeartbeat()
+        
         #The configuration Dialog (which is launched with the configurationAction)
         self.configurationDialog = ConfigurationDialog(self)
         
@@ -170,48 +195,20 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         self.__createActions()
         
         #logger dock widget
-        import taurus.qt.qtgui.table
-        loggingWidget = taurus.qt.qtgui.table.QLoggingWidget()
-        self.__loggerDW = Qt.QDockWidget("Taurus logs", self)
-        self.__loggerDW.setWidget(loggingWidget)
-        self.__loggerDW.setObjectName("loggerDW")
-        self.addDockWidget(Qt.Qt.BottomDockWidgetArea, self.__loggerDW)
-        self.__loggerDW.hide()
+        if self._showLogger:
+            self.addLoggerWidget()
         
         #Create Menus
-        #File menu
-        self.fileMenu = self.menuBar().addMenu("File")
-        self.fileMenu.addAction(self.importSettingsFileAction)
-        self.fileMenu.addAction(self.exportSettingsFileAction)
-        #self.fileMenu.addAction(self.resetSettingsAction)
-        self.fileMenu.addSeparator()
-        self.fileMenu.addAction(self.quitApplicationAction)
-        
-        #View menu
-        self.viewMenu = self.menuBar().addMenu("View")
-        self.viewMenu.addAction(self.__loggerDW.toggleViewAction())
-        self.viewToolBarsMenu = self.viewMenu.addMenu("Tool Bars")
-        self.viewMenu.addSeparator()
-        self.viewMenu.addAction(self.toggleFullScreenAction)
-        self.viewMenu.addSeparator()
-        self.perspectivesMenu = Qt.QMenu("Load Perspectives", self)
-        self.viewMenu.addMenu(self.perspectivesMenu)
-        self.viewMenu.addAction(self.savePerspectiveAction)
-        self.viewMenu.addAction(self.deletePerspectiveAction)
-        
-        #Taurus Menu
-        self.taurusMenu = self.menuBar().addMenu("Taurus")
-        self.taurusMenu.addAction(self.changeTangoHostAction)
-        
-        #Tools Menu
-        self.toolsMenu = self.menuBar().addMenu("Tools")
-        self.externalAppsMenu = self.toolsMenu.addMenu("External Applications")
-        self.toolsMenu.addAction(self.configurationAction)
-        
-        #Help Menu
-        self.helpMenu = self.menuBar().addMenu("Help")
-        self.helpMenu.addAction("About ...", self.showHelpAbout)
-        self.helpMenu.addAction(taurus.qt.qtgui.resource.getThemeIcon("help-browser"),"Manual", self.onShowManual)
+        if self._showFileMenu:#File menu
+            self.createFileMenu()
+        if self._showViewMenu:#View menu
+            self.createViewMenu()
+        if self._showTaurusMenu:#Taurus Menu
+            self.createTaurusMenu()
+        if self._showToolsMenu:#Tools Menu
+            self.createToolsMenu()
+        if self._showHelpMenu:#Help Menu
+            self.createHelpMenu()
         
         #View Toolbar
         self.viewToolBar = self.addToolBar("View")
@@ -219,10 +216,72 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         self.viewToolBar.addAction(self.toggleFullScreenAction)
         
         #Perspectives Toolbar
+        if self._supportUserPerspectives:
+            self.createPerspectivesToolBar()
+        
+        #disable the configuration action if there is nothing to configure
+        self.configurationAction.setEnabled(self.configurationDialog._tabwidget.count())
+                
+    def addLoggerWidget(self, hidden=True):
+        '''adds a QLoggingWidget as a dockwidget of the main window (and hides it by default)'''
+        from taurus.qt.qtgui.table import QLoggingWidget
+        loggingWidget = QLoggingWidget()
+        self.__loggerDW = Qt.QDockWidget("Taurus logs", self)
+        self.__loggerDW.setWidget(loggingWidget)
+        self.__loggerDW.setObjectName("loggerDW")
+        self.addDockWidget(Qt.Qt.BottomDockWidgetArea, self.__loggerDW)
+        if hidden:
+            self.__loggerDW.hide()
+    
+    def createFileMenu(self):
+        '''adds a "File" Menu'''
+        self.fileMenu = self.menuBar().addMenu("File")
+        if self._supportUserPerspectives:
+            self.fileMenu.addAction(self.importSettingsFileAction)
+            self.fileMenu.addAction(self.exportSettingsFileAction)
+            #self.fileMenu.addAction(self.resetSettingsAction)
+            self.fileMenu.addSeparator()
+        self.fileMenu.addAction(self.quitApplicationAction)
+        
+    def createViewMenu(self):
+        '''adds a "View" Menu'''
+        self.viewMenu = self.menuBar().addMenu("View")
+        if self._showLogger:
+            self.viewMenu.addAction(self.__loggerDW.toggleViewAction())
+        self.viewToolBarsMenu = self.viewMenu.addMenu("Tool Bars")
+        self.viewMenu.addSeparator()
+        self.viewMenu.addAction(self.toggleFullScreenAction)
+        if self._supportUserPerspectives:
+            self.viewMenu.addSeparator()
+            self.perspectivesMenu = Qt.QMenu("Load Perspectives", self)
+            self.viewMenu.addMenu(self.perspectivesMenu)
+            self.viewMenu.addAction(self.savePerspectiveAction)
+            self.viewMenu.addAction(self.deletePerspectiveAction)
+    
+    def createTaurusMenu(self):
+        '''adds a "Taurus" Menu'''
+        self.taurusMenu = self.menuBar().addMenu("Taurus")
+        self.taurusMenu.addAction(self.changeTangoHostAction)
+        
+    def createToolsMenu(self):
+        '''adds a "Tools" Menu'''
+        self.toolsMenu = self.menuBar().addMenu("Tools")
+        self.externalAppsMenu = self.toolsMenu.addMenu("External Applications")
+        self.toolsMenu.addAction(self.configurationAction)
+        
+    def createHelpMenu(self):
+        '''adds a "Help" Menu'''
+        self.helpMenu = self.menuBar().addMenu("Help")
+        self.helpMenu.addAction("About ...", self.showHelpAbout)
+        self.helpMenu.addAction(getThemeIcon("help-browser"),"Manual", self.onShowManual)
+        
+    def createPerspectivesToolBar(self):
+        '''adds a Perspectives ToolBar'''
         self.perspectivesToolBar = self.addToolBar("Perspectives")
         self.perspectivesToolBar.setObjectName("perspectivesToolBar")
-        self.viewToolBarsMenu.addAction(self.perspectivesToolBar.toggleViewAction())
         pbutton = Qt.QToolButton()
+        if not hasattr(self, 'perspectivesMenu'): #it may have been created earlier (for the view menu)
+            self.perspectivesMenu = Qt.QMenu("Load Perspectives", self)
         self.perspectivesMenu.setIcon(getThemeIcon("document-open"))
         pbutton.setToolTip("Load Perspectives")
         pbutton.setText("Load Perspectives")
@@ -230,9 +289,8 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         pbutton.setMenu(self.perspectivesMenu)
         self.perspectivesToolBar.addWidget(pbutton)
         self.perspectivesToolBar.addAction(self.savePerspectiveAction)
-        
-        #disable the configuration action if there is nothing to configure
-        self.configurationAction.setEnabled(self.configurationDialog._tabwidget.count())
+        if self._showViewMenu:
+            self.viewToolBarsMenu.addAction(self.perspectivesToolBar.toggleViewAction())
         
     def updatePerspectivesMenu(self):
         '''re-checks the perspectives available to update self.perspectivesMenu
@@ -240,8 +298,10 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         .. note:: This method may need be called by derived classes at the end
                   of their initialization.
         
-        :return: (QMenu) the updated perspectives menu
+        :return: (QMenu) the updated perspectives menu (or None if self._supportUserPerspectives is False)
         '''
+        if not self._supportUserPerspectives:
+            return None
         self.perspectivesMenu.clear()
         for pname in self.getPerspectivesList():
             self.perspectivesMenu.addAction(pname, self.__onPerspectiveSelected)
@@ -488,28 +548,33 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         if group is not None: 
             settings.beginGroup(group)
         if not ignoreGeometry: 
-            self.restoreGeometry(settings.value("MainWindow/Geometry").toByteArray()) 
+            ba = Qt.from_qvariant(settings.value("MainWindow/Geometry"), 'toByteArray') or Qt.QByteArray() #With API2, from_qvariant is returning None instead
+                                                                                                           # of an empty QByTeArray
+                                                                                                           # and this caused an exception later on. Hence the "or"
+            self.restoreGeometry(ba) 
         #restore the Taurus config
         try:
-            self.applyQConfig(settings.value('TaurusConfig').toByteArray())
+            ba = Qt.from_qvariant(settings.value("TaurusConfig"), 'toByteArray') or Qt.QByteArray()
+            self.applyQConfig(ba)
         except Exception,e:
             msg = 'Problem loading configuration from "%s". Some settings may not be restored.\n Details: %s'%(unicode(settings.fileName()), repr(e))
             self.error(msg)
             Qt.QMessageBox.warning(self,'Error Loading settings', msg, Qt.QMessageBox.Ok)
-        self.restoreState(settings.value("MainWindow/State").toByteArray()) 
+        ba = Qt.from_qvariant(settings.value("MainWindow/State"), 'toByteArray') or Qt.QByteArray()
+        self.restoreState(ba) 
         #hide all dockwidgets (so that they are shown only if they were present in the settings)
         dockwidgets = [c for c in self.children() if isinstance(c, Qt.QDockWidget)]
         for d in dockwidgets:
             r = self.restoreDockWidget(d)
             d.hide()
-        self.restoreState(settings.value("MainWindow/State").toByteArray()) 
+        ba = Qt.from_qvariant(settings.value("MainWindow/State"), 'toByteArray') or Qt.QByteArray()
+        self.restoreState(ba) 
         
         if group is not None: 
             settings.endGroup()
         self.updatePerspectivesMenu()
         self.info('MainWindow settings restored')
         
-    
     def saveSettings(self, group=None):
         '''saves the application settings (so that they can be restored with :meth:`loadSettings`)
         
@@ -532,7 +597,6 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
             settings.endGroup()
         self.info('MainWindow settings saved in "%s"'%settings.fileName())
         
-        
     def savePerspective(self, name=None):
         '''Stores current state of the application as a perspective with the given name
         
@@ -553,7 +617,8 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         self.updatePerspectivesMenu()
         
     def loadPerspective(self, name=None, settings=None):
-        '''Loads the settings saved for the given perspective
+        '''Loads the settings saved for the given perspective.
+        It emits a 'perspectiveChanged' signal with name as its parameter
                 
         :param name: (str) name of the perspective            
         :param settings: (QSettings or None) a QSettings object. If None given,
@@ -562,11 +627,12 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         '''
         if name is None:
             perspectives = self.getPerspectivesList()
-            if perspectives.isEmpty(): return
+            if len(perspectives) == 0: return
             name,ok = Qt.QInputDialog.getItem(self, "Load Perspective", "Change perspective to:",
                                               perspectives, 0, False) 
             if not ok: return
         self.loadSettings(settings=settings, group="Perspectives/%s"%name, ignoreGeometry=True)
+        self.emit(Qt.SIGNAL('perspectiveChanged'), name)
     
     def getPerspectivesList(self, settings=None):
         '''Returns the list of saved perspectives 
@@ -595,7 +661,7 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         if settings is None: settings = self.getQSettings()
         if name is None:
             perspectives = self.getPerspectivesList()
-            if perspectives.isEmpty(): return
+            if len(perspectives) == 0: return
             name,ok = Qt.QInputDialog.getItem(self, "Delete Perspective", "Choose perspective to be deleted:",
                                               perspectives, 0, False) 
             if not ok: return
@@ -613,9 +679,9 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         :param fname: (str) name of output file. If None given, a file dialog will be shown.
         '''
         if fname is None:
-            fname = Qt.QFileDialog.getSaveFileName(self, 'Choose file where the current settings should be saved', 
-                                                   '', "Ini files (*.ini);;All files (*)")
-            if fname.isNull():
+            fname = unicode(Qt.QFileDialog.getSaveFileName(self, 'Choose file where the current settings should be saved', 
+                                                   '', "Ini files (*.ini);;All files (*)"))
+            if not fname:
                 return
         self.saveSettings()
         ok = Qt.QFile.copy(self.getQSettings().fileName(), fname)
@@ -633,9 +699,9 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         :param fname: (str) name of ini file. If None given, a file dialog will be shown.
         '''
         if fname is None:
-            fname = Qt.QFileDialog.getOpenFileName(self, 'Select a ini-format settings file',
-                                                   '', "Ini files (*.ini);;All files (*)")
-            if fname.isNull():
+            fname = unicode(Qt.QFileDialog.getOpenFileName(self, 'Select a ini-format settings file',
+                                                   '', "Ini files (*.ini);;All files (*)"))
+            if not fname:
                 return
         s = Qt.QSettings(fname, Qt.QSettings.IniFormat)
         #clone the perspectives found in the "factory" settings
@@ -650,6 +716,11 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
 #        self.__settings = self.newQSettings()
 #        self.saveSettings()
 
+    def showEvent(self,event):
+        '''This event handler receives widget show events'''
+        if self.__splashScreen is not None and not event.spontaneous():
+            self.__splashScreen.finish(self)
+
     def closeEvent(self,event):
         '''This event handler receives widget close events'''
         self.saveSettings() #save current window state before closing
@@ -687,15 +758,13 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
                 self.extAppsBar = self.addToolBar("External Applications")
                 self.extAppsBar.setObjectName("External Applications")
                 self.extAppsBar.setToolButtonStyle(Qt.Qt.ToolButtonTextBesideIcon)
-                self.viewToolBarsMenu.addAction(self.extAppsBar.toggleViewAction())
+                if self._showViewMenu:
+                    self.viewToolBarsMenu.addAction(self.extAppsBar.toggleViewAction())
             self.extAppsBar.addAction(extapp)
             
-        if toMenu:
+        if toMenu and self._showToolsMenu:
             if self.toolsMenu is None:
-                self.toolsMenu = Qt.QMenu("Tools", self)
-                self.menuBar().insertMenu(self.helpMenu.menuAction(), self.toolsMenu) #insert it before the Help menu
-            if self.externalAppsMenu is None:
-                self.externalAppsMenu = self.toolsMenu.addMenu("External Applications")
+                self.createToolsMenu()
             self.externalAppsMenu.addAction(extapp)
         #register this action for config
         self.registerConfigDelegate(extapp, "_extApp[%s]"%str(extapp.text()))
@@ -729,7 +798,7 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         self.__helpManualURI = uri
         if self.helpManualBrowser is None:
             try:
-                from PyQt4.QtWebKit import QWebView
+                from taurus.qt.QtWebKit import QWebView
                 self.helpManualBrowser = QWebView()
             except:
                 self.helpManualBrowser = Qt.QLabel('QWebkit is not available')
@@ -744,15 +813,15 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         return self.__helpManualURI
     
     def resetHelpManualURI(self):
-        from taurus.core import Release
-        uri = getattr(self, 'MANUAL_URI',Release.url)
+        from taurus.core import release
+        uri = getattr(self, 'MANUAL_URI', release.url)
         self.setHelpManualURI(uri)
     
     def showHelpAbout(self):
         appname = unicode(Qt.qApp.applicationName())
         appversion = unicode(Qt.qApp.applicationVersion())
-        from taurus.core import Release
-        abouttext = "%s %s\n\nUsing %s %s"%(appname, appversion, Release.name, Release.version)
+        from taurus.core import release
+        abouttext = "%s %s\n\nUsing %s %s"%(appname, appversion, release.name, release.version)
         Qt.QMessageBox.about(self, 'About', abouttext)
             
     def onShowManual(self, anchor=None):
@@ -772,11 +841,11 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         it means that this application is not the only one
         '''
         if key is None:
-            from taurus.core.util import getSystemUserName
+            from taurus.core.util.user import getSystemUserName
             username = getSystemUserName()
             appname = unicode(Qt.QApplication.applicationName())
             key = "__socket_%s-%s__"%(username,appname)
-        from PyQt4 import QtNetwork       
+        from taurus.qt import QtNetwork       
         socket = QtNetwork.QLocalSocket(self) 
         socket.connectToServer(key)
         alive = socket.waitForConnected(3000)
@@ -788,7 +857,12 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
             self.connect(self.socketServer, Qt.SIGNAL("newConnection()"), self.onIncommingSocketConnection)
             ok = self.socketServer.listen(key)
             if not ok:
-                if self.socketServer.serverError() == Qt.QAbstractSocket.AddressInUseError:
+                try:
+                    AddressInUseError = Qt.QAbstractSocket.AddressInUseError #This fails in some PyQt4 versions...
+                except:
+                    from PyQt4.QtNetwork import QAbstractSocket #...so we try this other way of accessing
+                    AddressInUseError = QAbstractSocket.AddressInUseError 
+                if self.socketServer.serverError() == AddressInUseError:
                     self.info('Resetting unresponsive socket with key "%s"',key)
                     if hasattr(self.socketServer, 'removeServer'): #removeServer() was added in Qt4.5. (In Qt4.4 a call to listen() removes a previous server)  
                         self.socketServer.removeServer(key)
@@ -814,10 +888,25 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
         socket.deleteLater()
         self.raise_()
         self.activateWindow()
-        
-        
-        
 
+    def setHeartbeat(self, interval):
+        '''sets the interval of the heartbeat LED for the window. 
+        The heartbeat is displayed by a Led in the status bar unless 
+        it is disabled by setting the interval to 0
+        
+        :param interval: (int) heart beat interval in millisecs. Set to 0 to disable
+        '''
+        self.heartbeatLed.setBlinkingInterval(interval)
+        self.heartbeatLed.setVisible(interval>0)
+        
+    def getHeartbeat(self):
+        '''returns the heart beat interval'''
+        return self.heartbeatLed.getBlinkingInterval()
+        
+    def resetHeartbeat(self):
+        '''resets the heartbeat interval'''
+        self.setHeartbeat(self.__class__._heartbeat)
+            
     
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
     # Public slots for apply/restore changes
@@ -855,18 +944,46 @@ class TaurusMainWindow(Qt.QMainWindow, TaurusBaseContainer):
     helpManualURI = Qt.pyqtProperty("QString", getHelpManualURI, 
                                         setHelpManualURI, 
                                         resetHelpManualURI)
+    
+    heartbeat = Qt.pyqtProperty("int", getHeartbeat, 
+                                      setHeartbeat, 
+                                      resetHeartbeat)
 
 #---------
 
 if __name__ == "__main__":
-    
-    import sys
+
     import taurus.qt.qtgui.application
     app = taurus.qt.qtgui.application.TaurusApplication()
     app.setApplicationName('TaurusMainWindow-test')
     app.setOrganizationName('ALBA')
     app.basicConfig()
-    form = TaurusMainWindow()
+        
+    class MyMainWindow(TaurusMainWindow):
+        _heartbeat = 300 #blinking semi-period in ms. Set to None for not showing the Heart beat LED 
+        _showFileMenu = True
+        _showViewMenu = True
+        _showTaurusMenu = False
+        _showToolsMenu = True
+        _showHelpMenu = True
+        _supportUserPerspectives = True #Allows the user to change/create/delete perspectives
+        _showLogger = True
+        _splashLogo = ":/TaurusSplash.png" #set to None for disabling splash screen
+        _splashMessage = "Initializing Main window..."
+        def __init__(self):
+            TaurusMainWindow.__init__(self, parent=None, designMode=False, splash=None)
+            #simulating a lengthy initialization
+            import time
+            for i in range(5):
+                time.sleep(0.5)
+                self.splashScreen().showMessage("starting: step %i/5"%(i+1))
+            
+            
+    
+    
+    #MainWindowKlass = TaurusMainWindow
+        
+    form = MyMainWindow()
     
     #ensure only a single instance of this application is running
     single = form.checkSingleInstance()
@@ -879,10 +996,9 @@ if __name__ == "__main__":
     
     #form.setCentralWidget(Qt.QMdiArea()) #just for testing
     
-#    form.addExternalAppLauncher('pwd')
+    #form.addExternalAppLauncher('pwd')
     
     form.show()
     
-    form.splashScreen().finish(form)
     sys.exit(app.exec_())
     
