@@ -3,28 +3,28 @@
 #############################################################################
 ##
 ## This file is part of Taurus, a Tango User Interface Library
-## 
+##
 ## http://www.tango-controls.org/static/taurus/latest/doc/html/index.html
 ##
 ## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
-## 
+##
 ## Taurus is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU Lesser General Public License as published by
 ## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
-## 
+##
 ## Taurus is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU Lesser General Public License for more details.
-## 
+##
 ## You should have received a copy of the GNU Lesser General Public License
 ## along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
 ##
 #############################################################################
 
-"""This module provides a demo for the :class:`taurus.qt.qtgui.display.TaurusLabel`
-widget """
+"""This module provides a demo for the
+:class:`taurus.qt.qtgui.display.TaurusLabel` widget """
 
 __all__ = ["demo", "main"]
 
@@ -32,6 +32,7 @@ __docformat__ = 'restructuredtext'
 
 from taurus.qt import Qt
 
+
 def demo():
     import sys
     import taurus.qt.qtgui.application
@@ -41,18 +42,17 @@ def demo():
     getPixmap = taurus.qt.qtgui.resource.getPixmap
     Application = taurus.qt.qtgui.application.TaurusApplication
     QPixmapWidget = taurus.qt.qtgui.display.QPixmapWidget
-    
+
     app = Application.instance()
     owns_app = app is None
-    
+
     if owns_app:
-        import taurus.core.util.argparse
         app = Application()
 
     M = 2
 
     class QPixmapWidgetTestPanel(Qt.QWidget):
-        
+
         def __init__(self, parent=None):
             Qt.QWidget.__init__(self, parent)
             panel_l = Qt.QVBoxLayout()
@@ -83,7 +83,7 @@ def demo():
             control_l.addRow("Transformation mode:", transformation_widget)
             control_l.addRow("Horiz. alignment:", halign_widget)
             control_l.addRow("Vert. alignment:", valign_widget)
-            
+
             panel_l.addWidget(display_panel, 1)
             panel_l.addWidget(control_panel, 0)
 
@@ -95,13 +95,13 @@ def demo():
             valign_widget.addItem("Top", Qt.QVariant(Qt.Qt.AlignTop))
             valign_widget.addItem("Center", Qt.QVariant(Qt.Qt.AlignVCenter))
             valign_widget.addItem("Bottom", Qt.QVariant(Qt.Qt.AlignBottom))
-            
+
             Qt.QObject.connect(pixmap_widget, Qt.SIGNAL("textChanged(const QString &)"), self.changePixmap)
             Qt.QObject.connect(aspect_ratio_widget, Qt.SIGNAL("currentIndexChanged(int)"), self.changeAspectRatio)
             Qt.QObject.connect(transformation_widget, Qt.SIGNAL("currentIndexChanged(int)"), self.changeTransformationMode)
             Qt.QObject.connect(halign_widget, Qt.SIGNAL("currentIndexChanged(int)"), self.changeAlignment)
             Qt.QObject.connect(valign_widget, Qt.SIGNAL("currentIndexChanged(int)"), self.changeAlignment)
-            
+
             self.w = w
             self.w_pixmap = pixmap_widget
             self.w_aspect_ratio = aspect_ratio_widget
@@ -114,10 +114,10 @@ def demo():
             transformation_widget.setCurrentIndex(1)
             halign_widget.setCurrentIndex(0)
             valign_widget.setCurrentIndex(1)
-    
+
         def changePixmap(self, name):
             self.w.pixmap = getPixmap(name)
-            
+
         def changeAspectRatio(self, i):
             v = Qt.Qt.IgnoreAspectRatio
             if i == 1:
@@ -125,18 +125,20 @@ def demo():
             elif i == 2:
                 v = Qt.Qt.KeepAspectRatioByExpanding
             self.w.setAspectRatioMode(v)
-        
+
         def changeTransformationMode(self, i):
             v = Qt.Qt.FastTransformation
             if i == 1:
                 v = Qt.Qt.SmoothTransformation
             self.w.setTransformationMode(v)
-        
+
         def changeAlignment(self, i):
-            halign = self.w_halign.itemData(self.w_halign.currentIndex()).toInt()[0]
-            valign = self.w_valign.itemData(self.w_valign.currentIndex()).toInt()[0]
+            halign = self.w_halign.itemData(self.w_halign.currentIndex())
+            halign = Qt.from_qvariant(halign, int)
+            valign = self.w_valign.itemData(self.w_valign.currentIndex())
+            valign = Qt.from_qvariant(valign, int)
             self.w.alignment = halign | valign
-        
+
     panel = Qt.QWidget()
     layout=Qt.QGridLayout()
     panel.setLayout(layout)
@@ -149,9 +151,9 @@ def demo():
         sys.exit(app.exec_())
     else:
         return panel
-    
+
 def main():
     return demo()
 
 if __name__ == "__main__":
-    main()
\ No newline at end of file
+    main()
