@@ -32,24 +32,23 @@ __docformat__ = 'restructuredtext'
 import sys
 
 from taurus.qt import Qt
-import taurus.core.util
-import taurus.qt.qtgui.resource
-import qpixmapwidget
+from taurus.core.util.enumeration import Enumeration
+from taurus.qt.qtgui.resource import getPixmap
+from .qpixmapwidget import QPixmapWidget
 
-getPixmap = taurus.qt.qtgui.resource.getPixmap
-
-LedColor  = taurus.core.util.Enumeration("LedColor",  ["BLUE", "GREEN", "RED", "YELLOW", "ORANGE", "MAGENTA", "GRENOBLE", "BLACK", "WHITE"])
-LedStatus = taurus.core.util.Enumeration("LedStatus", ["ON", "OFF"])
-LedSize   = taurus.core.util.Enumeration("LedSize",   [("SMALL", 24), ("LARGE", 48)])
+LedColor  = Enumeration("LedColor",  ["BLUE", "GREEN", "RED", "YELLOW", "ORANGE", "MAGENTA", "GRENOBLE", "BLACK", "WHITE"])
+LedStatus = Enumeration("LedStatus", ["ON", "OFF"])
+LedSize   = Enumeration("LedSize",   [("SMALL", 24), ("LARGE", 48)])
 
     
-class QLed(qpixmapwidget.QPixmapWidget):
+class QLed(QPixmapWidget):
     """A Led"""
     
     DefaultLedPattern = ":leds/images256/led_{color}_{status}.png"
     DefaultLedColor = "green"
     DefaultLedStatus = True
     DefaultLedInverted = False
+    DefaultBlinkingInterval = 0
     
     def __init__(self, parent = None, designMode=False):
         self._ledStatus = self.DefaultLedStatus
@@ -57,13 +56,14 @@ class QLed(qpixmapwidget.QPixmapWidget):
         self._ledPatternName = self.DefaultLedPattern
         self._ledInverted = self.DefaultLedInverted
         self._ledName = self.toLedName()
-        qpixmapwidget.QPixmapWidget.__init__(self, parent)
+        self._timer = None
+        QPixmapWidget.__init__(self, parent)
         self._refresh()
     
     def sizeHint(self):
         if self.layout() is None:
             return Qt.QSize(24, 24)
-        return qpixmapwidget.QPixmapWidget.sizeHint(self)
+        return QPixmapWidget.sizeHint(self)
     
     def minimumSizeHint(self):
         """Overwrite the default minimum size hint (0,0) to be (16,16)
@@ -159,6 +159,10 @@ class QLed(qpixmapwidget.QPixmapWidget):
     def resetLedStatus(self):
         """Resets the led status"""
         self.setLedStatus(self.DefaultLedStatus)
+        
+    def toggleLedStatus(self):
+        """toggles the current status of the led"""
+        self.setLedStatus(not self.getLedStatus())
 
     def getLedInverted(self):
         """Returns if the led is inverted.
@@ -196,6 +200,36 @@ class QLed(qpixmapwidget.QPixmapWidget):
     def resetLedColor(self):
         """Resets the led color"""
         self.setLedColor(self.DefaultLedColor)
+        
+    def setBlinkingInterval(self, interval):
+        """sets the blinking interval (the time between status switching).
+        Set to a nonpositive number for disabling blinking
+        
+        :param interval: (int) the blinking interval in millisecs. Set to 0 for disabling blinking
+        """
+        if interval > 0:
+            if self._timer is None:
+                self._timer = Qt.QTimer(self)
+                self.connect(self._timer, Qt.SIGNAL('timeout()'), self.toggleLedStatus)
+            self._timer.start(interval)
+        else:
+            if self._timer is not None:
+                self._timer.stop()
+                self._timer = None
+        
+    def getBlinkingInterval(self):
+        """returns the blinking interval
+        
+        :return: (int) blinking interval or 0 if blinking is not enabled.
+        """ 
+        if self._timer is None: 
+            return 0
+        else:
+            return self._timer.interval()
+    
+    def resetBlinkingInterval(self):
+        """resets the blinking interval"""
+        self.setBlinkingInterval(self.__class__.DefaultBlinkingInterval)
     
     #: This property holds the led status: False means OFF, True means ON
     #:
@@ -238,6 +272,18 @@ class QLed(qpixmapwidget.QPixmapWidget):
     ledPattern = Qt.pyqtProperty("QString", getLedPatternName, setLedPatternName,
                                  resetLedPatternName, doc="led pattern name")
     
+    #: This property holds the blinking interval in millisecs. 0 means no blinking
+    #:
+    #: **Access functions:**
+    #:
+    #:     * :meth:`QLed.getBlinkingInterval`
+    #:     * :meth:`QLed.setBlinkingInterval`
+    #:     * :meth:`QLed.resetBlinkingInterval`
+    blinkingInterval = Qt.pyqtProperty("int", getBlinkingInterval, setBlinkingInterval,
+                                resetBlinkingInterval, doc="blinking interval")
+    
+    
+    
 class QLedOld(Qt.QLabel):
     ledDirPattern = ":leds/images%(size)d"
 
@@ -345,6 +391,10 @@ def main():
         led.ledColor = color
         led.ledStatus = False
         layout.addWidget(led, i, 1)
+        led = QLed()
+        led.ledColor = color
+        led.blinkingInterval = 500
+        layout.addWidget(led, i, 2)
     w.show()
     
     if owns_app:
