@@ -31,17 +31,15 @@ __all__=['TaurusCurveItemTableModel','CurveItemConf', 'CurveItemConfDlg']
 
 import copy
 
-from taurus.qt import Qt
-from PyQt4 import Qwt5
-import taurus
-from taurus.core import TaurusException
-from taurus.qt.qtcore.mimetypes import TAURUS_MODEL_LIST_MIME_TYPE, TAURUS_ATTR_MIME_TYPE
-from taurus.qt.qtgui.resource import getThemeIcon, getIcon
-from ui.ui_CurveItemConfDlg import Ui_CurveItemConfDlg    
+from taurus.qt import Qt, Qwt5
 from guiqwt.styles import CurveParam, AxesParam, update_style_attr
-from taurus.qt.qtgui.extra_guiqwt.styles import TaurusCurveParam
 from guiqwt.builder import make
 
+import taurus
+from taurus.qt.qtcore.mimetypes import TAURUS_MODEL_LIST_MIME_TYPE, TAURUS_ATTR_MIME_TYPE
+from taurus.qt.qtgui.resource import getThemeIcon, getIcon
+from .ui.ui_CurveItemConfDlg import Ui_CurveItemConfDlg    
+from taurus.qt.qtgui.extra_guiqwt.styles import TaurusCurveParam
 
 AXIS_ID2NAME = {Qwt5.QwtPlot.yLeft:'left', Qwt5.QwtPlot.yRight:'right',
                 Qwt5.QwtPlot.xBottom:'bottom', Qwt5.QwtPlot.xTop:'top'}
@@ -52,6 +50,7 @@ NUMCOLS = 3
 X, Y, TITLE = range(NUMCOLS)
 SRC_ROLE =  Qt.Qt.UserRole + 1
 
+
 class Component(object):
     def __init__(self, src):
         self.display = ''
@@ -216,7 +215,7 @@ class TaurusCurveItemTableModel(Qt.QAbstractTableModel):
             row = index.row()
             curve = self.curves[row]
             column = index.column()
-            value = str(value.toString())
+            value = Qt.from_qvariant(value, str)
             if column == X: 
                 curve.taurusparam.xModel = value
                 curve.x.processSrc(value)
@@ -284,8 +283,10 @@ class TaurusCurveItemTableModel(Qt.QAbstractTableModel):
     def mimeData(self, indexes):
         mimedata = Qt.QAbstractTableModel.mimeData(self, indexes)
         if len(indexes)==1:
-#            mimedata.setData(TAURUS_ATTR_MIME_TYPE, str(self.data(indexes[0]).toString()))
-            mimedata.setText(self.data(indexes[0],role=SRC_ROLE).toString())
+#            data = Qt.from_qvariant(self.data(indexes[0], str)
+#            mimedata.setData(TAURUS_ATTR_MIME_TYPE, data)
+            data = Qt.from_qvariant(self.data(indexes[0], role=SRC_ROLE), str)
+            mimedata.setText(data)
         return mimedata
         #mimedata.setData()
     
@@ -313,8 +314,8 @@ class CurveItemConfDlg(Qt.QWidget):
             nexusWidget = TaurusNeXusBrowser()
             self.ui.tabWidget.addTab(nexusWidget,'NeXus')
         except:
-            import taurus.core.util
-            _logger = taurus.core.util.Logger('CurveItemConfDlg')
+            import taurus.core.util.log
+            _logger = taurus.core.util.log.Logger('CurveItemConfDlg')
             _logger.warning('TaurusNeXusBrowser not available')
             self.traceback()
         
@@ -437,4 +438,4 @@ class CurveItemConfDlg(Qt.QWidget):
 #
 #if __name__ == "__main__":
 #    import sys
-#    test2()    
\ No newline at end of file
+#    test2()    
