@@ -26,17 +26,19 @@
 """
 Extension of :mod:`guiqwt.image`
 """
-__all__=["TaurusImageItem","TaurusRGBImageItem","TaurusTrend2DItem","TaurusTrend2DScanItem"]
+__all__=["TaurusImageItem","TaurusRGBImageItem","TaurusTrend2DItem",
+         "TaurusTrend2DScanItem","TaurusEncodedImageItem"]
 
 from taurus.qt import Qt
 from taurus.qt.qtgui.base import TaurusBaseComponent
 import taurus.core
-from taurus.core.util import ArrayBuffer
+from taurus.core.util.containers import ArrayBuffer
 
 from guiqwt.image import ImageItem, RGBImageItem, XYImageItem, INTERP_NEAREST, INTERP_LINEAR
 
 import numpy
 
+
 class TaurusBaseImageItem(TaurusBaseComponent):
     '''A ImageItem that gets its data from a taurus attribute'''
     def __init__(self, classname):
@@ -54,7 +56,7 @@ class TaurusBaseImageItem(TaurusBaseComponent):
         #... and fire a fake event for initialization
         try:
             value = self.getModelObj().read()
-            self.fireEvent(self, taurus.core.TaurusEventType.Change, value)
+            self.fireEvent(self, taurus.core.taurusbasetypes.TaurusEventType.Change, value)
         except:
             pass
 
@@ -62,14 +64,28 @@ class TaurusBaseImageItem(TaurusBaseComponent):
         if evt_value is None or getattr(evt_value,'value', None) is None:
             self.debug('Ignoring event from %s'%repr(evt_src))
             return
+        
+        #Try to cast if value type is not one supported by guiqwt
+        #see: http://code.google.com/p/guiqwt/issues/detail?id=44 and
+        #     https://sourceforge.net/tracker/?func=detail&atid=484769&aid=3603991&group_id=57612
+        v = evt_value.value
+        if (not isinstance(v, (float, numpy.double, numpy.uint32, numpy.int32, numpy.uint16, numpy.int16, numpy.uint8, numpy.int8, bool)) 
+                and numpy.issubdtype(getattr(v,'dtype', type(v)), int)):  
+            try:
+                v = numpy.int32(v)
+            except OverflowError:
+                self.info("type %s not supported by qwt and cannot be casted to int32. Dropping event"%repr(v.dtype))
+                return
+        
         lut_range = self.get_lut_range() #this is the range of the z axis (color scale)
         if lut_range[0] == lut_range[1]: lut_range = None #if the range was not set, make it None (autoscale z axis)
-        self.set_data(evt_value.value, lut_range=lut_range)
+        self.set_data(v, lut_range=lut_range)
         self.getSignaller().emit(Qt.SIGNAL('dataChanged'))
         p = self.plot()
         if p is not None:
             p.update_colormap_axis(self)
             p.replot()
+        
 
 
 class TaurusImageItem(ImageItem, TaurusBaseImageItem):
@@ -79,6 +95,38 @@ class TaurusImageItem(ImageItem, TaurusBaseImageItem):
         TaurusBaseImageItem.__init__(self, self.__class__.__name__)
 
 
+class TaurusEncodedImageItem(TaurusImageItem):
+    '''A ImageItem that gets its data from a DevEncoded attribute'''
+    def __init__(self, param=None):
+        TaurusImageItem.__init__(self,param=param)
+        
+    def setModel(self, model):
+        #do the standard stuff
+        TaurusBaseComponent.setModel(self, model)
+        #... and fire a fake event for initialization
+        try:
+            format,value = self.codec.decode(self.getModelObj().read())
+            self.fireEvent(self, taurus.core.taurusbasetypes.TaurusEventType.Change, value)
+        except:
+            pass
+
+    def set_data(self, data, lut_range=None, **kwargs):
+        '''reimplementation to decode data before passing it to 
+           TaurusImageItem implementation'''
+        if type(data) == tuple:
+            from taurus.core.util.codecs import CodecFactory
+            codec = CodecFactory().getCodec(data[0])
+            format,decoded_data = codec.decode(data)
+            TaurusImageItem.set_data(self, decoded_data, lut_range=lut_range)
+
+
+class TaurusXYImageItem(XYImageItem, TaurusBaseImageItem):
+    '''A XYImageItem that gets its data from a taurus attribute'''
+    def __init__(self, param=None):
+        XYImageItem.__init__(self, numpy.arange(2), numpy.arange(2), numpy.zeros((2,2)), param=param)
+        TaurusBaseImageItem.__init__(self, self.__class__.__name__)
+
+
 class TaurusRGBImageItem(RGBImageItem, TaurusBaseImageItem):
     '''A RGBImageItem that gets its data from a taurus attribute'''
     def __init__(self, param=None):
@@ -131,7 +179,7 @@ class TaurusTrend2DItem(XYImageItem, TaurusBaseComponent):
         #... and fire a fake event for initialization
         try:
             value = self.getModelObj().read()
-            self.fireEvent(self, taurus.core.TaurusEventType.Change, value)
+            self.fireEvent(self, taurus.core.taurusbasetypes.TaurusEventType.Change, value)
         except:
             pass
 
@@ -417,12 +465,15 @@ def test1():
         
     #define a taurus image
     model1 = 'sys/tg_test/1/short_image_ro'
-    taurusimage = make.image(taurusmodel= model1)
-    taurusrgbimage = make.rgbimage(taurusmodel= 'eval://array([[[ 222, 0, 0], [0, 222, 0]], [[0, 0, 222], [222, 222, 222]]])')
+    model1 = 'sys/tg_test/1/long64_image_ro'
+    #taurusimage = make.image(taurusmodel= model1)
+    #taurusrgbimage = make.rgbimage(taurusmodel= 'eval://array([[[ 222, 0, 0], [0, 222, 0]], [[0, 0, 222], [222, 222, 222]]])')
+    taurusxyimage= make.xyimage(taurusmodel= model1)
+    taurusxyimage.set_xy(numpy.arange(251)*10,numpy.arange(251)*100 )
     
     #define normal image (guiqwt standard)
     data = numpy.random.rand(100,100)
-    image = make.image(data=data)
+    #image = make.image(data=data)
     
     #create a dialog with a plot and add the images
     win = ImageDialog(edit=False, toolbar=True, wintitle="Taurus Cross sections test",
@@ -430,18 +481,19 @@ def test1():
     from taurus.qt.qtgui.extra_guiqwt.tools import TaurusImageChooserTool
     win.add_tool(TaurusImageChooserTool)
     plot = win.get_plot()
-    plot.add_item(taurusimage)
-    plot.add_item(image)
-    plot.add_item(taurusrgbimage)
+#    plot.add_item(taurusimage)
+    plot.add_item(taurusxyimage)
+#    plot.add_item(image)
+#    plot.add_item(taurusrgbimage)
+
 #    win.get_itemlist_panel().show()
     
     #IMPORTANT: connect the cross section plots to the taurusimage so that they are updated when the taurus data changes
-    win.connect(taurusimage.getSignaller(), Qt.SIGNAL("dataChanged"), win.update_cross_sections)
+    #win.connect(taurusimage.getSignaller(), Qt.SIGNAL("dataChanged"), win.update_cross_sections)
     
     win.exec_()
 
-
 if __name__ == "__main__":
-    #test1()
-    taurusImageMain()   
+    test1()
+    #taurusImageMain()   
 
