@@ -31,4 +31,4 @@ from sequenceeditor import TaurusSequencerWidget, TaurusSequencer
 from common import TaurusMacroConfigurationDialog
 from macrodescriptionviewer import TaurusMacroDescriptionViewer
 from dooroutput import DoorOutput, DoorDebug, DoorResult
-from macrobutton import MacroButton
+from macrobutton import MacroButton, MacroButtonAbortDoor
