@@ -31,7 +31,7 @@ import os, sys
 from taurus.qt import Qt
 import PyTango
 
-from taurus.core.util import etree
+from lxml import etree
 from taurus.qt.qtgui.extra_macroexecutor.common import MacroExecutionWindow, MacroComboBox, standardPlotablesFilter
 from taurus import Device
 
@@ -637,6 +637,7 @@ class TaurusSequencerWidget(TaurusWidget):
         if macro is None: return
         data = data[1][0]
         state, range, step, id = str(data["state"]), data["range"], data["step"], data["id"]
+        if id is None: return
         id = int(id)
         if not id in self.macroIds(): return
         macroName = macro.name
