@@ -30,6 +30,7 @@ __all__ = ["ExpDescriptionEditor"]
 from taurus.qt import Qt
 import copy
 import taurus
+import taurus.core
 from taurus.qt.qtgui.base import TaurusBaseWidget
 from taurus.qt.qtcore.tango.sardana.model import SardanaBaseProxyModel, SardanaTypeTreeItem
 
@@ -45,8 +46,13 @@ class SardanaAcquirableProxyModel(SardanaBaseProxyModel):
 #        item = idx.internalPointer()
 #        return 'Acquirable' in item.itemData().interfaces
 
-    ALLOWED_TYPES = ('Motor', 'CTExpChannel') #@todo: add IORegisters too!
-    
+#    ALLOWED_TYPES = ['Motor', 'CTExpChannel', 'ZeroDExpChannel', 'OneDExpChannel',
+#                     'TwoDExpChannel', 'ComChannel', 'IORegister', 'PseudoMotor',
+#                     'PseudoCounter']
+
+    from sardana.sardanadefs import ElementType, TYPE_ACQUIRABLE_ELEMENTS
+    ALLOWED_TYPES = [ElementType[t] for t in TYPE_ACQUIRABLE_ELEMENTS]
+
     def filterAcceptsRow(self, sourceRow, sourceParent):
         sourceModel = self.sourceModel()
         idx = sourceModel.index(sourceRow, 0, sourceParent)
@@ -69,7 +75,6 @@ class ExpDescriptionEditor(Qt.QWidget, TaurusBaseWidget):
         from ui.ui_ExpDescriptionEditor import Ui_ExpDescriptionEditor
         self.ui = Ui_ExpDescriptionEditor()
         self.ui.setupUi(self)
-        BB = Qt.QDialogButtonBox
         self.ui.buttonBox.setStandardButtons(Qt.QDialogButtonBox.Reset | Qt.QDialogButtonBox.Apply)
         newperspectivesDict = copy.deepcopy(self.ui.sardanaElementTree.KnownPerspectives)
         #newperspectivesDict[self.ui.sardanaElementTree.DftPerspective]['model'] = [SardanaAcquirableProxyModel, SardanaElementPlainModel]
@@ -102,7 +107,7 @@ class ExpDescriptionEditor(Qt.QWidget, TaurusBaseWidget):
              
     def getModelClass(self):
         '''reimplemented from :class:`TaurusBaseWidget`'''
-        return taurus.core.TaurusDevice
+        return taurus.core.taurusdevice.TaurusDevice
         
     def onChooseScanDirButtonClicked(self):
         ret = Qt.QFileDialog.getExistingDirectory ( self, 'Choose directory for saving files', self.ui.pathLE.text())
@@ -152,7 +157,7 @@ class ExpDescriptionEditor(Qt.QWidget, TaurusBaseWidget):
         #set a list of available channels
         avail_channels = {}
         for ch_info in door.macro_server.getExpChannelElements().values():
-            avail_channels[ch_info.name] = ch_info.getData()  #@todo: Once it changes in the Pool, this should use full_name!
+            avail_channels[ch_info.full_name] = ch_info.getData()
         self.ui.channelEditor.getQModel().setAvailableChannels(avail_channels)
     
     def _setDirty(self,dirty):
@@ -258,13 +263,31 @@ class ExpDescriptionEditor(Qt.QWidget, TaurusBaseWidget):
         self._setDirty(True)
         
     def createMntGrp(self):
-        '''creates a new Measurement Group'''        
+        '''creates a new Measurement Group'''     
+        
+        if self._localConfig is None:
+            return
+           
         mntGrpName, ok = Qt.QInputDialog.getText(self, "New Measurement Group", 
                                                  "Enter a name for the new measurement Group")
         if not ok: return
-        if self._localConfig is None:
-            return
         mntGrpName = str(mntGrpName)
+        
+        #check that the given name is not an existing pool element
+        ms = self.getModelObj().macro_server
+        poolElementNames = [v.name for v in ms.getElementsWithInterface("PoolElement").values()]
+        while mntGrpName in poolElementNames:
+            Qt.QMessageBox.warning(self, "Cannot create Measurement group", 
+                "The name '%s' already is used for another pool element. Please Choose a different one."%mntGrpName, 
+                Qt.QMessageBox.Ok)
+            mntGrpName, ok = Qt.QInputDialog.getText(self, "New Measurement Group", 
+                                                     "Enter a name for the new measurement Group",
+                                                     Qt.QLineEdit.Normal,
+                                                     mntGrpName)
+            if not ok: return
+            mntGrpName = str(mntGrpName)
+            
+        #check that the measurement group is not already in the localConfig
         if mntGrpName in self._localConfig['MntGrpConfigs']:
             Qt.QMessageBox.warning(self, "%s already exists"%mntGrpName, 
                 'A measurement group named "%s" already exists. A new one will not be created'%mntGrpName)
@@ -316,23 +339,34 @@ class ExpDescriptionEditor(Qt.QWidget, TaurusBaseWidget):
         self._localConfig['ScanFile'] = [v.strip() for v in str(text).split(',')]
         self._setDirty(True)
         
-    def onPreScanSnapshotChanged(self, items):  
-        self._localConfig['PreScanSnapshot'] = [(e.src,e.display) for e in items]
+    def onPreScanSnapshotChanged(self, items):
+        door = self.getModelObj()
+        ms = door.macro_server
+        preScanList = []
+        for e in items:
+            nfo = ms.getElementInfo(e.src)
+            if nfo is None:
+                preScanList.append((e.src,e.display)) 
+            else:
+                preScanList.append((nfo.full_name,nfo.name))
+        self._localConfig['PreScanSnapshot'] = preScanList
         self._setDirty(True)    
-    
-        
-            
-
-
-  
-            
-def demo(model="mydoor"):
-    """Table panels"""
+       
+   
+def demo(model=None):
+    """Experiment configuration"""
     #w = main_ChannelEditor()
-    w = ExpDescriptionEditor(door=model)
+    w = ExpDescriptionEditor()
+    if model is None:
+        from taurus.qt.qtgui.extra_macroexecutor import TaurusMacroConfigurationDialog
+        dialog = TaurusMacroConfigurationDialog(w)
+        accept = dialog.exec_()
+        if accept:
+            model = str(dialog.doorComboBox.currentText())
+    if model is not None:
+        w.setModel(model)
     return w
 
-
 def main():
     import sys
     import taurus.qt.qtgui.application
