@@ -33,10 +33,12 @@ import os
 import traceback
 
 from taurus.qt import Qt
-
-import taurus.core
-import taurus.core.util
-from taurus.qt.qtgui.graphic import TaurusBaseGraphicsFactory, TaurusGraphicsScene, TaurusGraphicsItem
+from taurus.core.util.log import Logger
+from taurus.core.util.singleton import Singleton
+from taurus.core.util.containers import CaselessDict
+from taurus.qt.qtgui.graphic import TaurusBaseGraphicsFactory, \
+    TaurusGraphicsScene, TaurusGraphicsItem, parseTangoUri, \
+    TaurusTextAttributeItem,TaurusTextStateItem
 
 
 LINESTYLE_JDW2QT = { 0: Qt.Qt.SolidLine,
@@ -58,7 +60,7 @@ FILLSTYLE_JDW2QT = { 0: Qt.Qt.NoBrush,
                      10:Qt.Qt.Dense4Pattern,
                      11:Qt.Qt.RadialGradientPattern }
 
-TEXTHINT_JDW2QT = taurus.core.util.CaselessDict({ 
+TEXTHINT_JDW2QT = CaselessDict({ 
     'helvetica'  : Qt.QFont.Helvetica,
     'serif'      : Qt.QFont.Serif,
     'sansserif'  : Qt.QFont.SansSerif,
@@ -68,7 +70,7 @@ TEXTHINT_JDW2QT = taurus.core.util.CaselessDict({
     ''           : Qt.QFont.AnyStyle,})
 
 
-class TaurusJDrawGraphicsFactory(taurus.core.util.Singleton, TaurusBaseGraphicsFactory, taurus.core.util.Logger):
+class TaurusJDrawGraphicsFactory(Singleton, TaurusBaseGraphicsFactory, Logger):
     
     def __init__(self,parent,alias = None, delayed = False):
         """ Initialization. Nothing to be done here for now."""
@@ -80,7 +82,7 @@ class TaurusJDrawGraphicsFactory(taurus.core.util.Singleton, TaurusBaseGraphicsF
 
     def init(self, *args, **kwargs):
         """Singleton instance initialization."""
-        self.call__init__(taurus.core.util.Logger, self.__class__.__name__) 
+        self.call__init__(Logger, self.__class__.__name__) 
         self.call__init__(TaurusBaseGraphicsFactory)
         
     def getZBufferLevel(self):
@@ -106,7 +108,7 @@ class TaurusJDrawGraphicsFactory(taurus.core.util.Singleton, TaurusBaseGraphicsF
                     scene.addItem(item)
             except:
                 self.warning("Unable to add item %s to scene" % str(item))
-                self.warning(traceback.format_exc()) #self.traceback()
+                self.debug("Details:", exc_info=1)
         return scene
     
     def getObj(self,name,params):
@@ -117,7 +119,8 @@ class TaurusJDrawGraphicsFactory(taurus.core.util.Singleton, TaurusBaseGraphicsF
             obj.setZValue(self.incZBufferLevel())
             return obj
         except:
-            self.warning(traceback.format_exc())
+            self.warning("Error fetching object")
+            self.debug("Details:", exc_info=1)
             pass
         return None
     
@@ -186,7 +189,7 @@ class TaurusJDrawGraphicsFactory(taurus.core.util.Singleton, TaurusBaseGraphicsF
         fnt = params.get('font')
         if fnt:
             family,style,size = fnt
-            f = Qt.QFont(family, size, Qt.QFont.Light, False)
+            f = Qt.QFont(family, int(.85*size), Qt.QFont.Light, False)
             f.setStyleHint(TEXTHINT_JDW2QT.get(family, Qt.QFont.AnyStyle))
             f.setStyleStrategy(Qt.QFont.PreferMatch)
             if style == 1:
@@ -209,8 +212,9 @@ class TaurusJDrawGraphicsFactory(taurus.core.util.Singleton, TaurusBaseGraphicsF
         if children:
             for child in children:
                 if child:
+                    #self.info('jdraw.py: "%s".addItem("%s")'%(str(params.get('name')),str(child)))
                     item.addToGroup(child)
-
+        if item._fillStyle: self.set_item_filling(item,expand=True)
         return item
     
     def getSwingObjectObj(self,params):
@@ -245,64 +249,7 @@ class TaurusJDrawGraphicsFactory(taurus.core.util.Singleton, TaurusBaseGraphicsF
     def set_common_params(self,item,params):
         if not item:
             return
-        name = params.get('name')
-
-        visibilitymapper = params.get('visibilitymapper')
-        if not visibilitymapper is None:
-            mapping_type = visibilitymapper['mapping_type']
-            mode = visibilitymapper['mode']
-            default = visibilitymapper['default']
-            map = visibilitymapper['map']
-            item._default = default
-            item._map = map      
-
-        visible = params.get('visible')
-        if not visible is None:
-            item.setVisible(visible)
-
-        extensions = params.get('extensions')
-        if extensions:
-            item._extensions = extensions
-
-        try:
-            getattr(item,'setPen')
-            fg = params.get("foreground", (0,0,0))
-            pen = Qt.QPen(Qt.QColor(fg[0],fg[1],fg[2]))
-            pen.setWidth(params.get("lineWidth", 1))
-            pen.setStyle(LINESTYLE_JDW2QT[params.get("lineStyle", 0)])
-            item.setPen(pen)
-        except AttributeError,ae: 
-            pass
-        except Exception,e:
-            self.warning('jdraw.set_common_params(%s(%s)).(foreground,width,style) failed!: \n\t%s'%(type(item).__name__,name,traceback.format_exc()))
-
-        fillStyle = FILLSTYLE_JDW2QT[params.get('fillStyle', 0)]
-
-        try:
-            getattr(item,'setBrush')
-            brush = Qt.QBrush() 
-            if fillStyle == Qt.Qt.RadialGradientPattern:
-                x1, y1, x2, y2 = params.get('summit')
-                w, h = (x2-x1)/2.0, (y2-y1)/2.0
-                gradient = Qt.QLinearGradient(params.get('gradX1',0)+w,
-                                                 params.get('gradY1',0)+h,
-                                                 params.get('gradX2',0)+w,
-                                                 params.get('gradY2',0)+h)
-                c = params.get('gradC1',(0,0,0))
-                gradient.setColorAt(0,Qt.QColor(c[0],c[1],c[2]))
-                c = params.get('gradC2',(255,255,255))
-                gradient.setColorAt(1,Qt.QColor(c[0],c[1],c[2])) 
-                brush = Qt.QBrush(gradient)
-            else:
-                brush.setStyle(fillStyle)
-            
-            bg = params.get('background',(255,255,255))
-            brush.setColor(Qt.QColor(bg[0],bg[1],bg[2]))
-            item.setBrush(brush)
-        except AttributeError,ae: pass
-        except Exception,e:
-            self.warning('jdraw.set_common_params(%s(%s)).(background,gradient,style) failed!: \n\t%s'%(type(item).__name__,name,traceback.format_exc()))
-
+        item._params = params
         name = params.get('name')  
         if self.alias: 
             for k,v in self.alias.items():
@@ -312,18 +259,89 @@ class TaurusJDrawGraphicsFactory(taurus.core.util.Singleton, TaurusBaseGraphicsF
         setattr(item,'_name',name)
         if name and not self._delayed:
             if isinstance(item, TaurusGraphicsItem):
-                self.debug('TaurusJDrawGraphicsFactory.set_common_params(): %s.setModel(%s)'%(item,name))
+                #self.debug('TaurusJDrawGraphicsFactory.set_common_params(): %s.setModel(%s)'%(item,name))
                 item.setModel(name)
             else:
                 self.debug('TaurusJDrawGraphicsFactory.set_common_params(%s): %s is not a TaurusGraphicsItem'%(name,type(item).__name__))
+
+        visibilitymapper = params.get('visibilitymapper')
+        if not visibilitymapper is None:
+            mapping_type = visibilitymapper['mapping_type']
+            mode = visibilitymapper['mode']
+            default = visibilitymapper['default']
+            item._default = default
+            item._map = visibilitymapper['map']
+
+        visible = params.get('visible')
+        if not visible is None:
+            item.setVisible(visible)
+
+        extensions = params.get('extensions')
+        if extensions:
+            item._extensions = extensions
+
+        if isinstance(item,Qt.QGraphicsTextItem):
+          try:
+            fg = params.get("foreground", (0,0,0))
+            color = Qt.QColor(fg[0],fg[1],fg[2])
+            item.setDefaultTextColor(color)
+          except:
+            self.warning('jdraw.set_common_params(%s(%s)).(foreground,width,style) failed!: \n\t%s'%(type(item).__name__,name,traceback.format_exc()))
+
+        else:
+          try:
+              getattr(item,'setPen')
+              fg = params.get("foreground", (0,0,0))
+              pen = Qt.QPen(Qt.QColor(fg[0],fg[1],fg[2]))
+              pen.setWidth(params.get("lineWidth", 1))
+              pen.setStyle(LINESTYLE_JDW2QT[params.get("lineStyle", 0)])
+              item.setPen(pen)
+          except AttributeError,ae:
+              pass
+          except Exception,e:
+              self.warning('jdraw.set_common_params(%s(%s)).(foreground,width,style) failed!: \n\t%s'%(type(item).__name__,name,traceback.format_exc()))
+
+          fillStyle = FILLSTYLE_JDW2QT[params.get('fillStyle', 0)]
+          item._fillStyle = fillStyle
+          try:
+              if hasattr(item,'brush'):
+                  brush = Qt.QBrush()
+                  if fillStyle == Qt.Qt.RadialGradientPattern:
+                      x1, y1, x2, y2 = params.get('summit')
+                      w, h = (x2-x1)/2.0, (y2-y1)/2.0
+                      gradient = Qt.QLinearGradient(params.get('gradX1',0)+w,
+                                                      params.get('gradY1',0)+h,
+                                                      params.get('gradX2',0)+w,
+                                                      params.get('gradY2',0)+h)
+                      c = params.get('gradC1',(0,0,0))
+                      gradient.setColorAt(0,Qt.QColor(c[0],c[1],c[2]))
+                      c = params.get('gradC2',(255,255,255))
+                      gradient.setColorAt(1,Qt.QColor(c[0],c[1],c[2]))
+                      brush = Qt.QBrush(gradient)
+                  else:
+                      brush.setStyle(fillStyle)
+
+                  bg = params.get('background',(255,255,255))
+                  brush.setColor(Qt.QColor(bg[0],bg[1],bg[2]))
+                  item.setBrush(brush)
+          #except AttributeError,ae: pass
+          except Exception,e:
+              self.warning('jdraw.set_common_params(%s(%s)).(background,gradient,style) failed!: \n\t%s'%(type(item).__name__,name,traceback.format_exc()))
+        #self.debug('Out of TaurusJDrawGraphicsFactory.%s.set_common_params(%s)'%(type(item).__name__,name))
         
-        self.debug('Out of TaurusJDrawGraphicsFactory.%s.set_common_params(%s)'%(type(item).__name__,name))  
+    def set_item_filling(self,item,pattern=Qt.Qt.Dense4Pattern,expand=False):
+        count = 0
+        item._fillStyle = item._fillStyle or pattern
+        if hasattr(item,'brush'):
+            br = item.brush()
+            br.setStyle(item._fillStyle)
+            item.setBrush(br)
+        if expand:
+            for c in item.childItems():
+                if not c._fillStyle:
+                    self.set_item_filling(c,pattern=pattern,expand=True)
+        return
 
 if __name__ == "__main__":
-    import sys
     import jdraw_view
-    app = Qt.QApplication([])
-    gui = jdraw_view.TaurusJDrawSynopticsView()
-    gui.setModel(sys.argv[1])
-    gui.show()
-    sys.exit(app.exec_())
\ No newline at end of file
+    jdraw_view.jdraw_view_main()
