@@ -30,11 +30,7 @@ taurusgraphic.py:
 
 __docformat__ = 'restructuredtext'
 
-import time
-import sys
-import signal
 import re
-from threading import Thread 
 import os
 import subprocess
 import traceback
@@ -43,15 +39,28 @@ import types
 
 import Queue
 
+from taurus import Manager
+from taurus.core.util.containers import CaselessDefaultDict
+from taurus.core.util.log import Logger
+from taurus.core.taurusvalidator import DeviceNameValidator, AttributeNameValidator
+from taurus.core.taurusdevice import TaurusDevice
+from taurus.core.taurusattribute import TaurusAttribute
 from taurus.qt import Qt
-import taurus.core
-from taurus.core import DeviceNameValidator
-import taurus.core.util
-
 from taurus.qt.qtgui.base import TaurusBaseComponent
-from taurus.qt.qtgui.util import QT_ATTRIBUTE_QUALITY_PALETTE, QT_DEVICE_STATE_PALETTE
+from taurus.qt.qtgui.util import (QT_ATTRIBUTE_QUALITY_PALETTE, QT_DEVICE_STATE_PALETTE,
+                                  ExternalAppAction, TaurusWidgetFactory)
 
 
+def parseTangoUri(name):
+    from taurus.core import tango
+    validator = {tango.TangoDevice    : DeviceNameValidator,
+                 tango.TangoAttribute : AttributeNameValidator }
+    try: 
+        params = validator[tango.TangoFactory().findObjectClass(name)]().getParams(name)
+        return (params if 'devicename' in params else None)
+    except: 
+        return None
+
 class TaurusGraphicsUpdateThread(Qt.QThread):
     
     def __init__(self, parent=None, period=3):
@@ -60,7 +69,7 @@ class TaurusGraphicsUpdateThread(Qt.QThread):
             raise RuntimeError("Illegal parent for TaurusGraphicsUpdateThread")
         Qt.QThread.__init__(self, parent)
         self.period = period
-        self.log = taurus.core.util.Logger('TaurusGraphicsUpdateThread')
+        self.log = Logger('TaurusGraphicsUpdateThread')
 
     def _updateView(self,v):
         # The first one is the prefered one because it improves performance
@@ -96,7 +105,7 @@ class TaurusGraphicsUpdateThread(Qt.QThread):
             item_rects = [ i.boundingRect() for i in item ]
             
             for v in p.views():
-                #p.info("emit('updateView')")
+                #p.debug("emit('updateView')")
                 emitter.emit(Qt.SIGNAL("updateView"), v)
             self.sleep(self.period) #This sleep is needed to reduce CPU usage of the application!
             #End of while
@@ -168,40 +177,49 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
      of tuples. Empty tuples will insert separators in the menu.
     '''
     __pyqtSignals__ = ("refreshTree2","graphicItemSelected(QString)","graphicSceneClicked(QPoint)")
+    ANY_ATTRIBUTE_SELECTS_DEVICE = True
+    TRACE_ALL = False
     
     def __init__(self, parent = None, strt = True):
         name = self.__class__.__name__
-        #self.call__init__(taurus.core.util.Logger, name, parent) #Inheriting from Logger caused exceptions in CONNECT
+        #self.call__init__(Logger, name, parent) #Inheriting from Logger caused exceptions in CONNECT
         Qt.QGraphicsScene.__init__(self, parent)
         self.updateQueue = None
         self.updateThread = None
-        self._itemnames = taurus.core.util.CaselessDefaultDict(lambda k:set())
+        self._itemnames = CaselessDefaultDict(lambda k:set())
         self._selection = []
+        self._selectedItems = []
         self.threads = []
         self.pids = []
+        self.panel_launcher = ExternalAppAction(parent.defaultPanelClass().split() if parent else ['taurusdevicepanel'])
         
         try:
-            self.logger = taurus.core.util.Logger(name)
+            self.logger = Logger(name)
             #self.logger.setLogLevel(self.logger.Info)
-            self.debug = lambda l: self.logger.debug(l)
-            self.info = lambda l: self.logger.info(l)
-            self.warning = lambda l: self.logger.warning(l)
-            self.error = lambda l: self.logger.error(l)
+            if not self.TRACE_ALL:
+                self.debug = lambda l: self.logger.debug(l)
+                self.info = lambda l: self.logger.info(l)
+                self.warning = lambda l: self.logger.warning(l)
+            else: self.debug = self.info = self.warning = self.error = lambda l: self.logger.warning(l)
         except:
             print 'Unable to initialize TaurusGraphicsSceneLogger: %s'%traceback.format_exc()
             
         self.setSelectionMark()            
         if strt:self.start()
+        
+    def __del__(self):
+        self.panel_launcher.kill()
+        Qt.QGraphicsScene.__del__(self)
 
     def addItem(self,item):
-        self.debug('addItem(%s)'%item)
+        #self.debug('addItem(%s)'%item)
         def expand(i):
             name = str(getattr(i,'_name','')).lower()
             if name: 
                 self._itemnames[name].add(i)
-                self.debug('addItem(%s): %s'%(name,i))
+                #self.debug('addItem(%s): %s'%(name,i))
             if isinstance(i,Qt.QGraphicsItemGroup):
-                for j in i.children():
+                for j in i.childItems():
                     expand(j)
         expand(item)
         Qt.QGraphicsScene.addItem(self,item)
@@ -213,34 +231,70 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
         if flags is None: Qt.QGraphicsScene.addWidget(self,item)
         else: Qt.QGraphicsScene.addWidget(self,item,flags)
         
-    def getItemByName(self,item_name):
+    def getItemByName(self,item_name,strict=None):
         """
-        Returns a list with all items matching a given name
+        Returns a list with all items matching a given name.
+        
+        :param strict: (bool or None) controls whether full_name (strict=True) or only device name (False) must match
+        
+        :return: (list) items
         """
+        strict = (not self.ANY_ATTRIBUTE_SELECTS_DEVICE) if strict is None else strict
+        alnum = '(?:[a-zA-Z0-9-_\*]|(?:\.\*))(?:[a-zA-Z0-9-_\*]|(?:\.\*))*'
         target = str(item_name).strip().split()[0].lower().replace('/state','') #If it has spaces only the first word is used
-        if DeviceNameValidator().getParams(target): target+='(/state)?'
-        #if not target.endswith('$'): target+='$' #Device names should match also its attributes?
+        #Device names should match also its attributes or only state?
+        if not strict and AttributeNameValidator().getParams(target):
+            target = target.rsplit('/',1)[0]
+        if DeviceNameValidator().getParams(target): 
+            if strict: target+='(/state)?'
+            else: target+='(/'+alnum+')?'
+        if not target.endswith('$'): target+='$' 
         result = []
         for k in self._itemnames.keys():
             if re.match(target.lower(),k.lower()):
                 #self.debug('getItemByName(%s): _itemnames[%s]: %s'%(target,k,self._itemnames[k]))
                 result.extend(self._itemnames[k])
         return result
+            
+    def getItemByPosition(self,x,y):
+        """ This method will try first with named objects; if failed then with itemAt """
+        pos = Qt.QPointF(x,y)
+        itemsAtPos = []
+        for z,o in sorted((i.zValue(),i) for v in self._itemnames.values() for i in v if i.contains(pos)):
+            if not hasattr(o,'getExtensions'):
+                self.debug('getItemByPosition(%d,%d): adding Qt primitive %s'%(x,y,o))
+                itemsAtPos.append(o)
+            elif not o.getExtensions().get('noSelect'):
+                self.debug('getItemByPosition(%d,%d): adding GraphicsItem %s'%(x,y,o))
+                itemsAtPos.append(o)
+            else: self.debug('getItemByPosition(%d,%d): object ignored, %s'%(x,y,o))
+        if itemsAtPos:
+            obj = itemsAtPos[-1]
+            return self.getTaurusParentItem(obj) or obj
+        else: 
+            #return self.itemAt(x,y)
+            self.warning('getItemByPosition(%d,%d): no items found!'%(x,y))
+            return None
+            
+    def getItemClicked(self,mouseEvent):
+        pos = mouseEvent.scenePos()
+        x,y = pos.x(),pos.y()
+        self.emit(Qt.SIGNAL("graphicSceneClicked(QPoint)"),Qt.QPoint(x,y))
+        obj = self.getItemByPosition(x,y)
+        #self.debug('mouse clicked on %s(%s) at (%s,%s)'%(type(obj).__name__,getattr(obj,'_name',''),x,y))
+        return obj
 
     def mousePressEvent(self,mouseEvent):
-        #self.debug('In TaurusGraphicsScene.mousePressEvent(%s))'%str(mouseEvent.button()))
+        #self.debug('In TaurusGraphicsScene.mousePressEvent(%s,%s))'%(str(type(mouseEvent)),str(mouseEvent.button())))
         try: 
-            x = mouseEvent.scenePos().x()
-            y = mouseEvent.scenePos().y()
-            self.emit(Qt.SIGNAL("graphicSceneClicked(QPoint)"),Qt.QPoint(x,y))
-            obj = self.itemAt(x,y)
+            obj = self.getItemClicked(mouseEvent)
             obj_name = getattr(obj,'_name', '')
-            self.debug('mouse clicked on %s (%s,%s)' , type(obj).__name__,x,y)
-            
+            if not obj_name and isinstance(obj,Qt.QGraphicsTextItem): obj_name = obj.toPlainText()
             if (mouseEvent.button() == Qt.Qt.LeftButton):
-                self.selectGraphicItem(obj_name) # A null obj_name should deselect all, we don't send obj because we want all similar to be matched
-                self.emit(Qt.SIGNAL("graphicItemSelected(QString)"),obj_name) # A null obj_name should deselect all
-                
+                ## A null obj_name should deselect all, we don't send obj because we want all similar to be matched                
+                if self.selectGraphicItem(obj_name):
+                    self.debug(' => graphicItemSelected(QString)(%s)'%obj_name)
+                    self.emit(Qt.SIGNAL("graphicItemSelected(QString)"),obj_name) # A null obj_name should deselect all
             def addMenuAction(menu,k,action,last_was_separator=False):
                 try:
                     if k:
@@ -255,14 +309,15 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
                 except Exception,e: 
                     self.warning('Unable to add Menu Action: %s:%s'%(k,e))                    
                 return last_was_separator
-                
             if (mouseEvent.button() == Qt.Qt.RightButton):
                 ''' This function is called when right clicking on TaurusDevTree area. A pop up menu will be shown with the available options. '''
-                self.debug('RightButton Mouse Event on %s (%s,%s)',obj_name,x,y)
+                self.debug('RightButton Mouse Event on %s'%(obj_name))
                 if isinstance(obj,TaurusGraphicsItem) and (obj_name or obj.contextMenu() or obj.getExtensions()):
                     menu = Qt.QMenu(None)#self.parent)    
                     last_was_separator = False
-                    if obj_name: menu.addAction(obj_name)
+                    if obj_name: 
+                        #menu.addAction(obj_name)
+                        addMenuAction(menu,obj_name,lambda x=obj_name: self.panel_launcher.actionTriggered([x]))
                     if obj.contextMenu():
                         if obj_name: 
                             menu.addSeparator()
@@ -275,14 +330,23 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
                             addMenuAction(menu,'Execute',lambda d,x=obj: self.getShellCommand(x))
                         if obj.getExtensions().get('className'):
                             self.debug('launching className extension object')
-                            addMenuAction(menu,obj.getExtensions().get('className'),lambda d,x=obj: self.getClassName(x))                     
+                            addMenuAction(menu,obj.getExtensions().get('className'),lambda d,x=obj: self.getClassName(x))
                     if not menu.isEmpty():
                         menu.exec_(Qt.QPoint(mouseEvent.screenPos().x(),mouseEvent.screenPos().y()))
                     del menu
         except Exception,e:
-            self.error( traceback.format_exc())            
+            self.error( traceback.format_exc())
             
+    def mouseDoubleClickEvent(self,event):
+        try:
+            obj = self.getItemClicked(event)
+            obj_name = getattr(obj,'_name', '')
+            if obj_name: self.panel_launcher.actionTriggered([obj_name])
+        except:
+            self.error( traceback.format_exc())
+        
     def launchProcess(self,process):
+        """ This method is DEPRECATED, use taurus.qt.qtgui.util.ExternalAppAction instead! """
         if not hasattr(self,'ChildrenProcesses'): self.ChildrenProcesses = {}
         if process in self.ChildrenProcesses: 
             self.warning( 'Process %s is already running!'%process)
@@ -291,6 +355,7 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
         return
         
     def killProcess(self,regexp):
+        """ This method is DEPRECATED, use taurus.qt.qtgui.util.ExternalAppAction instead! """
         if '*' in regexp and not '.*' in regexp:
             regexp = regexp.replace('*','.*')
         for name,process in self.ChildrenProcesses.iteritems():
@@ -305,25 +370,40 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
         A blue circle is drawn around the matching item name.
         If the item_name is empty, or it is a reserved keyword, or it has the "noSelect" extension, then the blue circle is removed from the synoptic.
         """      
-        #self.info('In TaurusGraphicsScene.selectGraphicItem(%s))',item_name)
+        #self.debug('In TaurusGraphicsScene.selectGraphicItem(%s))'%item_name)
         retval = False
-        self.clearSelection()
+        selected = [str(getattr(item,'_name',item)) for item in self._selectedItems if item]
+        if selected:
+            iname = str(getattr(item_name,'_name',item_name))
+            #self.debug('In TauGraphicsScene.selectGraphicItem(%s): already selected: %s'%(iname,selected))
+            if not iname.strip():
+                self.clearSelection()
+                return False
+            elif any(iname not in i for i in selected): 
+                self.clearSelection()
+            else: 
+                self.debug('In TauGraphicsScene.selectGraphicItem(%s): already selected!'%item_name)
+                return True
         if any(isinstance(item_name,t) for t in (TaurusGraphicsItem,Qt.QGraphicsItem)):
             if not getattr(item_name,'_name', ''): 
-                self.warning('In TauGraphicsScene.selectGraphicItem(%s): item name not found.'%item_name)
+                self.debug('In TauGraphicsScene.selectGraphicItem(%s): item name not found.'%item_name)
                 return False
             items = [item_name]
         else:
             from taurus.qt.qtgui.graphic import jdraw_parser
             if not item_name or (str(item_name).startswith('JD') and str(item_name) in jdraw_parser.reserved):
-                self.warning('In TauGraphicsScene.selectGraphicItem(%s): item name not found or name is a reserved keyword.'%item_name)
+                self.debug('In TauGraphicsScene.selectGraphicItem(%s): item name not found or name is a reserved keyword.'%item_name)
                 return False
-            items = self.getItemByName(item_name) or []       
+            items = self.getItemByName(item_name) or []
+            items = [i for i in items if self.getTaurusParentItem(i) not in (items+self._selectedItems)]
             self.debug('In TaurusGraphicsScene.selectGraphicItem(%s)): matched %d items'%(item_name,len(items)))
 
         for item in items:
             try:
-                if isinstance(item,TaurusGraphicsItem) and item.getExtensions().get('noSelect'):
+                if ( (isinstance(item,TaurusGraphicsItem) and item.getExtensions().get('noSelect'))
+                    or (item in self._selection)
+                    #or (item in tangoGroup)
+                    ):
                     continue
                 x,y = item.x(),item.y() 
                 rect = item.boundingRect()
@@ -334,12 +414,13 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
                     x,y =  rx,ry #If the object is in the corner it will be also 0
                 w,h= rect.width(),rect.height()
                 if x<0 or y<0: 
-                    self.warning('Cannot draw SelectionMark for %s(%s)(%s,%s) in a negative position (%f,%f)' % (type(item).__name__,item._name,w,h,x,y))
+                    self.debug('Cannot draw SelectionMark for %s(%s)(%s,%s) in a negative position (%f,%f)' % (type(item).__name__,item._name,w,h,x,y))
                 else:
                     if type(item) in (TaurusTextAttributeItem,TaurusTextStateItem) and isinstance(self.getSelectionMark(),Qt.QGraphicsPixmapItem):
                         x,y,w,h = x-20,y,20,20
                     self.drawSelectionMark(x,y,w,h)
                     self.debug('> Moved the SelectionMark to item %s(%s)(%s,%s) at %f,%f' % (type(item).__name__,item._name,w,h,x,y))
+                if item not in self._selectedItems: self._selectedItems.append(item)
                 retval = True
             except Exception,e:
                 self.warning('selectGraphicsItem(%s) failed! %s' % (getattr(item,'_name',item),str(e)))
@@ -348,11 +429,12 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
         return retval
 
     def clearSelection(self):
-        self.debug('In clearSelection([%d])'%len(self._selection))
+        #self.debug('In clearSelection([%d])'%len(self._selectedItems))
         for i in self._selection:
             i.hide()
             self.removeItem(i)
         self._selection = []
+        self._selectedItems = []
         self.updateSceneViews()
 
     def setSelectionMark(self,picture=None,w=10,h=10):
@@ -363,7 +445,7 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
         If no picture is provided, a blue ellipse will be drawn around the selected object.
         h/w will be used for height/width of the drawn object.
         """
-        self.debug('In setSelectionMark(%s,%d,%d)'%(picture,w,h))
+        #self.debug('In setSelectionMark(%s,%d,%d)'%(picture,w,h))
         if picture is None: self.SelectionMark = None #Reset of previous icon generators
         else: self.SelectionMark = (lambda p=picture,x=w,y=h:self.getSelectionMark(p,x,y))
         return self.SelectionMark
@@ -410,7 +492,7 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
         If h or w has a value the mark is drawn in the center of the region ((x,y)(x+w,y+h))
         """        
         #self.debug('%s has parent %s' % (item_name,getattr(item.parentItem(),'_name','ung')  if item.parentItem() else 'None'))
-        self.debug('drawSelectionMark(): center and width,height are: (%d,%d),(%d,%d)' % (x,y,w,h))
+        #self.debug('drawSelectionMark(): center and width,height are: (%d,%d),(%d,%d)' % (x,y,w,h))
 
         mark = self.getSelectionMark()
         self._selection.append(mark)
@@ -436,7 +518,7 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
                     x,y = x-.5*w,y-.5*h
             else: 
                 w,h = [.5*t for t in MAX_CIRCLE_SIZE]
-                self.debug('drawSelectionMark(): center and width,height are: (%d,%d),(%d,%d)' % (x,y,w,h))
+                #self.debug('drawSelectionMark(): center and width,height are: (%d,%d),(%d,%d)' % (x,y,w,h))
             mark.setRect(*bound((x,y,w*2,h*2)))
             #mark.setRect(x,y,w*2,h*2)
         elif isinstance(mark,Qt.QGraphicsPixmapItem):
@@ -483,7 +565,7 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
         return
 
     def getClass(self,clName,clParam,objName,standAlone=False):
-        #self.info('getClass(%s,%s,%s)'%(clName,clParam,objName))
+        #self.debug('getClass(%s,%s,%s)'%(clName,clParam,objName))
         if clName in globals():
             myclass = globals()[clName]
         elif clName in locals():
@@ -493,7 +575,7 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
                 myclass = getattr(Qt,clName)
             except:
                 try:
-                    wf = taurus.qt.qtgui.util.TaurusWidgetFactory()
+                    wf = TaurusWidgetFactory()
                     myclass = wf.getTaurusWidgetClass(clName)
                 except:
                     self.warning( "The class ",clName, "can not be found!\n" + '-'*80)
@@ -516,14 +598,38 @@ class TaurusGraphicsScene(Qt.QGraphicsScene):
             obj.setVisible(True)
             obj.exec_()
         return
+    
+    @staticmethod
+    def getTaurusParentItem(item,top=True):
+        """ Searches within a group hierarchy and returns a parent Taurus component or None if no parent TaurusBaseComponent 
+            is found."""
+        if item is None: return None
+        first,p,next= None,item.parentItem(),None
+        while p:
+            if isinstance(p, TaurusGraphicsItem):
+                if first is None: 
+                    first = p
+                    if not top: break
+                elif str(p.getModel())!=str(first.getModel()):
+                    break
+                else: first = p
+            p = p.parentItem()
+        return first
+        
+    def getAllChildren(self,item,klass=None):
+        """ Returns all children elements, filtering by klass if wanted """
+        result = []
+        try: 
+            children = item.childItems()
+            result.extend(c for c in children if not klass or isinstance(c,klass))
+            result.extend(c.childItems() for c in children if not klass or isinstance(c,klass))
+        except: pass
+        return result
 
     def getTaurusDevicePanel(self,obj,standAlone=False):
         try:
-            try:
-                from taurusDevicePanel import taurusDevicePanel
-            except:
-                from taurusPanel.taurusDevicePanel import taurusDevicePanel
-            nameclass = taurusDevicePanel()
+            from taurus.qt.qtgui.panel import TaurusDevicePanel
+            nameclass = TaurusDevicePanel()
             name = "TaurusDevicePanel"
             nameclass.setModel(obj._name)
             nameclass.setSpectraAtkMode(True)
@@ -569,19 +675,27 @@ class TaurusGraphicsItem(TaurusBaseComponent):
     """Base class for all Taurus Graphics Items"""
     
     def __init__(self, name = None, parent = None):
-        self._name = name or self.__class__.__name__ #srubio@cells.es: modified to store ._name since initialization (even if a model is not set)
-        self.call__init__(TaurusBaseComponent, name, parent)
-        self._name = None #@todo A set/getGraphicName getter/setter should be implemented
+        self.call__init__(TaurusBaseComponent, name, parent) #<- log created here
+        #self.debug('TaurusGraphicsItem(%s,%s)' % (name,parent))
+        self.setName(name)
         self._currFgBrush = None
         self._currBgBrush = None
         self._currText = None
-        self._currHtmlText = None        
+        self._currHtmlText = None
         self._map = None
         self._default = None
         self._visible = None
-        self.getExtensions()        
+        #self.getExtensions() <= It must be called AFTER set_common_params() in getGraphicsItem()
         self._contextMenu = []
         
+    def setName(self,name):
+        #print 'In %s.setName(%s)' % (self.__class__.__name__,name)
+        name = str(name or self.__class__.__name__)
+        self._name = name#srubio@cells.es: modified to store ._name since initialization (even if a model is not set)
+        
+    def getName(self):
+        return self._name
+        
     def setContextMenu(self,menu):
         '''Context Menu must be a list of tuples (ActionName,ActionMethod), empty tuples insert separators between options.'''
         self._contextMenu = menu
@@ -603,9 +717,12 @@ class TaurusGraphicsItem(TaurusBaseComponent):
         self.noPrompt = self._extensions.get('noPrompt',False)
         self.standAlone = self._extensions.get('standAlone',False)
         self.noTooltip = self._extensions.get('noTooltip',False)
-        self.ignoreRepaint = self._extensions.get('ignoreRepaint',False)            
-        self.setToolTip('' if self.noTooltip else str(self._name))
-        self.debug('getExtensions(): %s'%self._extensions)
+        self.ignoreRepaint = self._extensions.get('ignoreRepaint',False)
+        self.setName(self._extensions.get('name',self._name))
+        tooltip = '' if (self.noTooltip or self._name==self.__class__.__name__ or self._name is None) else str(self._name)
+        #self.debug('setting %s.tooltip = %s'%(self._name,tooltip))
+        self.setToolTip(tooltip)
+        #self.debug('%s.getExtensions(): %s'%(self._name,self._extensions))
         return self._extensions
             
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
@@ -613,9 +730,10 @@ class TaurusGraphicsItem(TaurusBaseComponent):
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 
     def setModel(self,model):
-        #self.info('In %s.setModel(%s)'%(type(self).__name__,model))
-        self._name = str(model)
-        if taurus.core.TaurusManager().findObjectClass(self._name) == taurus.core.tango.TangoDevice:
+        #self.debug('In %s.setModel(%s)'%(type(self).__name__,model))
+        self.setName(model)
+        
+        if issubclass(Manager().findObjectClass(self._name), TaurusDevice):
             model = self._name+'/state'
         TaurusBaseComponent.setModel(self, model)        
         
@@ -637,12 +755,13 @@ class TaurusGraphicsItem(TaurusBaseComponent):
             in the style."""
         if self.scene():
             self.scene().updateSceneItem(self)
-            
-    def getDisplayValue(self):
-        attrvalue = self.getModelValueObj()
-        if not attrvalue:
-            return self.getNoneValue()
-        return str(attrvalue.value)            
+
+    #getDisplayValue is already (and better) implemented in TaurusBaseComponent
+    #def getDisplayValue(self):
+        #attrvalue = self.getModelValueObj()
+        #if not attrvalue:
+            #return self.getNoneValue()
+        #return str(attrvalue.value)            
 
     def isReadOnly(self):
         return True
@@ -651,7 +770,7 @@ class TaurusGraphicsItem(TaurusBaseComponent):
         return self.log_name + "(" + self.modelName + ")"
 
     def getModelClass(self):
-        return taurus.core.TaurusAttribute
+        return TaurusAttribute
 
 class TaurusGraphicsAttributeItem(TaurusGraphicsItem):
     """
@@ -675,7 +794,7 @@ class TaurusGraphicsAttributeItem(TaurusGraphicsItem):
     def updateStyle(self):
         v = self.getModelValueObj()
         self._currText = self.getDisplayValue()
-        
+        #self.debug('In TaurusGraphicsAttributeItem(%s).updateStyle(%s)'%(self.getName(),self._currText))
         if self.getShowQuality():
             try:
                 quality = None
@@ -723,21 +842,31 @@ class TaurusGraphicsStateItem(TaurusGraphicsItem):
                 if self.getModelObj().getType() == PyTango.ArgType.DevState:
                     bg_brush, fg_brush = QT_DEVICE_STATE_PALETTE.qbrush(v.value)
                 elif self.getModelObj().getType() == PyTango.ArgType.DevBoolean:
-                    bg_brush, fg_brush = QT_DEVICE_STATE_PALETTE.qbrush((PyTango.DevState.FAULT,PyTango.DevState.ON)[v.value])                    
+                    bg_brush, fg_brush = QT_DEVICE_STATE_PALETTE.qbrush((PyTango.DevState.FAULT,PyTango.DevState.ON)[v.value])
                 elif self.getShowQuality():
                     bg_brush, fg_brush = QT_ATTRIBUTE_QUALITY_PALETTE.qbrush(v.quality)            
                 if None not in (bg_brush,fg_brush):
                     self._currBgBrush = bg_brush
                     self._currFgBrush = fg_brush
+                    #If there's no filling, applying background brush to foreground
+                    if Qt.Qt.NoBrush!=getattr(self,'_fillStyle',Qt.Qt.NoBrush):
+                        #self.debug('In TaurusGraphicsStateItem(%s).updateStyle(%s): switching background to foreground'%(self._name,self._currText))
+                        self._currFgBrush = bg_brush
                     if self._currText: self._currHtmlText = '<p style="color:%s">%s</p>' % (self._currBgBrush.color().name(),self._currText)
             except:
                 self.warning('In TaurusGraphicsStateItem(%s).updateStyle(%s): colors failed!'%(self._name,self._currText))
                 self.warning(traceback.format_exc())
                 
-        states = {'ON':0,'OFF':1,'CLOSE':2,'OPEN':3,'INSERT':4,'EXTRACT':5,'MOVING':6,'STANDBY':7,'FAULT':8,'INIT':9,'RUNNING':10,'ALARM':11,'DISABLE':12,'UNKNOWN':13}
+        states = {
+            'ON':0,'OFF':1,'CLOSE':2,'OPEN':3,
+            'INSERT':4,'EXTRACT':5,'MOVING':6,
+            'STANDBY':7,'FAULT':8,'INIT':9,
+            'RUNNING':10,'ALARM':11,'DISABLE':12,
+            'UNKNOWN':13
+            }
         #Parsing _map to manage visibility (a list of values for which the item is visible or not)
         if v and not self._map is None and self._currText in states:
-            #self.info('In TaurusGraphicsStateItem.updateStyle(): mapping %s'%self._currText)
+            #self.debug('In TaurusGraphicsStateItem.updateStyle(): mapping %s'%self._currText)
             if states[self._currText] == self._map[1]:
                 self.setVisible(self._map[2])
                 self._visible = self._map[2]
@@ -783,7 +912,6 @@ class TaurusGroupStateItem(Qt.QGraphicsItemGroup, TaurusGraphicsStateItem):
 
     def paint(self,painter,option,widget):
         Qt.QGraphicsItemGroup.paint(self,painter,option,widget)
-
         
 class TaurusPolygonStateItem(Qt.QGraphicsPolygonItem,TaurusGraphicsStateItem):
     
@@ -839,6 +967,7 @@ class TaurusTextAttributeItem(Qt.QGraphicsTextItem, TaurusGraphicsAttributeItem,
         self.call__init__(TaurusGraphicsAttributeItem, name, parent)
         
     def paint(self,painter,option,widget):
+        #self.debug('TaurusTextAttributeItem(%s,%s,%s)'%(self.getName(),self._currText,self._currHtmlText))
         if self._currHtmlText:
             self.setHtml(self._currHtmlText)
         else:
@@ -846,8 +975,6 @@ class TaurusTextAttributeItem(Qt.QGraphicsTextItem, TaurusGraphicsAttributeItem,
         Qt.QGraphicsTextItem.paint(self,painter,option,widget)
 
 
-import taurus.core
-
 TYPE_TO_GRAPHICS = { 
     None : { "Rectangle"      : Qt.QGraphicsRectItem,
              "RoundRectangle" : Qt.QGraphicsRectItem,
@@ -859,7 +986,7 @@ TYPE_TO_GRAPHICS = {
              "SwingObject"    : Qt.QGraphicsRectItem, 
              "Image"          : Qt.QGraphicsPixmapItem, },
              
-    taurus.core.TaurusDevice : { "Rectangle"      : TaurusRectStateItem,
+    TaurusDevice : { "Rectangle"      : TaurusRectStateItem,
                            "RoundRectangle" : TaurusRectStateItem,
                            "Ellipse"        : TaurusEllipseStateItem,
                            "Polyline"       : TaurusPolygonStateItem, 
@@ -869,7 +996,7 @@ TYPE_TO_GRAPHICS = {
                            "SwingObject"    : TaurusTextAttributeItem,
                            "Image"          : Qt.QGraphicsPixmapItem, },
 
-    taurus.core.TaurusAttribute : { "Rectangle"      : TaurusRectStateItem,
+    TaurusAttribute : { "Rectangle"      : TaurusRectStateItem,
                            "RoundRectangle" : TaurusRectStateItem,
                            "Ellipse"        : TaurusEllipseStateItem,
                            "Polyline"       : TaurusPolygonStateItem, 
@@ -922,8 +1049,8 @@ class TaurusBaseGraphicsFactory:
     def getGraphicsClassItem(self,cls,type_):
         ncls = cls
         try:
-            if   issubclass(cls, taurus.core.TaurusDevice):    ncls = taurus.core.TaurusDevice
-            elif issubclass(cls, taurus.core.TaurusAttribute): ncls = taurus.core.TaurusAttribute
+            if   issubclass(cls, TaurusDevice):    ncls = TaurusDevice
+            elif issubclass(cls, TaurusAttribute): ncls = TaurusAttribute
         except:
             pass
         ncls = TYPE_TO_GRAPHICS.get(ncls,TYPE_TO_GRAPHICS.get(None)).get(type_)
@@ -938,14 +1065,23 @@ class TaurusBaseGraphicsFactory:
                 params[self.getNameParam()] = name
         cls = None
         if '/' in name:
+            #replacing Taco identifiers in %s'%name
+            if name.lower().startswith('tango:') and (name.count('/')==2 or not 'tango:/' in name.lower()): 
+                nname = name.split(':',1)[-1]
+                params[self.getNameParam()] = name = nname
             if name.lower().endswith('/state'): name = name.rsplit('/',1)[0]
-            cls = taurus.core.TaurusManager().findObjectClass(name)
+            cls = Manager().findObjectClass(name)
         else: 
             if name: self.debug('%s does not match a tango name'%name)
         klass = self.getGraphicsClassItem(cls, type_)
         item = klass()
         ## It's here were Attributes are subscribed
         self.set_common_params(item,params)
+        if hasattr(item,'getExtensions'):
+            item.getExtensions() #<= must be called here to take extensions from params
+        if 'text' in klass.__name__.lower():
+            #print '\tadjusting %s font size'%klass.__name__
+            item.scale(.8,.8)
         return item
 
     def getNameParam(self):
@@ -959,14 +1095,3 @@ class TaurusBaseGraphicsFactory:
            Overwrite has necessary."""
         pass
     
-#if __name__ == "__main__":
-#    import sys
-#    app = Qt.QApplication(sys.argv)
-#    try:
-#        from taurusDevicePanel import *
-#    except:
-#        from taurusPanel.taurusDevicePanel import *
-#    cos = taurusDevicePanel()
-#    print "ala ma kota"
-#    cos.show()
-#    sys.exit(app.exec_())
