@@ -3,21 +3,21 @@
 #############################################################################
 ##
 ## This file is part of Taurus, a Tango User Interface Library
-## 
+##
 ## http://www.tango-controls.org/static/taurus/latest/doc/html/index.html
 ##
 ## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
-## 
+##
 ## Taurus is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU Lesser General Public License as published by
 ## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
-## 
+##
 ## Taurus is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU Lesser General Public License for more details.
-## 
+##
 ## You should have received a copy of the GNU Lesser General Public License
 ## along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
 ##
@@ -34,7 +34,6 @@ __docformat__ = 'restructuredtext'
 
 from taurus.qt import Qt
 
-from taurus.qt.qtcore.model import *
 from taurus.qt.qtgui.util import ActionFactory
 from taurus.qt.qtgui.resource import getIcon, getThemeIcon
 from taurus.qt.qtgui.base import TaurusBaseWidget
@@ -56,7 +55,7 @@ class BaseToolBar(Qt.QToolBar):
 
 class FilterToolBar(BaseToolBar):
     """Internal widget providing quick filter to be placed in a _QToolArea"""
-    
+
     def __init__(self, view=None, parent=None, designMode=False):
         BaseToolBar.__init__(self, name="Taurus filter toolbar", view=view,
                              parent=parent, designMode=designMode)
@@ -67,6 +66,9 @@ class FilterToolBar(BaseToolBar):
         Qt.QObject.connect(filterLineEdit,
                            Qt.SIGNAL("textChanged(const QString &)"),
                            self.onFilterChanged)
+        Qt.QObject.connect(filterLineEdit,
+                           Qt.SIGNAL("textEdited(const QString &)"),
+                           self.onFilterEdited)
         self.addWidget(filterLineEdit)
 
         af = ActionFactory()
@@ -76,26 +78,33 @@ class FilterToolBar(BaseToolBar):
                                           triggered=self.onClearFilter)
         self.addAction(self._clearFilterAction)
 
-    def filterLineEdit(self):
+    def getFilterLineEdit(self):
         return self._filterLineEdit
-        
+
     def onClearFilter(self):
-        self.filterLineEdit().setText("")
+        self.getFilterLineEdit().setText("")
         self.emit(Qt.SIGNAL("clearFilterTriggered"))
-        
+
     def onFilterChanged(self, text=None):
-        text = text or self.filterLineEdit().text()
+        text = text or self.getFilterLineEdit().text()
         self.emit(Qt.SIGNAL("filterChanged"), text)
 
+    def onFilterEdited(self, text=None):
+        text = text or self.getFilterLineEdit().text()
+        self.emit(Qt.SIGNAL("filterEdited"), text)
+    
+    def setFilterText(self, text):
+        self.getFilterLineEdit().setText(text)
+
 
 class EditorToolBar(BaseToolBar):
     """Internal widget to be placed in a _QToolArea providing buttons for
     moving, adding and removing items from a view based widget"""
-    
+
     def __init__(self, view=None, parent=None, designMode=False):
         BaseToolBar.__init__(self, name="Taurus editor toolbar", view=view,
                              parent=parent, designMode=designMode)
-        
+
         af = ActionFactory()
         self._addAction = af.createAction(self, "New item",
                                           icon=getThemeIcon("list-add"),
@@ -128,7 +137,7 @@ class EditorToolBar(BaseToolBar):
         self.addAction(self._moveDownAction)
         self.addAction(self._moveBottomAction)
         #self.setStyleSheet("QWidget {background : red; }")
-    
+
     def onAdd(self):
         self.emit(Qt.SIGNAL("addTriggered"))
 
@@ -137,7 +146,7 @@ class EditorToolBar(BaseToolBar):
 
     def onMoveTop(self):
         self.emit(Qt.SIGNAL("moveTopTriggered"))
-    
+
     def onMoveUp(self):
         self.emit(Qt.SIGNAL("moveUpTriggered"))
 
@@ -149,7 +158,7 @@ class EditorToolBar(BaseToolBar):
 
 
 class SelectionToolBar(BaseToolBar):
-    
+
     def __init__(self, view=None, parent=None, designMode=False):
         BaseToolBar.__init__(self, name="Taurus selection toolbar", view=view,
                              parent=parent, designMode=designMode)
@@ -166,7 +175,7 @@ class SelectionToolBar(BaseToolBar):
 
         self.addAction(self._selectAllAction)
         self.addAction(self._clearSelectionAction)
-    
+
     def onSelectAll(self):
         self.emit(Qt.SIGNAL("selectAllTriggered"))
 
@@ -175,7 +184,7 @@ class SelectionToolBar(BaseToolBar):
 
 
 class RefreshToolBar(BaseToolBar):
-    
+
     def __init__(self, view=None, parent=None, designMode=False):
         BaseToolBar.__init__(self, name="Taurus refresh toolbar", view=view,
                              parent=parent, designMode=designMode)
@@ -186,13 +195,13 @@ class RefreshToolBar(BaseToolBar):
                                               tip="Refresh view",
                                               triggered=self.onRefresh)
         self.addAction(self._refreshAction)
-    
+
     def onRefresh(self):
         self.emit(Qt.SIGNAL("refreshTriggered"))
 
 
 class PerspectiveToolBar(BaseToolBar):
-    
+
     def __init__(self, perspective, view=None, parent=None, designMode=False):
         BaseToolBar.__init__(self, name="Taurus refresh toolbar", view=view,
                              parent=parent, designMode=designMode)
@@ -202,7 +211,7 @@ class PerspectiveToolBar(BaseToolBar):
         b.setToolTip("Perspective selection")
         b.setPopupMode(Qt.QToolButton.InstantPopup)
         b.setToolButtonStyle(Qt.Qt.ToolButtonTextBesideIcon)
-        
+
         menu = Qt.QMenu("Perspective", b)
         b.setMenu(menu)
         af = ActionFactory()
@@ -216,16 +225,16 @@ class PerspectiveToolBar(BaseToolBar):
             menu.addAction(action)
             if persp == perspective:
                 b.setDefaultAction(action)
-        
+
         self._perspectiveAction = self.addWidget(b)
 
     def switchPerspectiveButton(self):
         """Returns the QToolButton that handles the switch perspective.
-        
+
         :return: (PyQt4.QtGui.QToolButton) the switch perspective tool button
         """
         return self._perspective_button
-    
+
     def onSwitchPerspective(self):
         action = self.sender()
         self._perspective = action.perspective
@@ -241,56 +250,115 @@ class QBaseModelWidget(Qt.QMainWindow):
     example), envolved by optional toolbar and statusbar.
     The Qt model associated with the internal Qt view widget should be a
     :class:`taurus.qt.qtcore.model.TaurusBaseModel`"""
-    
-    def __init__(self, parent=None, designMode=False, with_filter_widget=True):
+
+    KnownPerspectives = { }
+    DftPerspective = None
+
+    def __init__(self, parent=None, designMode=False, with_filter_widget=True,
+                 with_selection_widget=True, with_refresh_widget=True,
+                 perspective=None, proxy=None):
         Qt.QMainWindow.__init__(self, parent)
         self.setWindowFlags(Qt.Qt.Widget)
         self._baseQModel = None
         self._toolBars = []
-        self._with_filter_widget = with_filter_widget
-        
+
+        if with_filter_widget:
+            if isinstance(with_filter_widget, (bool, int)):
+                self._with_filter_widget = FilterToolBar
+            else:
+                self._with_filter_widget = with_filter_widget
+        else:
+            self._with_filter_widget = None
+
+        if with_selection_widget:
+            if isinstance(with_selection_widget, (bool, int)):
+                self._with_selection_widget = SelectionToolBar
+            else:
+                self._with_selection_widget = with_selection_widget
+        else:
+            self._with_selection_widget = None
+
+        if with_refresh_widget:
+            if isinstance(with_refresh_widget, (bool, int)):
+                self._with_refresh_widget = RefreshToolBar
+            else:
+                self._with_refresh_widget = with_refresh_widget
+        else:
+            self._with_refresh_widget = None
+            
+        self._proxyModel = proxy
+
         toolBars = self.createToolArea()
         self._viewWidget = self.createViewWidget()
         statusbar = self.createStatusBar()
-        
+
         for toolBar in toolBars:
-            toolBar.addSeparator()
+            #toolBar.addSeparator()
             self.addToolBar(toolBar)
         self.setContentsMargins(0, 0, 0, 0)
         self.setCentralWidget(self._viewWidget)
         self.setStatusBar(statusbar)
-    
-    def createViewWidget(self):
+
+        if perspective is None:
+            perspective = self.DftPerspective
+
+        if len(self.KnownPerspectives) > 1:
+            p_bar = self._perspectiveBar = PerspectiveToolBar(perspective, view=self, parent=self)
+            self.connect(p_bar, Qt.SIGNAL("perspectiveChanged"), self.onSwitchPerspective)
+            self.addToolBar(p_bar)
+        else:
+            self._perspectiveBar = None
+        self._setPerspective(perspective)
+
+    def createViewWidget(self, klass=None):
         raise NotImplementedError
-    
+
     def createStatusBar(self):
         sb = Qt.QStatusBar()
         sb.setSizeGripEnabled(False)
         return sb
-    
+
     def createToolArea(self):
         tb = [] # tb = self._toolArea = QToolArea(self)
         if self._with_filter_widget:
-            f_bar = self._filterBar = FilterToolBar(view=self, parent=self)
+            f_bar = self._filterBar = self._with_filter_widget(view=self, parent=self)
             Qt.QObject.connect(f_bar, Qt.SIGNAL("filterChanged"),
                                self.onFilterChanged)
             tb.append(f_bar)
         else:
             self._filterBar = None
+        
+        if self._with_selection_widget:
+            s_bar = self._selectionBar = self._with_selection_widget(view=self, parent=self)
+            Qt.QObject.connect(s_bar, Qt.SIGNAL("selectAllTriggered"),
+                               self.onSelectAll)
+            Qt.QObject.connect(s_bar, Qt.SIGNAL("clearSelectionTriggered"),
+                               self.onClearSelection)
+            tb.append(s_bar)
+        else:
+            self._selectionBar = None
+        
+        if self._with_refresh_widget:
+            r_bar = self._refreshBar = self._with_refresh_widget(view=self, parent=self)
+            Qt.QObject.connect(r_bar, Qt.SIGNAL("refreshTriggered"),
+                               self.onRefreshModel)
+            tb.append(r_bar)
+        else:
+            self._refreshBar = None
 
-        s_bar = self._selectionBar = SelectionToolBar(view=self, parent=self)
-        Qt.QObject.connect(s_bar, Qt.SIGNAL("selectAllTriggered"),
-                           self.onSelectAll)
-        Qt.QObject.connect(s_bar, Qt.SIGNAL("clearSelectionTriggered"),
-                           self.onClearSelection)
-        tb.append(s_bar)
-        
-        r_bar = self._selectionBar = RefreshToolBar(view=self, parent=self)
-        Qt.QObject.connect(r_bar, Qt.SIGNAL("refreshTriggered"),
-                           self.onRefreshModel)
-        tb.append(r_bar)
-        
         return tb
+    
+    def getPerspectiveBar(self):
+        return self._perspectiveBar
+    
+    def getFilterBar(self):
+        return self._filterBar
+    
+    def getSelectionBar(self):
+        return self._selectionBar
+    
+    def getRefreshBar(self):
+        return self._refreshBar
 
     def onRefreshModel(self):
         self.getQModel().refresh(True)
@@ -308,7 +376,7 @@ class QBaseModelWidget(Qt.QMainWindow):
         as arguments'''
         item = self._mapToSource(index).internalPointer()
         self.emit(Qt.SIGNAL('itemClicked'),item, index.column())
-        
+
     def _onDoubleClicked (self, index):
         '''Emits an "itemDoubleClicked" signal with the clicked item and column
         as arguments'''
@@ -317,7 +385,7 @@ class QBaseModelWidget(Qt.QMainWindow):
 
     def viewWidget(self):
         return self._viewWidget
-    
+
     def getQModel(self):
         return self.viewWidget().model()
 
@@ -337,11 +405,11 @@ class QBaseModelWidget(Qt.QMainWindow):
         return index
 
     def setQModel(self, qmodel):
-        
+
         self._baseQModel = qmodel
         while isinstance(self._baseQModel, Qt.QAbstractProxyModel):
             self._baseQModel = self._baseQModel.sourceModel()
-        
+
         view = self.viewWidget()
         old_selection_model = view.selectionModel()
         CC = 'currentChanged(const QModelIndex &,const QModelIndex &)'
@@ -360,40 +428,40 @@ class QBaseModelWidget(Qt.QMainWindow):
                                self.viewSelectionChanged)
         view.setCurrentIndex(view.rootIndex())
         self._updateToolBar()
-    
+
     def viewSelectionChanged(self, selected, deselected):
         self.emit(Qt.SIGNAL("itemSelectionChanged"))
-    
+
     def viewCurrentIndexChanged(self, current, previous):
         # if there is a proxy model we have to translate the selection
         base_current = self._mapToSource(current)
         base_previous = self._mapToSource(previous)
-        
+
         self._updateToolBar(current)
-        
+
         if base_current.isValid():
             currentTaurusTreeItem = base_current.internalPointer()
         else:
             currentTaurusTreeItem = None
-            
+
         if base_previous.isValid():
             previousTaurusTreeItem = base_previous.internalPointer()
         else:
             previousTaurusTreeItem = None
         self.emit(Qt.SIGNAL("currentItemChanged"), currentTaurusTreeItem,
                   previousTaurusTreeItem)
-    
+
     def _updateToolBar(self, current=None, previous=None):
         pass
-    
+
     def selectedItems(self):
         """Returns a list of all selected non-hidden items
-        
+
         :return: (list<TaurusTreeItem>)
         """
         view = self.viewWidget()
         return [self._mapToSource(index).internalPointer() for index in view.selectedIndexes()]
-    
+
     def onFilterChanged(self, filter):
         if not self.usesProxyQModel():
             return
@@ -404,17 +472,54 @@ class QBaseModelWidget(Qt.QMainWindow):
         #proxy_model.setFilterFixedString(filter)
         #proxy_model.setFilterWildcard(filter)
         #self.update()
-    
+
     def refresh(self):
         self.getQModel().refresh()
 
+    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
+    # Perspective handling
+    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
+
+    def perspective(self):
+        return self._perspectiveBar.perspective()
+
+    def onSwitchPerspective(self, perspective):
+        old_qmodel =  self.getQModel()
+        self._setPerspective(perspective)
+        
+        #set the selectables as they where in the previous model
+        if hasattr(old_qmodel, "selectables"):
+            self.getQModel().setSelectables(old_qmodel.selectables())
+        
+        #set the taurus model (if any) to the qmodel
+        if hasattr(self,'getModelObj'):
+            taurusModel =  self.getModelObj()
+            if taurusModel is not None:
+                self.getQModel().setDataSource(taurusModel)
+
+    def _setPerspective(self, perspective):
+        qmodel_classes = self.KnownPerspectives[perspective]["model"]
+        qmodel_class, qmodel_proxy_classes = qmodel_classes[-1], qmodel_classes[-2::-1] #reversed
+        qmodel = qmodel_class(self)
+        qmodel_source = qmodel
+        if self._proxyModel is None: #applies the chain of proxies
+            for qmodel_proxy_class in qmodel_proxy_classes:
+                qproxy = qmodel_proxy_class(self)
+                qproxy.setSourceModel(qmodel_source)
+                qmodel_source = qproxy
+        else:
+            self._proxyModel.setSourceModel(qmodel_source)
+            qmodel_source = self._proxyModel
+        self.setQModel(qmodel_source)
+
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
     # QMainWindow overwriting
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
+
     def addToolBar(self, toolbar):
         Qt.QMainWindow.addToolBar(self, toolbar)
         self._toolBars.append(toolbar)
-    
+
     def insertToolBar(self, before, toolbar):
         if isinstance(before, Qt.QToolBar):
             index = self._toolBars.index(before)
@@ -427,48 +532,12 @@ class QBaseModelWidget(Qt.QMainWindow):
 
 class TaurusBaseModelWidget(TaurusBaseWidget):
     """A class:`taurus.qt.qtgui.base.TaurusBaseWidget` that connects to a
-    taurus model. It must be used together with class:`taurus.qt.qtgui.base.QBaseModelWidget`"""
-    
-    KnownPerspectives = { }
-    DftPerspective = None
+    taurus model. It must be used together with
+    class:`~taurus.qt.qtgui.base.QBaseModelWidget`"""
 
-    def __init__(self, designMode=False, perspective=None, proxy=None):
+    def __init__(self, designMode=False):#, perspective=None, proxy=None):
         name = self.__class__.__name__
-        self._proxyModel = proxy
         self.call__init__(TaurusBaseWidget, name, designMode=designMode)
-        if perspective is None:
-            perspective = self.DftPerspective
-        
-        if len(self.KnownPerspectives) > 1:
-            p_bar = self._perspectiveBar = PerspectiveToolBar(perspective, view=self, parent=self)
-            self.connect(p_bar, Qt.SIGNAL("perspectiveChanged"), self.onSwitchPerspective)
-            self.addToolBar(p_bar)
-        else:
-            self._perspectiveBar = None
-        self._setPerspective(perspective)
-
-    def perspective(self):
-        return self._perspectiveBar.perspective()
-    
-    def onSwitchPerspective(self, perspective):
-        self._setPerspective(perspective)
-    
-    def _setPerspective(self, perspective):
-        qmodel_classes = self.KnownPerspectives[perspective]["model"]
-        qmodel_class, qmodel_proxy_classes = qmodel_classes[-1], qmodel_classes[:-1]
-        qmodel_proxy_classes.reverse()
-        qmodel = qmodel_class(self, self.getModelObj())
-        qmodel_source = qmodel
-        if self._proxyModel is None:
-            for qmodel_proxy_class in qmodel_proxy_classes:
-                qproxy = qmodel_proxy_class(self)
-                qproxy.setSourceModel(qmodel_source)
-                qmodel_source = qproxy
-        else:
-            self._proxyModel.setSourceModel(qmodel_source)
-            qmodel_source = self._proxyModel
-        self.setQModel(qmodel_source)
-    
 
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
     # TaurusBaseWidget overwriting
@@ -479,18 +548,19 @@ class TaurusBaseModelWidget(TaurusBaseWidget):
 
         view, modelObj = self.viewWidget(), self.getModelObj()
         model = view.model()
-        if model is None: return
+        if model is None:
+            return
         model.setDataSource(modelObj)
-    
+
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
     # QT property definition
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
-    
-    #: This property holds the unique URI string representing the model name 
-    #: with which this widget will get its data from. The convention used for 
+
+    #: This property holds the unique URI string representing the model name
+    #: with which this widget will get its data from. The convention used for
     #: the string can be found :ref:`here <model-concept>`.
-    #: 
-    #: In case the property :attr:`useParentModel` is set to True, the model 
+    #:
+    #: In case the property :attr:`useParentModel` is set to True, the model
     #: text must start with a '/' followed by the attribute name.
     #:
     #: **Access functions:**
@@ -501,4 +571,4 @@ class TaurusBaseModelWidget(TaurusBaseWidget):
     #:
     #: .. seealso:: :ref:`model-concept`
     model = Qt.pyqtProperty("QString", TaurusBaseWidget.getModel, setModel,
-                            TaurusBaseWidget.resetModel)
\ No newline at end of file
+                            TaurusBaseWidget.resetModel)
