@@ -54,12 +54,14 @@ class QConfigEditorModel(Qt.QStandardItemModel):
         
     def setData(self,index, value, role = Qt.Qt.DisplayRole):
         '''see :meth:`Qt.QAbstractTableModel.setData`'''
-
-        if str(index.data().toString()) == str(value.toString()):
+        
+        idx_data_str = Qt.from_qvariant(index.data(), str)
+        value_str = Qt.from_qvariant(value, str)
+        if idx_data_str == value_str:
             return False
         #self.itemFromIndex(index).setData(value, role)
         try:
-            self.valueChanged(str(value.toString()), index)
+            self.valueChanged(value_str, index)
         except:
             self.emit(Qt.SIGNAL("showError"),'Wrong value!','The value you entered is wrong. The old value will be restored.')
             return Qt.QStandardItemModel.setData(self,index,index.data(),role)
@@ -91,7 +93,7 @@ class QConfigEditorModel(Qt.QStandardItemModel):
         '''
         tmpindex = self._toDeleteIndex
         item = self.itemFromIndex(tmpindex)
-        path = str(item.data(Qt.Qt.UserRole).toString())
+        path = Qt.from_qvariant(item.data(Qt.Qt.UserRole), str)
         self._delete = False
         self._configurationDictionaries = self.removeBranch(self._configurationDictionaries, path)
         
@@ -152,7 +154,7 @@ class QConfigEditorModel(Qt.QStandardItemModel):
         :param index: (QModelIndex) index of the model
         '''
         changedItem = self.itemFromIndex(index)
-        path = changedItem.data(Qt.Qt.UserRole).toString()
+        path = Qt.from_qvariant(changedItem.data(Qt.Qt.UserRole), str)
         self._configurationDictionaries = self.changeTreeValue(self._configurationDictionaries, path, value)
         try: group = eval(str(path).split(';',1)[0])
         except: group = str(path).split(';',1)[0]
@@ -248,7 +250,8 @@ class QConfigEditorModel(Qt.QStandardItemModel):
                 child.setEditable(False)
                 item.appendRow(child)
                 
-                path = Qt.QVariant(str(item.data(Qt.Qt.UserRole).toString()) + ";__itemConfigurations__;" + k)
+                txt = Qt.from_qvariant(item.data(Qt.Qt.UserRole), str)
+                path = Qt.QVariant(txt + ";__itemConfigurations__;" + k)
                 child.setData(path, Qt.Qt.UserRole)
                 self.fillTaurusConfig(child, value) #recursive call to fill all nodes
             else:
@@ -260,7 +263,8 @@ class QConfigEditorModel(Qt.QStandardItemModel):
                 
                 item.appendRow([child,typeV,valueV])
                 
-                path = Qt.QVariant(str(item.data(Qt.Qt.UserRole).toString()) + ";__itemConfigurations__;" + k)
+                txt = Qt.from_qvariant(item.data(Qt.Qt.UserRole), str)
+                path = Qt.QVariant(txt + ";__itemConfigurations__;" + k)
                 child.setEditable(False)
                 typeV.setEditable(False)
                 
@@ -284,8 +288,8 @@ class QConfigEditorModel(Qt.QStandardItemModel):
                 if BaseConfigurableClass.isTaurusConfig(value):
                     child.setEditable(False)
                     item.appendRow(child)
-                
-                    path = Qt.QVariant(str(item.data(Qt.Qt.UserRole).toString()) +";" + k)
+                    txt = Qt.from_qvariant(item.data(Qt.Qt.UserRole), str)
+                    path = Qt.QVariant(txt +";" + k)
                     child.setData(path, Qt.Qt.UserRole)
                     self.fillTaurusConfig(child, value) #recursive call to fill all nodes
                 else:
@@ -294,7 +298,8 @@ class QConfigEditorModel(Qt.QStandardItemModel):
                     typeV.setForeground(Qt.QBrush(Qt.QColor('gray')))
                     child.setForeground(Qt.QBrush(Qt.QColor('gray')))
                     item.appendRow([child,typeV,valueV])
-                    path = Qt.QVariant(str(item.data(Qt.Qt.UserRole).toString()) + ";" + k)
+                    txt = Qt.from_qvariant(item.data(Qt.Qt.UserRole), str)
+                    path = Qt.QVariant(txt + ";" + k)
     
                     child.setData(path, Qt.Qt.UserRole)
                     child.setEditable(False)
@@ -312,8 +317,8 @@ class QConfigEditorModel(Qt.QStandardItemModel):
         :returns (dict)
         '''
         result = None
-        qstate = self._settings.value(key).toByteArray()
-        if not qstate.isNull():
+        qstate = Qt.from_qvariant(self._settings.value(key), 'toByteArray')
+        if qstate is not None and not qstate.isNull():
             try: result = pickle.loads(qstate.data())
             except Exception,e: 
                 msg = 'problems loading TaurusConfig: \n%s'%repr(e)
@@ -400,7 +405,7 @@ class QConfigEditor(TaurusWidget):
         '''Reimplemented from :meth:`QWidget.contextMenuEvent`'''
   
         self.tree._toDeleteIndex = self.treeview.selectedIndexes()[0]
-        text = str(self.tree._toDeleteIndex.data().toString())
+        text = Qt.from_qvariant(self.tree._toDeleteIndex.data(), str)
         if self.tree._toDeleteIndex.column() in [1,2] or text in ['LAST', '[custom]'] or text in self.tree.perspectives:
             return
         menu = Qt.QMenu()
