@@ -36,7 +36,7 @@ import taurus.core.util
 import taurus.qt.qtgui.base
 import taurus.qt.qtgui.resource
 
-ElemType = taurus.core.TaurusElementType
+ElemType = taurus.core.taurusbasetypes.TaurusElementType
 getElementTypeIcon = taurus.qt.qtgui.resource.getElementTypeIcon
 getPixmap = taurus.qt.qtgui.resource.getPixmap
 getThemeIcon = taurus.qt.qtgui.resource.getThemeIcon
@@ -73,25 +73,25 @@ class BaseElementFilter(BaseFilter):
 
 class DeviceFilter(BaseElementFilter):
     
-    def __init__(self, re_expr, func=taurus.core.TaurusDevInfo.name):
+    def __init__(self, re_expr, func=taurus.core.taurusdatabase.TaurusDevInfo.name):
         super(DeviceFilter, self).__init__(re_expr, func=func)
 
 
 class DeviceClassFilter(BaseElementFilter):
     
-    def __init__(self, re_expr, func=taurus.core.TaurusDevClassInfo.name):
+    def __init__(self, re_expr, func=taurus.core.taurusdatabase.TaurusDevInfo.name):
         super(DeviceClassFilter, self).__init__(re_expr, func=func)
 
 
 class ServerFilter(BaseElementFilter):
 
-    def __init__(self, re_expr, func=taurus.core.TaurusServInfo.name):
+    def __init__(self, re_expr, func=taurus.core.taurusdatabase.TaurusServInfo.name):
         super(ServerFilter, self).__init__(re_expr, func=func)
 
 
 class AttributeFilter(BaseElementFilter):
     
-    def __init__(self, re_expr, func=taurus.core.TaurusAttrInfo.name):
+    def __init__(self, re_expr, func=taurus.core.taurusdatabase.TaurusAttrInfo.name):
         super(AttributeFilter, self).__init__(re_expr, func=func)
 
 
@@ -110,27 +110,27 @@ class KlassFilter(BaseFilter):
 def getFilter(type, re_expr=None):
     if re_expr is None:
         if type == ElemType.Device:
-            return KlassFilter(taurus.core.TaurusDevInfo)
+            return KlassFilter(taurus.core.taurusdatabase.TaurusDevInfo)
         elif type == ElemType.Server:
-            return KlassFilter(taurus.core.TaurusServInfo)
+            return KlassFilter(taurus.core.taurusdatabase.TaurusServInfo)
         elif type == ElemType.DeviceClass:
-            return KlassFilter(taurus.core.TaurusDevClassInfo)
+            return KlassFilter(taurus.core.taurusdatabase.TaurusDevInfo)
         return None
 
     if type == ElemType.Device:
         return DeviceFilter(re_expr)
     elif type == ElemType.Domain:
-        return DeviceFilter(re_expr, taurus.core.TaurusDevInfo.domain)
+        return DeviceFilter(re_expr, taurus.core.taurusdatabase.TaurusDevInfo.domain)
     elif type == ElemType.Family:
-        return DeviceFilter(re_expr, taurus.core.TaurusDevInfo.family)
+        return DeviceFilter(re_expr, taurus.core.taurusdatabase.TaurusDevInfo.family)
     elif type == ElemType.Member:
-        return DeviceFilter(re_expr, taurus.core.TaurusDevInfo.member)
+        return DeviceFilter(re_expr, taurus.core.taurusdatabase.TaurusDevInfo.member)
     elif type == ElemType.Server:
         return ServerFilter(re_expr)
     elif type == ElemType.ServerName:
-        return ServerFilter(re_expr, taurus.core.TaurusServInfo.serverName)
+        return ServerFilter(re_expr, taurus.core.taurusdatabase.TaurusServInfo.serverName)
     elif type == ElemType.ServerInstance:
-        return ServerFilter(re_expr, taurus.core.TaurusServInfo.serverInstance)
+        return ServerFilter(re_expr, taurus.core.taurusdatabase.TaurusServInfo.serverInstance)
     elif type == ElemType.DeviceClass:
         return DeviceClassFilter(re_expr)
     elif type == ElemType.Attribute:
@@ -254,7 +254,7 @@ class TaurusFilterPanelOld1(Qt.QWidget, taurus.qt.qtgui.base.TaurusBaseWidget):
         g_layout = self.layout()
         header = g_layout.itemAt(0).widget()
         headerCombo = header.layout().itemAt(1).widget()
-        return headerCombo.itemData(headerCombo.currentIndex()).toPyObject()
+        return Qt.from_qvariant(headerCombo.itemData(headerCombo.currentIndex()))
 
     def calculate(self):
         db = self.getModelObj()
@@ -269,7 +269,7 @@ class TaurusFilterPanelOld1(Qt.QWidget, taurus.qt.qtgui.base.TaurusBaseWidget):
             comboBox = field_layout.itemAt(1).widget()
             edit = field_layout.itemAt(2).widget()
             
-            type = comboBox.itemData(comboBox.currentIndex()).toPyObject()
+            type = Qt.from_qvariant(comboBox.itemData(comboBox.currentIndex()))
             expr = str(edit.text())
             f = getFilter(type, expr)
             filters.append(f)
@@ -284,7 +284,7 @@ class TaurusFilterPanelOld1(Qt.QWidget, taurus.qt.qtgui.base.TaurusBaseWidget):
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 
     def getModelClass(self):
-        return taurus.core.TaurusDatabase
+        return taurus.core.taurusdatabase.TaurusDatabase
 
     #: This property holds the unique URI string representing the model name 
     #: with which this widget will get its data from. The convention used for 
@@ -367,7 +367,7 @@ class TaurusFilterPanelOld2(Qt.QWidget, taurus.qt.qtgui.base.TaurusBaseWidget):
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 
     def getModelClass(self):
-        return taurus.core.TaurusDatabase
+        return taurus.core.taurusdatabase.TaurusDatabase
 
     def setModel(self, m):
         taurus.qt.qtgui.base.TaurusBaseWidget.setModel(self, m)
@@ -655,7 +655,7 @@ class TaurusFilterPanel(Qt.QWidget, taurus.qt.qtgui.base.TaurusBaseWidget):
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 
     def getModelClass(self):
-        return taurus.core.TaurusDatabase
+        return taurus.core.taurusdatabase.TaurusDatabase
 
     def setModel(self, m):
         taurus.qt.qtgui.base.TaurusBaseWidget.setModel(self, m)
