@@ -69,6 +69,14 @@ class TaurusForm(TaurusWidget):
     can be changed and specific mappings can be defined using the
     :meth:`setCustomWidgetMap` method.
     
+    Item objects can be accessed by index using a list-like notation::
+    
+      form = TaurusForm()
+      form.model = ['sys/tg_test/1'+a for a in ('short_image','/float_scalar','/double_scalar')] 
+      form[0].labelConfig = 'dev_alias'
+      form[-1].writeWidgetClass = 'TaurusWheelEdit'
+      print(len(form))  # --> outputs '3' (the length of the form is the number of items)
+    
     By default, the form provides global Apply and Cancel buttons.
     
     You can also see some code that exemplifies the use of TaurusForm in :ref:`Taurus
@@ -87,7 +95,6 @@ class TaurusForm(TaurusWidget):
         self._model = []
         self._children = []
         self.setFormWidget(formWidget)
-        self._withButtons = withButtons
         
         self.setLayout(Qt.QVBoxLayout())
         
@@ -98,13 +105,12 @@ class TaurusForm(TaurusWidget):
         self.scrollArea.setWidget(frame)
         self.scrollArea.setWidgetResizable(True)
         self.layout().addWidget(self.scrollArea)
-        self.__modelChooser = None
+        self.__modelChooserDlg = None
         
         self.buttonBox = QButtonBox(buttons = buttons, parent = self)
         self.layout().addWidget(self.buttonBox)
         
         self._connectButtons()
-        self._manageButtonBox()
         
         
         #Actions (they automatically populate the context menu)
@@ -114,8 +120,29 @@ class TaurusForm(TaurusWidget):
         self.addAction(self.chooseModelsAction)
         self.connect(self.chooseModelsAction, Qt.SIGNAL("triggered()"), self.chooseModels)
         
+        self.showButtonsAction = Qt.QAction('Show Buttons', self)
+        self.showButtonsAction.setCheckable(True)
+        self.addAction(self.showButtonsAction)
+        self.connect(self.showButtonsAction, Qt.SIGNAL("triggered(bool)"), self.setWithButtons)
+        self.setWithButtons(withButtons)
+        
+        self.changeLabelsAction = Qt.QAction('Change labels (all items)', self)
+        self.addAction(self.changeLabelsAction)
+        self.connect(self.changeLabelsAction, Qt.SIGNAL("triggered()"), self.onChangeLabelsAction)
+        
         self.resetModifiableByUser()
         self.setSupportedMimeTypes([TAURUS_MODEL_LIST_MIME_TYPE, TAURUS_DEV_MIME_TYPE, TAURUS_ATTR_MIME_TYPE, TAURUS_MODEL_MIME_TYPE, 'text/plain'])
+
+        #properties
+        self.registerConfigProperty(self.isWithButtons, self.setWithButtons, 'withButtons')
+
+    def __getitem__(self, key):
+        '''provides a list-like interface: items of the form can be accessed using slice notation'''
+        return self.getItemByIndex(key)
+    
+    def __len__(self):
+        '''returns the number of items contained by the form'''
+        return len(self.getItems())
     
     def _splitModel(self, modelNames):
         '''convert str to list if needed (commas and whitespace are considered as separators)'''
@@ -155,13 +182,33 @@ class TaurusForm(TaurusWidget):
     
     def chooseModels(self):
         '''launches a model chooser dialog to modify the contents of the form'''
-        if self.__modelChooser is None: 
-            self.__modelChooser = TaurusModelChooser()
-            self.connect(self.__modelChooser, Qt.SIGNAL("updateModels"), self.setModel)
-            self.__modelChooser.setWindowTitle("%s - Model Chooser"%unicode(self.windowTitle()))
-        self.__modelChooser.setListedModels(self.getModel())
-        self.__modelChooser.show()
-        self.__modelChooser.raise_()
+        if self.__modelChooserDlg is None: 
+            self.__modelChooserDlg = Qt.QDialog(self)
+            self.__modelChooserDlg.setWindowTitle("%s - Model Chooser"%unicode(self.windowTitle()))
+            self.__modelChooserDlg.modelChooser = TaurusModelChooser()
+            layout = Qt.QVBoxLayout()
+            layout.addWidget(self.__modelChooserDlg.modelChooser)
+            self.__modelChooserDlg.setLayout(layout)
+            self.connect(self.__modelChooserDlg.modelChooser, Qt.SIGNAL("updateModels"), self.setModel)
+            
+        models_and_labels = []
+        models = [m.lower() for m in self.getModel()]
+        indexdict = {}
+        for m in models:
+            indexdict[m]=indexdict.get(m,-1)+1
+            item = self.getItemByModel(m,indexdict[m])
+            if item is None:
+                label = None
+            else:
+                try: 
+                    label = str(item.labelWidget().text()) #this assumes that the readwidget is a QLabel subclass (or something that has text())
+                except:
+                    label = None
+            models_and_labels.append((m,label))
+            
+        self.__modelChooserDlg.modelChooser.setListedModels(models_and_labels)
+        self.__modelChooserDlg.show()
+        self.__modelChooserDlg.raise_()
         
     def chooseAttrs(self):
         self.info('TaurusForm.chooseAttrs() ahs been deprecated. Use TaurusForm.chooseModels() instead')
@@ -207,11 +254,11 @@ class TaurusForm(TaurusWidget):
         model = self._splitModel(model)
         self.destroyChildren()
         self._model = model
-        if True or model is not None:
+        if True or model is not None: #@todo: !NOTE THAT This if has been disabled by the first True. Check why!
             self.fillWithChildren()
         #update the modelchooser list
-        if self.__modelChooser is not None:
-            self.__modelChooser.setListedModels(self._model)
+        if self.__modelChooserDlg is not None:
+            self.__modelChooserDlg.modelChooser.setListedModels(self._model)
                 
     def resetModel(self):
         self.destroyChildren()
@@ -234,7 +281,11 @@ class TaurusForm(TaurusWidget):
             obj=taurus.Attribute(model) #if it is not an attribute, it will get an exception here
             return self._defaultFormWidget,(),{}
         except:
-            obj=taurus.Device(model)
+            try:
+                obj=taurus.Device(model)
+            except:
+                self.warning('Cannot handle model "%s". Using default widget.'%(model))
+                return self._defaultFormWidget,(),{}
             try:
                 key = obj.getHWObj().info().dev_class
             except:
@@ -280,11 +331,11 @@ class TaurusForm(TaurusWidget):
     
     def setWithButtons(self, trueFalse):
         self._withButtons = trueFalse
-        self._manageButtonBox()
+        self.buttonBox.setVisible(self._withButtons)
+        self.showButtonsAction.setChecked(self._withButtons)
         
     def resetWithButtons(self):
-        self._withButtons = True
-        self._manageButtonBox()
+        self.setWithButtons(True)
     
     def dropEvent(self, event):
         '''reimplemented to support dropping of modelnames in forms'''
@@ -306,6 +357,8 @@ class TaurusForm(TaurusWidget):
         '''
         TaurusWidget.setModifiableByUser(self, modifiable)
         self.chooseModelsAction.setEnabled(modifiable)
+        self.showButtonsAction.setEnabled(modifiable)
+        self.changeLabelsAction.setEnabled(modifiable)
         for item in self.getItems():
             try: 
                 item.setModifiableByUser(modifiable)
@@ -380,11 +433,24 @@ class TaurusForm(TaurusWidget):
         '''returns a list of the objects that have been created as childs of the form'''
         return self._children
     
-    def _manageButtonBox(self):
-        if self.isWithButtons():
-            self.buttonBox.setVisible(True)
-        else:
-            self.buttonBox.setVisible(False)
+#    def _manageButtonBox(self):
+#        if self.isWithButtons():
+#            self.buttonBox.setVisible(True)
+#        else:
+#            self.buttonBox.setVisible(False)
+
+    def onChangeLabelsAction(self):
+        '''changes the labelConfig of all its items'''
+        keys = ['label', 'attr_name', 'attr_fullname', 'dev_alias', 'dev_name', 'dev_fullname']
+            
+        msg = 'Choose new source for the label. \n'+\
+              'You can also write a more complex text\n'+\
+              'using any of the proposed sources as a\n'+\
+              'placeholder by enclosing it in "< >" brackets'
+        labelConfig, ok = Qt.QInputDialog.getItem(self, 'Change Label', msg, keys, 0, True)
+        if ok:
+            for item in self.getItems():
+                item.labelConfig=str(labelConfig)
 
     @Qt.pyqtSignature("apply()")        
     def apply(self):
@@ -419,7 +485,7 @@ class TaurusForm(TaurusWidget):
                                                 resetWithButtons)
     
     modifiableByUser = Qt.pyqtProperty("bool", TaurusWidget.isModifiableByUser, 
-                                               TaurusWidget.setModifiableByUser,
+                                               setModifiableByUser,
                                                TaurusWidget.resetModifiableByUser)
       
     
@@ -488,7 +554,7 @@ class TaurusCommandsForm(TaurusWidget):
         
     def getModelClass(self):
         '''see :meth:`TaurusBaseComponent.getModelClass`'''
-        return taurus.core.TaurusDevice
+        return taurus.core.taurusdevice.TaurusDevice
         
     def _updateCommandWidgets(self, *args):
         '''
@@ -497,7 +563,7 @@ class TaurusCommandsForm(TaurusWidget):
         '''
         #self.debug('In TaurusCommandsForm._updateCommandWidgets())')
         dev = self.getModelObj()
-        if dev is None or dev.getSWState() != taurus.core.TaurusSWDevState.Running:
+        if dev is None or dev.getSWState() != taurus.core.taurusbasetypes.TaurusSWDevState.Running:
             self.debug('Cannot connect to device')
             self._clearFrame()
             return
@@ -666,13 +732,13 @@ class TaurusAttrForm(TaurusWidget):
         
     def getModelClass(self):
         '''see :meth:`TaurusBaseComponent.getModelClass`'''
-        return taurus.core.TaurusDevice
+        return taurus.core.taurusdevice.TaurusDevice
     
     def _updateAttrWidgets(self):
         '''Populates the form with an item for each of the attributes shown
         '''
         dev = self.getModelObj()
-        if dev is None or dev.getSWState() != taurus.core.TaurusSWDevState.Running:
+        if dev is None or dev.getSWState() != taurus.core.taurusbasetypes.TaurusSWDevState.Running:
             self.debug('Cannot connect to device')
             self._form.setModel([])
             return
@@ -840,7 +906,8 @@ def taurusFormMain():
     parser.set_usage("%prog [options] [model1 [model2 ...]]")
     parser.set_description("the taurus form panel application")
     parser.add_option("--window-name", dest="window_name", default="TaurusForm", help="Name of the window")
-    
+    parser.add_option("--config", "--config-file", dest="config_file", default=None,
+                  help="use the given config file for initialization")
     app = TaurusApplication(cmd_line_parser=parser,
                             app_name="taurusform",
                             app_version=taurus.Release.version)
@@ -849,15 +916,31 @@ def taurusFormMain():
 
     dialog = TaurusForm()
     dialog.setModifiableByUser(True)
+    dialog.setModelInConfig(True)
     dialog.setWindowTitle(options.window_name)
+            
+    from taurus.qt.qtgui.resource import getThemeIcon
+    quitApplicationAction =  Qt.QAction(getThemeIcon("process-stop"),'Close Form', dialog)
+    dialog.connect(quitApplicationAction, Qt.SIGNAL("triggered()"), dialog.close)
     
+    saveConfigAction = Qt.QAction("Save current settings...", dialog)
+    saveConfigAction.setShortcut(Qt.QKeySequence.Save)
+    dialog.connect(saveConfigAction, Qt.SIGNAL("triggered()"), dialog.saveConfigFile)  
+    
+    loadConfigAction = Qt.QAction("&Retrieve saved settings...", dialog)
+    loadConfigAction.setShortcut(Qt.QKeySequence.Open)
+    dialog.connect(loadConfigAction, Qt.SIGNAL("triggered()"), dialog.loadConfigFile)
+    
+    dialog.addActions ((saveConfigAction, loadConfigAction, quitApplicationAction) )
     
     #set the default map for this installation
-    from taurus.TaurusCustomSettings import T_FORM_CUSTOM_WIDGET_MAP
-    dialog.setCustomWidgetMap(T_FORM_CUSTOM_WIDGET_MAP)
+    from taurus import tauruscustomsettings
+    dialog.setCustomWidgetMap(getattr(tauruscustomsettings,'T_FORM_CUSTOM_WIDGET_MAP',{}))
     
     #set a model list from the command line or launch the chooser  
-    if len(args)>0:
+    if options.config_file is not None: 
+        dialog.loadConfigFile(options.config_file)
+    elif len(args)>0:
         models=args
         dialog.setModel(models)
     else:
