@@ -33,7 +33,7 @@ from taurus.qt import Qt
 import taurus.core
 from taurus.qt.qtgui.container import TaurusWidget
 from taurus.qt.qtgui.tree import TaurusDbTreeWidget
-from taurus.core.util import CaselessList
+from taurus.core.util.containers import CaselessList
 from taurusmodellist import TaurusModelList
 import taurus.qt.qtgui.resource
 
@@ -41,11 +41,12 @@ import taurus.qt.qtgui.resource
 class TaurusModelSelectorTree(TaurusWidget):
     def __init__(self, parent = None, selectables=None, buttonsPos=None, designMode = None):
         TaurusWidget.__init__(self, parent)
-        if selectables is None: selectables = [taurus.core.TaurusElementType.Attribute, taurus.core.TaurusElementType.Member]
+        if selectables is None: selectables = [taurus.core.taurusbasetypes.TaurusElementType.Attribute, taurus.core.taurusbasetypes.TaurusElementType.Member, 
+                                               taurus.core.taurusbasetypes.TaurusElementType.Device]
         self._selectables = selectables
                         
         #tree
-        self._deviceTree = TaurusDbTreeWidget(perspective=taurus.core.TaurusElementType.Device)
+        self._deviceTree = TaurusDbTreeWidget(perspective=taurus.core.taurusbasetypes.TaurusElementType.Device)
         self._deviceTree.getQModel().setSelectables(self._selectables)
         self._deviceTree.setUseParentModel(True)
         
@@ -98,9 +99,9 @@ class TaurusModelSelectorTree(TaurusWidget):
         selected = []
         for item in self._deviceTree.selectedItems():
             nfo = item.itemData()
-            if isinstance(nfo, taurus.core.TaurusDevInfo):
+            if isinstance(nfo, taurus.core.taurusdatabase.TaurusDevInfo):
                 selected.append(nfo.fullName())
-            elif isinstance(nfo, taurus.core.TaurusAttrInfo):
+            elif isinstance(nfo, taurus.core.taurusdatabase.TaurusAttrInfo):
                 selected.append( "%s/%s"%(nfo.device().fullName(),nfo.name()) )
             else: 
                 self.info("Unknown item '%s' in selection"%repr(nfo))
@@ -247,7 +248,7 @@ class TaurusModelChooser(TaurusWidget):
     def _onUpdateModels(self):
         models = self.getListedModels()
         self.emit(Qt.SIGNAL("updateModels"), models)
-        if taurus.core.TaurusElementType.Attribute in self.tree._selectables:
+        if taurus.core.taurusbasetypes.TaurusElementType.Attribute in self.tree._selectables:
             self.emit(Qt.SIGNAL("UpdateAttrs"), models) #for backwards compatibility with the old AttributeChooser
     
     def setSingleModelMode(self, single):
