@@ -32,9 +32,9 @@ __all__=['TaurusModelModel','TaurusModelItem', 'TaurusModelList']
 import copy
 
 from taurus.qt import Qt
-from PyQt4 import Qwt5
 import taurus
-from taurus.core import TaurusException, TaurusElementType
+from taurus.core.taurusbasetypes import TaurusElementType
+from taurus.core.taurusexception import TaurusException
 from taurus.qt.qtcore.mimetypes import TAURUS_MODEL_LIST_MIME_TYPE, TAURUS_ATTR_MIME_TYPE, TAURUS_MODEL_MIME_TYPE
 from taurus.qt.qtgui.resource import getThemeIcon, getElementTypeIcon, getIcon
 
@@ -91,6 +91,9 @@ class TaurusModelItem(object):
             except TaurusException:
                 self.display, self.icon, self.ok = src, getThemeIcon('dialog-warning'), False
                 return
+            except Exception: #@todo: this catchall except is here as an emergency bugfix, but should probably be narrowed to PyTango DevFailed. 
+                self.display, self.icon, self.ok = src, getThemeIcon('network-error'),False
+                return
             if not dev.isValidDev():
                 self.display, self.icon, self.ok = src, getThemeIcon('network-error'),False
                 return
@@ -155,7 +158,7 @@ class TaurusModelModel(Qt.QAbstractListModel):
         if index.isValid() and (0 <= index.row() < self.rowCount()):
             row = index.row()
             item = self.items[row]
-            value = unicode(value.toString())
+            value = Qt.from_qvariant(value, unicode)
             if role == Qt.Qt.EditRole:
                 item.src = value
             elif role == Qt.Qt.DisplayRole:
@@ -169,7 +172,7 @@ class TaurusModelModel(Qt.QAbstractListModel):
         if position is None or position==-1: position = self.rowCount()
         if parentindex is None: parentindex = Qt.QModelIndex()
         if items is None:
-            slice = [TaurusModelItem() for i in range(rows)]
+            slice = [TaurusModelItem() for i in xrange(rows)]
         else:
             slice=list(items)
             rows = len(slice) #note that the rows parameter is ignored if items is passed
@@ -244,8 +247,9 @@ class TaurusModelModel(Qt.QAbstractListModel):
         '''reimplemented from :class:`Qt.QAbstractListModel`'''
         mimedata = Qt.QAbstractListModel.mimeData(self, indexes)
         if len(indexes)==1:
-#            mimedata.setData(TAURUS_ATTR_MIME_TYPE, str(self.data(indexes[0]).toString()))
-            mimedata.setText(self.data(indexes[0],role=SRC_ROLE).toString())
+#            mimedata.setData(TAURUS_ATTR_MIME_TYPE, Qt.from_qvariant(self.data(indexes[0]), str)))
+            txt = Qt.from_qvariant(self.data(indexes[0],role=SRC_ROLE), str)
+            mimedata.setText(txt)
         return mimedata
         #mimedata.setData()
     
@@ -254,7 +258,7 @@ class TaurusModelList(Qt.QListView):
     '''A list view widget to display and manage a list of models
     
     Tries to identify the type of model and show the state of the device/attr
-    associated qith it. It also allows drag and drop of models and sorting.
+    associated with it. It also allows drag and drop of models and sorting.
     '''
     
     def __init__(self, parent=None, items=None, designMode=False):
@@ -294,8 +298,8 @@ class TaurusModelList(Qt.QListView):
             idx = selected[0]
         else:
             return
-        value = str(self._model.data(idx, role=Qt.Qt.DisplayRole).toString())
-        src = str(self._model.data(idx, role=SRC_ROLE).toString())
+        value = Qt.from_qvariant(self._model.data(idx, role=Qt.Qt.DisplayRole), str)
+        src = Qt.from_qvariant(self._model.data(idx, role=SRC_ROLE), str)
         value,ok = Qt.QInputDialog.getText(self, "Display Value", "Display value for %s?"%src, Qt.QLineEdit.Normal, value)
         if not ok:
             return
@@ -401,4 +405,4 @@ if __name__ == "__main__":
     w = TaurusModelList()
     w.addModels(["item%i"%i for i in range(3)]+[TaurusModelItem(src='src1',display='d1')]+[('src2','d2')])
     w.show()    
-    sys.exit(app.exec_()) 
\ No newline at end of file
+    sys.exit(app.exec_()) 
