@@ -47,9 +47,7 @@ from taurus.qt.qtgui.display import TaurusLed
 from taurus.qt.qtgui.input import TaurusValueSpinBox, TaurusValueCheckBox
 from taurus.qt.qtgui.input import TaurusWheelEdit, TaurusValueLineEdit
 from taurus.qt.qtgui.button import TaurusLauncherButton
-from taurus.qt.qtgui.util import TaurusWidgetFactory
-
-from taurus.qt.qtgui.resource import getIcon
+from taurus.qt.qtgui.util import TaurusWidgetFactory, ConfigurationMenu
 
 
 class DefaultTaurusValueCheckBox(TaurusValueCheckBox):
@@ -62,6 +60,13 @@ class DefaultTaurusValueCheckBox(TaurusValueCheckBox):
         return None
     
 class DefaultLabelWidget(TaurusLabel):
+    '''
+    The base class used by default for showing the label of a TaurusValue. 
+        
+    .. note:: It only makes sense to use this class as a part of a TaurusValue,
+              since it assumes that it can get a reference to a TaurusValue via 
+              the :meth:`getTaurusValueBuddy` member
+    '''
     
     _dragEnabled=True
     
@@ -71,18 +76,18 @@ class DefaultLabelWidget(TaurusLabel):
         self.setSizePolicy(Qt.QSizePolicy.Preferred,Qt.QSizePolicy.Maximum)
         self.setBgRole(None)
         self.autoTrim = False
-        self.setStyleSheet('border-style: solid; border-width: 1px; border-color: transparent; border-radius: 4px;')
+        self.setStyleSheet('DefaultLabelWidget {border-style: solid; border-width: 1px; border-color: transparent; border-radius: 4px;}')
     
     def setModel(self, model):
         if model is None or model=='': 
             return TaurusLabel.setModel(self, None)
         try: config = self.taurusValueBuddy().getLabelConfig()
         except Exception: config = 'label'
-        if self.taurusValueBuddy().getModelClass() == taurus.core.TaurusAttribute:
+        if self.taurusValueBuddy().getModelClass() == taurus.core.taurusattribute.TaurusAttribute:
             config = self.taurusValueBuddy().getLabelConfig()
             TaurusLabel.setModel(self, model + "?configuration=%s"%config)
-        elif self.taurusValueBuddy().getModelClass() == taurus.core.TaurusDevice:
-             TaurusLabel.setModel(self, model + "/state?configuration=dev_alias")
+        elif self.taurusValueBuddy().getModelClass() == taurus.core.taurusdevice.TaurusDevice:
+            TaurusLabel.setModel(self, model + "/state?configuration=dev_alias")
     
     def sizeHint(self):
         return Qt.QSize(Qt.QLabel.sizeHint(self).width(), 18)
@@ -92,11 +97,16 @@ class DefaultLabelWidget(TaurusLabel):
         
         see :meth:`QWidget.contextMenuEvent`"""
         menu = Qt.QMenu(self)  
-        menu.addMenu(taurus.qt.qtgui.util.ConfigurationMenu(self.taurusValueBuddy())) #@todo: This should be done more Taurus-ish 
+        menu.addMenu(ConfigurationMenu(self.taurusValueBuddy())) #@todo: This should be done more Taurus-ish 
+        if hasattr(self.taurusValueBuddy().writeWidget(), 'resetPendingOperations'):
+            r_action = menu.addAction("reset write value",self.taurusValueBuddy().writeWidget().resetPendingOperations)
+            r_action.setEnabled(self.taurusValueBuddy().hasPendingOperations())
         if self.taurusValueBuddy().isModifiableByUser():
-            cr_action = menu.addAction("Change Read Widget",self.taurusValueBuddy().onChangeReadWidget)
+            menu.addAction("Change label",self.taurusValueBuddy().onChangeLabelConfig)
+            menu.addAction("Change Read Widget",self.taurusValueBuddy().onChangeReadWidget)
             cw_action = menu.addAction("Change Write Widget",self.taurusValueBuddy().onChangeWriteWidget)
             cw_action.setEnabled(not self.taurusValueBuddy().isReadOnly()) #disable the action if the taurusValue is readonly
+            
         menu.exec_(event.globalPos())
         event.accept()
         
@@ -104,9 +114,9 @@ class DefaultLabelWidget(TaurusLabel):
         '''reimplemented to use the taurusValueBuddy model instead of its own model'''
         mimeData = TaurusLabel.getModelMimeData(self)
         mimeData.setData(TAURUS_MODEL_MIME_TYPE, self.taurusValueBuddy().getModelName())
-        if self.taurusValueBuddy().getModelClass() == taurus.core.TaurusDevice:
+        if self.taurusValueBuddy().getModelClass() == taurus.core.taurusdevice.TaurusDevice:
             mimeData.setData(TAURUS_DEV_MIME_TYPE, self.taurusValueBuddy().getModelName())
-        elif self.taurusValueBuddy().getModelClass() == taurus.core.TaurusAttribute:
+        elif self.taurusValueBuddy().getModelClass() == taurus.core.taurusattribute.TaurusAttribute:
             mimeData.setData(TAURUS_ATTR_MIME_TYPE, self.taurusValueBuddy().getModelName())
         return mimeData
     
@@ -114,6 +124,7 @@ class DefaultLabelWidget(TaurusLabel):
     def getQtDesignerPluginInfo(cls):
         return None
 
+
 class ExpandingLabel(TaurusLabel):
     '''just a expanding TaurusLabel'''
     def __init__(self,*args):
@@ -125,6 +136,7 @@ class CenteredLed(TaurusLed):
     '''just a centered TaurusLed'''
     DefaultAlignment = Qt.Qt.AlignHCenter | Qt.Qt.AlignVCenter
         
+        
 class DefaultUnitsWidget(TaurusLabel):
     def __init__(self,*args):
         TaurusLabel.__init__(self,*args)
@@ -143,6 +155,7 @@ class DefaultUnitsWidget(TaurusLabel):
     @classmethod
     def getQtDesignerPluginInfo(cls):
         return None
+    
 
 class _AbstractTaurusValueButton(TaurusLauncherButton):
     _deleteWidgetOnClose = True
@@ -155,6 +168,7 @@ class _AbstractTaurusValueButton(TaurusLauncherButton):
     def getQtDesignerPluginInfo(cls):
         return None
     
+    
 class TaurusPlotButton(_AbstractTaurusValueButton):
     '''A button that launches a TaurusPlot'''
     _widgetClassName = 'TaurusPlot'
@@ -179,14 +193,16 @@ class TaurusValuesTableButton(_AbstractTaurusValueButton):
     _widgetClassName = 'TaurusValuesTable'
     _icon  = ':/designer/table.png'
 
+
 class TaurusValuesTableButton_W(TaurusValuesTableButton):
     '''A button that launches a TaurusValuesTable'''
     _text = 'Edit'
     _kwargs={'defaultWriteMode':True}
 
+
 class TaurusDevButton(_AbstractTaurusValueButton):
     '''A button that launches a TaurusAttrForm'''
-    _widgetClassName = 'TaurusAttrForm'
+    _widgetClassName = 'TaurusDevicePanel'
     _icon = ':/places/folder-remote.svg'
     _text = 'Show Device'
 
@@ -263,6 +279,8 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
         if parent is not None:
             self.setParent(parent)
             
+        self.registerConfigProperty(self.getLabelConfig, self.setLabelConfig, 'labeConfig')
+            
     def setVisible(self, visible):
         for w in (self.labelWidget(), self.readWidget(), self.writeWidget(), self.unitsWidget(), self.customWidget(), self.extraWidget()):
             if w is not None: w.setVisible(visible)
@@ -369,7 +387,7 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
             if returnAll: return [ExpandingLabel]
             else: return ExpandingLabel
         
-        if self.getModelClass() == taurus.core.TaurusAttribute:
+        if self.getModelClass() == taurus.core.taurusattribute.TaurusAttribute:
             ##The model is an attribute
             config = modelobj.getConfig()
             #print "---------ATTRIBUTE OBJECT:----------\n",modelobj.read()
@@ -390,15 +408,18 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
                 else:
                     result = [ExpandingLabel]
             elif config.isSpectrum():
-                if PyTango.is_numerical_type(configType):
+                if PyTango.is_numerical_type(configType): #@todo: tango-centric!!
                     result = [TaurusPlotButton, TaurusValuesTableButton, ExpandingLabel]
                 else:
                     result = [TaurusValuesTableButton, ExpandingLabel]
             elif config.isImage():
-                try: 
-                    from taurus.qt.qtgui.extra_guiqwt import TaurusImageDialog
-                    result = [TaurusImageButton, TaurusValuesTableButton, ExpandingLabel]
-                except ImportError:
+                if PyTango.is_numerical_type(configType): #@todo: tango-centric!!
+                    try: 
+                        from taurus.qt.qtgui.extra_guiqwt import TaurusImageDialog #unused import but useful to determine if TaurusImageButton should be added
+                        result = [TaurusImageButton, TaurusValuesTableButton, ExpandingLabel]
+                    except ImportError:
+                        result = [TaurusValuesTableButton, ExpandingLabel]
+                else:
                     result = [TaurusValuesTableButton, ExpandingLabel]
             else:
                 self.warning('Unsupported attribute type %s'%configType)
@@ -425,7 +446,7 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
                  default one.
         '''
 #        if self._customWidget is not None: return None
-        if self.isReadOnly() or self.getModelClass() != taurus.core.TaurusAttribute: 
+        if self.isReadOnly() or self.getModelClass() != taurus.core.taurusattribute.TaurusAttribute: 
             if returnAll: return []
             else: return None
         modelobj = self.getModelObj()
@@ -442,23 +463,32 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
             else:
                 result = [TaurusValueLineEdit, TaurusValueSpinBox, TaurusWheelEdit]
         elif config.isSpectrum():
-            result = [TaurusArrayEditorButton, TaurusValuesTableButton_W, TaurusValueLineEdit]
+            configType = config.getType()
+            if configType in (PyTango.ArgType.DevDouble, PyTango.ArgType.DevFloat, 
+                              PyTango.ArgType.DevInt, PyTango.ArgType.DevLong, 
+                              PyTango.ArgType.DevLong64, PyTango.ArgType.DevShort, 
+                              PyTango.ArgType.DevULong, PyTango.ArgType.DevULong64, 
+                              PyTango.ArgType.DevUShort):
+                result = [TaurusArrayEditorButton, TaurusValuesTableButton_W, TaurusValueLineEdit]
+            else:
+                result = [TaurusValuesTableButton_W, TaurusValueLineEdit]
         elif config.isImage():
             result = [TaurusValuesTableButton_W]
         else:
-            self.debug('Unsupported attribute type for writting: %s'% str(config.getType()))
+            self.debug('Unsupported attribute type for writing: %s'% str(config.getType()))
+            result = [None]
             
         if returnAll: return result
         else: return result[0]
     
     def getDefaultUnitsWidgetClass(self):
 ##        if self._customWidget is not None: return None
-#        if self.getModelClass() != taurus.core.TaurusAttribute:
+#        if self.getModelClass() != taurus.core.taurusattribute.TaurusAttribute:
 #            return DefaultUnitsWidget
         return DefaultUnitsWidget
     
     def getDefaultCustomWidgetClass(self):
-        if self.getModelClass() == taurus.core.TaurusAttribute:
+        if self.getModelClass() == taurus.core.taurusattribute.TaurusAttribute:
             return None
         try:
             key = self.getModelObj().getHWObj().info().dev_class
@@ -486,7 +516,23 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
                  values are widgets to be used
         '''
         return self._customWidgetMap
-    
+     
+    def onChangeLabelConfig(self):
+        keys = ['label', 'attr_name', 'attr_fullname', 'dev_alias', 'dev_name', 'dev_fullname']
+        try:
+            current = keys.index(self.labelConfig)
+        except:
+            current= len(keys)
+            keys.append(self.labelConfig)
+            
+        msg = 'Choose new source for the label. \n'+\
+              'You can also write a more complex text\n'+\
+              'using any of the proposed sources as a\n'+\
+              'placeholder by enclosing it in "< >" brackets'
+        labelConfig, ok = Qt.QInputDialog.getItem(self, 'Change Label', msg, keys, current, True)
+        if ok:
+            self.labelConfig=str(labelConfig)  
+             
     def onChangeReadWidget(self):
         classnames = ['None', 'Auto']+[c.__name__ for c in self.getDefaultReadWidgetClass(returnAll=True)]
         cname, ok = Qt.QInputDialog.getItem(self, 'Change Read Widget', 'Choose a new read widget class', classnames, 1, True)
@@ -526,41 +572,41 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
 
     def labelWidgetClassFactory(self, classID):
         if self._customWidget is not None: return None
-        if classID is None or classID is 'None': return None
+        if classID is None or classID == 'None': return None
         if isinstance(classID, type): return classID
         elif str(classID) == 'Auto': return self.getDefaultLabelWidgetClass()
         else: return TaurusWidgetFactory().getTaurusWidgetClass(classID)
 
     def readWidgetClassFactory(self, classID):
         if self._customWidget is not None: return None
-        if classID is None or classID is 'None': return None
+        if classID is None or classID == 'None': return None
         if isinstance(classID, type): return classID
         elif str(classID) == 'Auto': return self.getDefaultReadWidgetClass()
         else: return TaurusWidgetFactory().getTaurusWidgetClass(classID)
     
     def writeWidgetClassFactory(self, classID):
         if self._customWidget is not None: return None
-        if classID is None or classID is 'None': return None
+        if classID is None or classID == 'None': return None
         if isinstance(classID, type): return classID
         elif str(classID) == 'Auto': return self.getDefaultWriteWidgetClass()
         else: return TaurusWidgetFactory().getTaurusWidgetClass(classID)
         
     def unitsWidgetClassFactory(self, classID):
         if self._customWidget is not None: return None
-        if classID is None or classID is 'None': return None
+        if classID is None or classID == 'None': return None
         if isinstance(classID, type): return classID
         elif str(classID) == 'Auto': return self.getDefaultUnitsWidgetClass()
         else: return TaurusWidgetFactory().getTaurusWidgetClass(classID)
         
     def customWidgetClassFactory(self, classID):
-        if classID is None or classID is 'None': return None
+        if classID is None or classID == 'None': return None
         if isinstance(classID, type): return classID
         elif str(classID) == 'Auto': return self.getDefaultCustomWidgetClass()
         else: return TaurusWidgetFactory().getTaurusWidgetClass(classID)
         
     def extraWidgetClassFactory(self, classID):
         if self._customWidget is not None: return None
-        if classID is None or classID is 'None': return None
+        if classID is None or classID == 'None': return None
         if isinstance(classID, type): return classID
         elif str(classID) == 'Auto': return self.getDefaultExtraWidgetClass()
         else: return TaurusWidgetFactory().getTaurusWidgetClass(classID)
@@ -874,9 +920,9 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
     def setModel(self, model):
         try:
             taurus.Attribute(model)
-            self.__modelClass = taurus.core.TaurusAttribute
+            self.__modelClass = taurus.core.taurusattribute.TaurusAttribute
         except:
-            self.__modelClass = taurus.core.TaurusDevice
+            self.__modelClass = taurus.core.taurusdevice.TaurusDevice
         TaurusBaseWidget.setModel(self,model)
         if not self._designMode:     #in design mode, no subwidgets are created
             self.updateCustomWidget()
@@ -890,7 +936,7 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
         """Reimplemented from :meth:`TaurusBaseWidget.handleEvent` 
         to update subwidgets on config events
         """
-        if evt_type == taurus.core.TaurusEventType.Config and not self._designMode:
+        if evt_type == taurus.core.taurusbasetypes.TaurusEventType.Config and not self._designMode:
             self.updateCustomWidget()
             self.updateLabelWidget()
             self.updateReadWidget()
@@ -919,23 +965,22 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
             pass
         
     def hasPendingOperations(self):
-        '''self.getPendingOperations will alwaysd return an empty list, but still
+        '''self.getPendingOperations will always return an empty list, but still
         self.hasPendingOperations will look at the writeWidget's operations.
         If you want to ask the TaurusValue for its pending operations, call
         self.writeWidget().getPendingOperations()'''
         w = self.writeWidget()
-        if w is None: return []
+        if w is None: return False
         return w.hasPendingOperations()
-        
-    
+                
     def updatePendingOpsStyle(self):
         if self._labelWidget is None: return
         if self.hasPendingOperations():
             self._labelWidget.setStyleSheet(
-                'border-style: solid ; border-width: 1px; border-color: blue; color: blue; border-radius:4px;')
+                '%s {border-style: solid ; border-width: 1px; border-color: blue; color: blue; border-radius:4px;}'%self._labelWidget.__class__.__name__)
         else:
             self._labelWidget.setStyleSheet(
-                'border-style: solid; border-width: 1px; border-color: transparent; color: black;  border-radius:4px;')
+                '%s {border-style: solid; border-width: 1px; border-color: transparent; color: black;  border-radius:4px;}'%self._labelWidget.__class__.__name__)
             
     def getLabelConfig(self):
         return self._labelConfig
@@ -970,6 +1015,7 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
     allowWrite = Qt.pyqtProperty("bool", getAllowWrite, setAllowWrite, resetAllowWrite)
     modifiableByUser = Qt.pyqtProperty("bool", TaurusBaseWidget.isModifiableByUser, TaurusBaseWidget.setModifiableByUser, TaurusBaseWidget.resetModifiableByUser)
 
+
 class TaurusValuesFrame(TaurusFrame):
     '''This is a container specialized into containing TaurusValue widgets.
     It should be used Only for TaurusValues'''
@@ -1053,15 +1099,17 @@ if __name__ == "__main__":
     #models=['bl97/pc/dummy-01/CurrentSetpoint','bl97/pc/dummy-02/Current','bl97/pc/dummy-02/RemoteMode','bl97/pysignalsimulator/1/value1']
     #models=['bl97/pc/dummy-01/CurrentSetpoint','bl97/pc/dummy-02/RemoteMode']
     #models=['sys/tg_test/1/state','sys/tg_test/1/status','sys/tg_test/1/short_scalar','sys/tg_test/1']
-    #models =  ['sys/tg_test/1']+['sys/tg_test/1/%s_spectrum'%s for s in ('float','short','string','long','boolean') ]
+    #models =  ['sys/tg_test/1']+['sys/tg_test/1/%s_scalar'%s for s in ('float','short','string','long','boolean') ]
+    #models =  ['sys/tg_test/1/float_scalar','sys/tg#_test/1/double_scalar']
+    
+    container.setModel(models)
+    
+    #container.getItemByIndex(0).writeWidget().setDangerMessage('BOOO') #uncomment to test the dangerous operation support
+    #container.getItemByIndex(0).readWidget().setShowState(True)
+    #container.getItemByIndex(0).setWriteWidgetClass(TaurusValueLineEdit)
+    #container[0].setWriteWidgetClass('None')
     #container.setModel(models)
     
-    #container.getTaurusValueByIndex(0).writeWidget().setDangerMessage('BOOO') #uncomment to test the dangerous operation support
-    #container.getTaurusValueByIndex(0).readWidget().setShowState(True)
-    #container.getTaurusValueByIndex(0).setWriteWidgetClass(TaurusValueLineEdit)
-    #container.setModel(models)
-    
-
     container.setModifiableByUser(True)
     form.show()
 
