@@ -27,8 +27,7 @@
 qwtdialog.py: Dialogs for Taurusplot
 """
 
-from taurus.qt import Qt
-from PyQt4 import Qwt5
+from taurus.qt import Qt, Qwt5
 import time
 
 from ui.ui_TaurusPlotConfigDialog import Ui_TaurusPlotConfigDialog
@@ -186,11 +185,11 @@ class TaurusPlotConfigDialog(Qt.QDialog):
 #        self.connect(self.ui.y2EditMax, Qt.SIGNAL("returnPressed()"),self.apply)
         self.connect(self.ui.xModeComboBox, Qt.SIGNAL("currentIndexChanged(const QString&)"),self.modeComboChanged)
         self.connect(self.ui.xDynScaleCheckBox, Qt.SIGNAL("toggled (bool)"),self.setXDynScale)
-        self.connect(self.ui.xRangeCB, Qt.SIGNAL("currentIndexChanged(const QString&)"),self.apply)
+        #self.connect(self.ui.xRangeCB, Qt.SIGNAL("currentIndexChanged(const QString&)"),self.apply)
         self.connect(self.ui.y1ModeComboBox, Qt.SIGNAL("currentIndexChanged(const QString&)"),self.modeComboChanged)
         self.connect(self.ui.y2ModeComboBox, Qt.SIGNAL("currentIndexChanged(const QString&)"),self.modeComboChanged)
         self.connect(self.ui.peaksComboBox, Qt.SIGNAL("currentIndexChanged(int)"),self.peaksComboChanged)
-        self.connect(self.curvesAppearanceChooser, Qt.SIGNAL("controlChanged"),self.apply) #"autoapply" mode for *all* the curve appearance controls
+        #self.connect(self.curvesAppearanceChooser, Qt.SIGNAL("controlChanged"),self.apply) #"autoapply" mode for *all* the curve appearance controls
         self.connect(self.curvesAppearanceChooser.assignToY1BT, Qt.SIGNAL("clicked()"),self.setCurvesYAxis)
         self.connect(self.curvesAppearanceChooser.assignToY2BT, Qt.SIGNAL("clicked()"),self.setCurvesYAxis)
         self.connect(self.curvesAppearanceChooser.bckgndBT, Qt.SIGNAL("clicked()"),self.changeBackgroundColor)
@@ -204,19 +203,19 @@ class TaurusPlotConfigDialog(Qt.QDialog):
         xMin = self.parent.axisScaleDiv(Qwt5.QwtPlot.xBottom).lowerBound()
         xMax = self.parent.axisScaleDiv(Qwt5.QwtPlot.xBottom).upperBound()
         if self.parent.getXIsTime():
-                self.ui.xEditMin.setText(Qt.QString(time.strftime('%Y/%m/%d %H:%M:%S',time.localtime(int(xMin)))))
-                self.ui.xEditMax.setText(Qt.QString(time.strftime('%Y/%m/%d %H:%M:%S',time.localtime(int(xMax)))))
+                self.ui.xEditMin.setText(time.strftime('%Y/%m/%d %H:%M:%S',time.localtime(int(xMin))))
+                self.ui.xEditMax.setText(time.strftime('%Y/%m/%d %H:%M:%S',time.localtime(int(xMax))))
         else:
-                self.ui.xEditMin.setText(Qt.QString.number(xMin))
-                self.ui.xEditMax.setText(Qt.QString.number(xMax))
+                self.ui.xEditMin.setText(str(xMin))
+                self.ui.xEditMax.setText(str(xMax))
     
     def _populateYAxisScales(self, axis=None):
         if axis is None or axis==Qwt5.QwtPlot.yLeft:
-            self.ui.y1EditMin.setText(Qt.QString.number(self.parent.axisScaleDiv(Qwt5.QwtPlot.yLeft).lowerBound()))
-            self.ui.y1EditMax.setText(Qt.QString.number(self.parent.axisScaleDiv(Qwt5.QwtPlot.yLeft).upperBound()))
+            self.ui.y1EditMin.setText(str(self.parent.axisScaleDiv(Qwt5.QwtPlot.yLeft).lowerBound()))
+            self.ui.y1EditMax.setText(str(self.parent.axisScaleDiv(Qwt5.QwtPlot.yLeft).upperBound()))
         if axis is None or axis==Qwt5.QwtPlot.yRight:
-            self.ui.y2EditMin.setText(Qt.QString.number(self.parent.axisScaleDiv(Qwt5.QwtPlot.yRight).lowerBound()))
-            self.ui.y2EditMax.setText(Qt.QString.number(self.parent.axisScaleDiv(Qwt5.QwtPlot.yRight).upperBound()))
+            self.ui.y2EditMin.setText(str(self.parent.axisScaleDiv(Qwt5.QwtPlot.yRight).lowerBound()))
+            self.ui.y2EditMax.setText(str(self.parent.axisScaleDiv(Qwt5.QwtPlot.yRight).upperBound()))
     
     def deltatime2str(self,dt, fuzzy=False):
         '''converts a time diff in secs to a string.
@@ -316,7 +315,10 @@ class TaurusPlotConfigDialog(Qt.QDialog):
                 rg = abs(self.str2deltatime(str(self.ui.xRangeCB.currentText())))
                 ok1= (rg is not None)
             else:
-                rg,ok1=self.ui.xRangeCB.currentText().toDouble()
+                try:
+                    rg, ok1 = float(self.ui.xRangeCB.currentText()), True
+                except:
+                    rg, ok1 = 0.0, False
             xMin=xMax-rg
             if not ok1 or (xMin >= xMax):
                 Qt.QMessageBox.warning(self, "Invalid parameters for X axis range")
@@ -328,8 +330,14 @@ class TaurusPlotConfigDialog(Qt.QDialog):
                 ok1= (xMin is not None)
                 ok2= (xMax is not None)
             else: #XY mode
-                xMin,ok1 = self.ui.xEditMin.text().toDouble()
-                xMax,ok2 = self.ui.xEditMax.text().toDouble()
+                try:
+                    xMin, ok1 = float(self.ui.xEditMin.text()), True
+                except:
+                    xMin, ok1 = 0.0, False
+                try:
+                    xMax, ok2 = float(self.ui.xEditMax.text()), True
+                except:
+                    xMax, ok2 = 0.0, False
             if not (ok1 and ok2) or (xMin >= xMax):
                 Qt.QMessageBox.warning(self, "Invalid parameters for X axis",
                                         "Min value must be strictly less than max value")
@@ -337,8 +345,14 @@ class TaurusPlotConfigDialog(Qt.QDialog):
             
         #Validating Y1 axis values
         if self.ui.y1AutoGroupBox.isChecked():
-            y1Min,ok1=self.ui.y1EditMin.text().toDouble()
-            y1Max,ok2=self.ui.y1EditMax.text().toDouble()
+            try:
+                y1Min, ok1 = float(self.ui.y1EditMin.text()), True
+            except:
+                y1Min, ok1 = 0.0, False
+            try:
+                y1Max, ok2 = float(self.ui.y1EditMax.text()), True
+            except:
+                y1Max, ok2 = 0.0, False
             if not(ok1 and ok2) or (y1Min >= y1Max):
                 Qt.QMessageBox.warning(self, "Invalid parameters for Y1 axis",
                                         "Min value must be strictly less than max value")
@@ -346,8 +360,14 @@ class TaurusPlotConfigDialog(Qt.QDialog):
         
         #Validating Y2 axis values
         if self.ui.y2AutoGroupBox.isChecked():
-            y2Min,ok1=self.ui.y2EditMin.text().toDouble()
-            y2Max,ok2=self.ui.y2EditMax.text().toDouble()
+            try:
+                y2Min, ok1 = float(self.ui.y2EditMin.text()), True
+            except:
+                y2Min, ok1 = 0.0, False
+            try:
+                y2Max, ok2 = float(self.ui.y2EditMax.text()), True
+            except:
+                y2Max, ok2 = 0.0, False
             if not (ok1 and ok2) or (y2Min >= y2Max):
                 Qt.QMessageBox.warning(self, "Invalid parameters for Y2 axis",
                                         "Min value must be strictly less than max value")
@@ -482,4 +502,4 @@ class TaurusPlotConfigDialog(Qt.QDialog):
         :param name: (QString) new title
         '''
         newTitlesDict = self.parent.setCurvesTitle(newTitle, [name])
-        self.curvesAppearanceChooser.updateTitles(newTitlesDict)
\ No newline at end of file
+        self.curvesAppearanceChooser.updateTitles(newTitlesDict)
