@@ -30,11 +30,9 @@ taurusdevicepropertytable.py:
 __all__ = ["TaurusPropTable"]
 
 from taurus.qt import Qt, QtCore, QtGui
-from PyQt4 import Qwt5
 from taurus.qt.qtgui.base import TaurusBaseWidget
 import taurus.core
 import PyTango
-import traceback
 
 class TaurusPropTable(QtGui.QTableWidget, TaurusBaseWidget):
     ''' 
@@ -69,7 +67,7 @@ class TaurusPropTable(QtGui.QTableWidget, TaurusBaseWidget):
         return QtGui.QTableWidget.minimumSizeHint(self)
 
     def getModelClass(self):
-        return taurus.core.TaurusDatabase
+        return taurus.core.taurusdatabase.TaurusDatabase
 
     @classmethod
     def getQtDesignerPluginInfo(cls):
@@ -125,6 +123,7 @@ class TaurusPropTable(QtGui.QTableWidget, TaurusBaseWidget):
         
         self.updateStyle()
         self.dev_name = dev_name
+        self.setWindowTitle('%s Properties'%dev_name)
         self.resizeColumnsToContents()
         self.resizeRowsToContents()        
         ##Signals @todo
@@ -134,6 +133,7 @@ class TaurusPropTable(QtGui.QTableWidget, TaurusBaseWidget):
 
     def defineStyle(self):
         """ Defines the initial style for the widget """
+        self.setWindowTitle('Properties')
         self.setColumnCount(2)
         self.setRowCount(0)
         self.setGeometry(QtCore.QRect(0,0,400,500))
@@ -380,4 +380,4 @@ if __name__ == '__main__':
     widget.setTable(sys.args)
     widget.show()
     app.exec_()
-		
\ No newline at end of file
+		
