@@ -37,13 +37,15 @@ __docformat__ = 'restructuredtext'
 import re
 import operator
 import traceback
+import Queue
 from functools import partial
 
-from taurus.qt import QtGui, QtCore
+from taurus.qt import Qt, QtGui, QtCore
 
 import taurus
-from taurus.core import TaurusManager
-from taurus.core.util import Logger
+from taurus.qt.qtcore.util.emitter import modelSetter,TaurusEmitterThread,SingletonWorker,MethodModel
+from taurus.core.taurusmanager import TaurusManager
+from taurus.core.util.log import Logger
 from taurus.qt.qtgui.base import TaurusBaseWidget
 from taurus.qt.qtgui.panel import TaurusValue
 from taurus.qt.qtgui.display import TaurusValueLabel,TaurusStateLabel
@@ -58,30 +60,34 @@ def get_all_models(expressions,limit=1000):
     '''
     All devices matching expressions must be obtained.
     For each device only the good attributes are read.
+    
+    It practically equals to fandango.get_matching_attributes; check which is better!
+    Move this method to taurus.core.tango.search 
     '''
-    print( 'In TaurusGrid.get_all_models(%s:"%s") ...' % (type(expressions),expressions))
+    #print( 'In TaurusGrid.get_all_models(%s:"%s") ...' % (type(expressions),expressions))
     if isinstance(expressions,str):
-        if any(re.match(s,expressions) for s in ('\{.*\}','\(.*\)','\[.*\]')):
-            #self.debug( 'evaluating expressions ....')
-            expressions = list(eval(expressions))
-        else:
-            #self.debug( 'expressions as string separated by commas ...')
-            expressions = expressions.split(',')
+        #if any(re.match(s,expressions) for s in ('\{.*\}','\(.*\)','\[.*\]')):
+            ##self.debug( 'evaluating expressions ....')
+            #expressions = list(eval(expressions))
+        #else:
+            ##self.debug( 'expressions as string separated by commas ...')
+        expressions = expressions.split(',')
             
     elif any(isinstance(expressions,klass) for klass in (QtCore.QStringList,list,tuple,dict)):
         #self.debug( 'expressions converted from list ...')
         expressions = list(str(e) for e in expressions)
         
     #self.debug( 'In TaurusGrid.get_all_models(%s:"%s") ...' % (type(expressions),expressions))
-    taurus_db = taurus.core.TaurusManager().getFactory()().getDatabase()
+    taurus_db = taurus.core.taurusmanager.TaurusManager().getFactory()().getDatabase()
+    #taurus_db = taurus.Database(os.environ['TANGO_HOST'])
     if 'SimulationDatabase' in str(type(taurus_db)):
-        #self.info( 'Using a simulated database ...')
+        #self.trace( 'Using a simulated database ...')
         models = expressions
     else:
         all_devs = taurus_db.get_device_exported('*')
         models = []
         for exp in expressions:
-            #self.info( 'evaluating exp = "%s"' % exp)
+            #self.trace( 'evaluating exp = "%s"' % exp)
             exp = str(exp)
             devs = []
             targets = []
@@ -94,15 +100,14 @@ def get_all_models(expressions,limit=1000):
             else:
                 devs = [device]
                 
-            #self.info( 'TaurusGrid.get_all_models(): devices matched by %s / %s are %d:' % (device,attribute,len(devs)))
+            #self.trace( 'TaurusGrid.get_all_models(): devices matched by %s / %s are %d:' % (device,attribute,len(devs)))
             #self.debug( '%s' % (devs))
             for dev in devs:
                 if any(c in attribute for c in '.*[]()+?'):
                     if '*' in attribute and '.*' not in attribute: attribute = attribute.replace('*','.*')
-                    #taurus_dp = taurus.core.TaurusManager().getFactory()().getDevice( 'test/sim/sergi')
                     try:
-                        #taurus_dp = taurus.core.TaurusDevice(dev)
-                        taurus_dp = taurus.core.TaurusManager().getFactory()().getDevice(dev)
+                        #taurus_dp = taurus.core.taurusdevice.TaurusDevice(dev)
+                        taurus_dp = taurus.core.taurusmanager.TaurusManager().getFactory()().getDevice(dev)
                         #self.debug( "taurus_dp = %s"%taurus_dp.getFullName())
                         attrs = [att.name for att in taurus_dp.attribute_list_query() if re_match_low(attribute,att.name)]
                         targets.extend(dev+'/'+att for att in attrs)
@@ -113,7 +118,7 @@ def get_all_models(expressions,limit=1000):
             #print 'TaurusGrid.get_all_models(): targets added by %s are: %s' % (exp,targets)
             models.extend(targets)
     models = models[:limit]
-    print( 'Out of TaurusGrid.get_all_models(...)')
+    #print( 'Out of TaurusGrid.get_all_models(...)')
     return models   
                     
 def get_readwrite_models(expressions,limit=1000):
@@ -124,16 +129,16 @@ def get_readwrite_models(expressions,limit=1000):
     #self.debug( 'In TaurusGrid.get_all_models(%s:"%s") ...' % (type(expressions),expressions))
     if isinstance(expressions,str):
         if any(re.match(s,expressions) for s in ('\{.*\}','\(.*\)','\[.*\]')):
-            #self.debug( 'evaluating expressions ....')
+            #self.trace( 'evaluating expressions ....')
             expressions = list(eval(expressions))
         else:
-            #self.debug( 'expressions as string separated by commas ...')
+            #self.trace( 'expressions as string separated by commas ...')
             expressions = expressions.split(',')
             
     elif any(isinstance(expressions,klass) for klass in (QtCore.QStringList,list,tuple,dict)):
         expressions = list(str(e) for e in expressions)
         
-    taurus_db = taurus.core.TaurusManager().getFactory()().getDatabase()
+    taurus_db = taurus.core.taurusmanager.TaurusManager().getFactory()().getDatabase()
     if 'SimulationDatabase' in str(type(taurus_db)):
       models = expressions
     else:
@@ -155,15 +160,10 @@ def get_readwrite_models(expressions,limit=1000):
           for dev in devs:
               if any(c in attribute for c in '.*[]()+?'):
                   if '*' in attribute and '.*' not in attribute: attribute = attribute.replace('*','.*')
-                  #taurus_dp = taurus.core.TaurusManager().getFactory()().getDevice( 'test/sim/sergi')
                   try: 
-                      taurus_dp = taurus.core.TaurusManager().getFactory()().getDevice(dev)
+                      taurus_dp = taurus.core.taurusmanager.TaurusManager().getFactory()().getDevice(dev)
                       attrs = [att.name for att in taurus_dp.attribute_list_query() if re_match_low(attribute,att.name) and att.isReadOnly()]
                       targets.extend(dev+'/'+att for att in attrs)
-#                      for att in attrs:
-#                          if taurus.core.TaurusManager().getFactory()().getAttribute(dev+'/'+att).isReadOnly:
-#                              continue
-#                          targets.extend(dev+'/'+att)
                   except Exception,e: 
                     pass
               else: targets.append(dev+'/'+attribute)
@@ -171,14 +171,6 @@ def get_readwrite_models(expressions,limit=1000):
     models = models[:limit]
     return models
 
-def modelSetter(args):
-    print '#'*80
-    obj,model = args[0],args[1]
-    print 'In modelSetter(%s,%s)' % (str(obj),str(model))
-    obj.setModel(model)
-    print 'Out of modelSetter(%s,%s)' % (str(obj),str(model))
-
-
 class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
     """ TaurusGrid is a Taurus widget designed to represent a set of attributes distributed in columns and rows.
     The Model will be a list with attributes or device names (for devices the State attribute will be shown).
@@ -196,7 +188,7 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
     #---------------------------------------------------------------------------
     # Write your own code here to define the signals generated by this widget
     #
-    __pyqtSignals__ = ("modelChanged(const QString &)",)
+    __pyqtSignals__ = ("modelChanged(const QString &)","itemSelected(QString)")
     
     _TAGS = ['DOMAIN','FAMILY','HOST','LEVEL','CLASS','ATTRIBUTE','DEVICE']
     
@@ -216,28 +208,56 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
         self._modelNames = []
         self.row_labels = []
         self.column_labels = []
-        self.defineStyle()
-        self._others = False
         self._widgets_list = []
+        self._last_selected = None
+        self._show_frames = True
+        self._show_row_frame = True
+        self._show_column_frame = True
+        self._show_others = False
+        self._show_attr_labels = True
+        self._show_attr_units = True
         self.hideLabels=False
+        
+        self.defineStyle()
         self.modelsQueue = Queue.Queue()
-        self.modelsThread = TaurusEmitterThread(parent=self,queue=self.modelsQueue,method=modelSetter )
+        #self.modelsThread = TaurusEmitterThread(parent=self,queue=self.modelsQueue,method=modelSetter )
+        self.modelsThread = SingletonWorker(parent=self,name='TaurusGrid',queue=self.modelsQueue,method=modelSetter,cursor=True)
         
     def save(self,filename):
         import pickle
-        d = {'model':self.filter,'row_labels':self.row_labels,'column_labels':self.column_labels}
+        d = {
+            'model':self.filter,
+            'row_labels':self.row_labels,'column_labels':self.column_labels,
+            'frames':self._show_row_frame or self._show_column_frame,'labels':self._show_attr_labels,
+            'units':self._show_attr_units,'others':self._show_others
+            }
         f = open(filename,'w')
         pickle.dump(d,f)
         f.close()
         
-    def load(self,filename):
-        import pickle
-        f = open(filename)
-        d = pickle.load(f)
-        f.close()
+    def load(self,filename,delayed=False):
+        self.trace('In TauGrid.load(%s,%s)'%(filename,delayed))
+        if not isinstance(filename,dict):
+            import pickle
+            f = open(filename)
+            d = pickle.load(f)
+            f.close()
+        else: 
+            d = filename
         self.setRowLabels(d['row_labels'])
-        self.setColumnLabels(d['column_labels'])        
-        self.setModel(d['model'])
+        self.setColumnLabels(d['column_labels'])  
+        #self._show_attr_labels = d.get('labels',True) #self.showAttributeLabels(d.get('labels',True))
+        #self._show_attr_units = d.get('units',True) #self.showAttributeUnits(d.get('units',True))
+        #self._show_others = d.get('others',True)
+        #self._show_row_frame = d.get('frames',True)
+        #self._show_column_frame = d.get('frames',True)
+        self.showAttributeLabels(d.get('labels',True))
+        self.showAttributeUnits(d.get('units',True))
+        self.showOthers(d.get('others',True))
+        self.showRowFrame(d.get('frames',True))
+        self.showColumnFrame(d.get('frames',True))
+        
+        self.setModel(d['model'],delayed=d.get('delayed',delayed))
         return self._modelNames 
     
     def defineStyle(self):
@@ -263,7 +283,7 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
         #-----------------------------------------------------------------------
         # [MANDATORY]
         # Replace your own code here
-        # ex.: return taurus.core.Attribute
+        # ex.: return taurus.core.taurusattribute.Attribute
         raise RuntimeError("Forgot to overwrite %s.getModelClass" % str(self)) 
         return list
     
@@ -317,12 +337,14 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
     def updateStyle(self):
         #-----------------------------------------------------------------------
         # Write your own code here to update your widget style
-        self.debug('@'*80)
-        self.debug('In TaurusGrid.updateStyle() ....... It seems never called!!!!')
-        self.debug('@'*80)
+        self.trace('@'*80)
+        self.trace('In TaurusGrid.updateStyle() ....... It seems never called!!!!')
+        self.trace('@'*80)
+        
+        #It was showing an annoying "True" in the widget
+        #value = self.getShowText() or ''
+        #if self._setText: self._setText(value) ##It must be included
         
-        value = self.getShowText() or ''
-        if self._setText: self._setText(value) ##It must be included
         #update tooltip
         self.setToolTip(self.getFormatedToolTip()) ##It must be included
                 
@@ -330,45 +352,67 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
         if hasattr(self,'title_widget'):
             if self.title: self.title_widget.show()
             else: self.title_widget.hide()
-        #if hasattr(self,'table'):
-            #self.table.resizeColumnsToContents()
-            #self.table.resizeRowsToContents()            
-            
-        if hasattr(self,'modelsThread') and not self.modelsThread.isRunning(): 
-            self.debug('<'*80)
-            self.modelsThread.start()#self.modelsThread.IdlePriority)
-            
+                       
         self.update()
         
     #---------------------------------------------------------------------------
     # Write your own code here for your own widget properties
     
-    @QtCore.pyqtSignature("setModel(QStringList)")
-    def setModel(self,model,devsInRows=False):
-        '''The model can be initialized as a list of devices or hosts or ...'''
+    def setModel(self,model,devsInRows=False,delayed=False,append=False,load=True):
+        '''The model can be initialized as a list of devices or hosts or dictionary or ...'''
         #self.setModelCheck(model) ##It must be included
         #differenciate if the model is a RegExp
-        model = isinstance(model,(str,QtCore.QString)) and [model] or list(model)
-        self.debug('#'*80)
-        self.debug('In TaurusGrid.setModel(%s)'%model)
-        self.filter = model
-        if any('*' in m for m in model):
-            model = get_all_models(model)
-            self.debug('model was a RegExp, done the query and converted to an attr list')
-
-        if not self._modelNames == []:#clean to start from scratch
-            self._modelNames = []
-            for widget in self._widgets_list:
-                del widget
-
-        #here we always have the reals model list, even if it comes from a regexp
-        self._modelNames = model
-        self.debug('In setModel(%s): modelNames are %s'%(model,self._modelNames))
-        if devsInRows:
-            self.setRowLabels(','.join(set(d.rsplit('/',1)[0] for d in self._modelNames)))
-        self.create_widgets_table(self._modelNames)
-        self.debug('Out of TaurusGrid.setModel(%s)'%model)
-        self.updateStyle()
+        if isinstance(model,dict):
+            self.load(model)
+        else:
+            model = isinstance(model,(str,QtCore.QString)) and [model] or list(model)
+            self.trace('#'*80)
+            self.trace('In TaurusGrid.setModel(%s)'%str(model)[:100])
+    
+            self.delayed = delayed
+            self.filter = model
+            if any('*' in m for m in model):
+                model = get_all_models(model)
+                self.debug('model was a RegExp, done the query and converted to an attr list')
+    
+            if not self._modelNames == []:#clean to start from scratch
+                for widget in self._widgets_list:
+                    del widget
+    
+            #here we always have the reals model list, even if it comes from a regexp
+            if append: self._modelNames = self._modelNames+model
+            else: self._modelNames = model
+            
+            self.debug(('In TaurusGrid.setModel(...): modelNames are %s'%(self._modelNames))[:100]+'...')
+            
+            if load:
+                self.trace('In TaurusGrid.setModel(%s,load=True): modelNames are %d'%(str(model)[:100]+'...',len(self._modelNames)))#,self._modelNames)) 
+                if devsInRows:
+                    self.setRowLabels(','.join(set(d.rsplit('/',1)[0] for d in self._modelNames)))
+                self.create_widgets_table(self._modelNames)
+                self.modelsQueue.put((MethodModel(self.showRowFrame),self._show_row_frame))
+                self.modelsQueue.put((MethodModel(self.showColumnFrame),self._show_column_frame))
+                self.modelsQueue.put((MethodModel(self.showOthers),self._show_others))
+                self.modelsQueue.put((MethodModel(self.showAttributeLabels),self._show_attr_labels))
+                self.modelsQueue.put((MethodModel(self.showAttributeUnits),self._show_attr_units))
+                self.updateStyle()        
+                        
+                if not self.delayed:
+                    self.trace('In setModel(): not delayed loading of models')
+                    if not self.modelsThread.isRunning(): 
+                        #print 'In setModel(): Starting Thread! (%d objs in queue)'%(self.modelsThread.queue.qsize())
+                        self.trace('<'*80)
+                        self.modelsThread.start()#self.modelsThread.IdlePriority)        
+                    else:
+                        #print 'In setModel(): Thread already started! (%d objs in queue)'%(self.modelsThread.queue.qsize())
+                        self.modelsThread.next()
+                else: 
+                    self.trace('In setModel(): models loading delayed!')
+                    pass
+                    
+            self.trace('Out of TaurusGrid.setModel(%s)'%str(model)[:100])
+            self.updateStyle()
+        return
     
     def getModel(self):
         return self._modelNames
@@ -378,7 +422,7 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
         self.updateFromList(self._modelNames)
         return
     
-    @QtCore.pyqtSignature("setTitle(QString)")    
+   
     def setTitle(self,title):
         self.title = str(title)
         if hasattr(self,'title_widget'):
@@ -396,11 +440,10 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
                 self.warning('ERROR! Unable to parse labels property: %s'%str(e))
                 return []
         else:
-            exprs = text.split(',')
-            labels = [(':' in e and tuple(e.split(':',1)) or (e,e)) for e in exprs]
+            exprs = [t.strip() for t in text.split(',')]
+            labels = [(':' in e and (e.split(':',1)[0].strip(),e.split(':',1)[-1].strip()) or (e,e)) for e in exprs]
             return labels
         
-    @QtCore.pyqtSignature("setRowLabels(QStringList)")
     def setRowLabels(self,rows):
         '''The model can be initialized as a list of devices or hosts or ...'''
         #self.setModelCheck(model) ##It must be included
@@ -421,7 +464,6 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
         self.row_labels = []
         return       
     
-    @QtCore.pyqtSignature("setColumnLabels(QStringList)")
     def setColumnLabels(self,columns):
         '''The model can be initialized as a list of devices or hosts or ...'''
         #self.setModelCheck(model) ##It must be included
@@ -441,6 +483,46 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
     def resetColumnLabels(self):
         self.column_labels = []
         return        
+    
+    #FIXME: when they are called before setModel they fails because 
+    # frames are not yet created, and it doesn't has memoty about this.
+    def showRowFrame(self,boolean):
+        self._show_row_frame = boolean
+        if hasattr(self,'rows_frame'):
+            if boolean:
+                self.rows_frame.show()
+            else:
+                self.rows_frame.hide()
+                
+    def showColumnFrame(self,boolean):
+        self._show_column_frame = boolean
+        if hasattr(self,'columns_frame'):
+            if boolean:
+                self.columns_frame.show()
+            else:
+                self.columns_frame.hide()
+                
+    def showAttributeLabels(self,boolean):
+        self.trace('In showAttributeLabels(%s)'%boolean)
+        self._show_attr_labels = boolean
+        for tv in self._widgets_list:
+            try:
+                if tv and tv.labelWidget:
+                    if boolean: tv.labelWidget().show()
+                    else: tv.labelWidget().hide()
+            except: pass
+        return self._show_attr_labels
+        
+    def showAttributeUnits(self,boolean):
+        self.trace('In showAttributeUnits(%s)'%boolean)
+        self._show_attr_units = boolean
+        for tv in self._widgets_list:
+            try:
+                if tv and tv.unitsWidget:
+                    if boolean: tv.unitsWidget().show()
+                    else: tv.unitsWidget().hide()
+            except: pass
+        return self._show_attr_units
 
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
     # QT properties 
@@ -576,10 +658,12 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
         
         self.rows_frame = self.create_frame_with_gridlayout()
         self.rows_frame.setFrameStyle(QtGui.QFrame.Box)
+        if not self._show_row_frame: self.rows_frame.hide()
         self.checkboxes_frame.layout().addWidget(self.rows_frame,0,0)
         
         self.columns_frame = self.create_frame_with_gridlayout()
         self.columns_frame.setFrameStyle(QtGui.QFrame.Box)
+        if not self._show_column_frame: self.columns_frame.hide()
         self.checkboxes_frame.layout().addWidget(self.columns_frame,0,1)
         
         layout_row = 0
@@ -591,7 +675,7 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
             checkbox = QtGui.QCheckBox(section)
             if checkbox.text() == 'Others':
                 checkbox.setChecked(False)
-                if not self._others: checkbox.hide()
+                if not self._show_others: checkbox.hide()
             else:
                 checkbox.setChecked(True)
             self.rows_frame.layout().addWidget(checkbox,layout_row,layout_col)
@@ -611,7 +695,7 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
             checkbox = QtGui.QCheckBox(column)
             if checkbox.text() == 'Others':
                 checkbox.setChecked(False)
-                if not self._others: checkbox.hide()
+                if not self._show_others: checkbox.hide()
             else:
                 checkbox.setChecked(True)
             self.columns_frame.layout().addWidget(checkbox,layout_row,layout_col)
@@ -650,43 +734,34 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
                     self.table.hideColumn(table_col)
 
     def showOthers(self,boolean):
-        self._others = boolean
+        self._show_others = boolean
         if hasattr(self,'rows_frame'):
             for checkbox in self.rows_frame.children():
                 if isinstance(checkbox,QtGui.QCheckBox) and checkbox.text() == 'Others':
-                    if self._others: checkbox.show()
+                    if self._show_others: checkbox.show()
                     else: checkbox.hide()
         if hasattr(self,'columns_frame'):
             for checkbox in self.columns_frame.children():
                 if isinstance(checkbox,QtGui.QCheckBox) and checkbox.text() == 'Others':
-                    if self._others: checkbox.show()
+                    if self._show_others: checkbox.show()
                     else: checkbox.hide()
 
-    #FIXME: when they are called before setModel they fails because 
-    # frames are not yet created, and it doesn't has memoty about this.
-    def showRowFrame(self,boolean):
-        if boolean == True:
-            self.rows_frame.show()
-        else:
-            self.rows_frame.hide()
-    def showColumnFrame(self,boolean):
-        if boolean == True:
-            self.columns_frame.show()
-        else:
-            self.columns_frame.hide()
-
     def build_table(self,values):
         """
         This is a builder. For all the elements in widgets matrix,
         just set the corresponding cells of the QTableWidget.
         """
-        self.debug('In TaurusGrid.build_table(%s)'%values)
+        self.trace('In TaurusGrid.build_table(%s)'%values)
         widgets_matrix = self.build_widgets(values,self.showLabels)
         rows = len(widgets_matrix)
-        cols = len(widgets_matrix[0])
+        cols = rows and len(widgets_matrix[0]) or 0
         
         table = QtGui.QTableWidget()
         table.setItemDelegate(Delegate(table))
+        # This example replaces the blue background of selected cells in tables
+        palette = Qt.QPalette()
+        palette.setBrush(palette.Active,palette.Highlight,Qt.QBrush(Qt.Qt.white))
+        table.setPalette(palette)        
         
         table.setRowCount(rows)
         table.setColumnCount(cols)
@@ -710,14 +785,14 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
             for cell in row:
                 cell_frame = self.create_frame_with_gridlayout()
                 count = 0
-                for synoptic in cell:
+                for synoptic in sorted(cell):
                     self.debug("processing synoptic %s"%synoptic)
                     name = model = synoptic
 
                     self.debug('Creating TaurusValue with model =  %s'%model)
                     synoptic_value = TaurusValue(cell_frame)    
-                    #synoptic_value.setModel(model)
                     self.modelsQueue.put((synoptic_value,model))
+                    QtCore.QObject.connect(synoptic_value, QtCore.SIGNAL("itemClicked(QString)"), self.itemClicked)
                             
                     if self.hideLabels:
                         synoptic_value.setLabelWidgetClass(None)
@@ -726,15 +801,51 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
                     cell_frame.layout().addWidget(synoptic_value,count,0)
                     self._widgets_list.append(synoptic_value)
                     count += 1
+                    
+                #Done in this way as TauValue.mousePressEvent are never called
+                def mousePressEvent(event,obj):
+                    #print 'In cell clicked'
+                    targets = set(str(child.getModelName()) for child in obj.children() 
+                        if hasattr(child,'underMouse') and child.underMouse() and hasattr(child,'getModelName'))
+                    [obj.emit(Qt.SIGNAL("itemClicked(QString)"),t) for t in targets]
+                
+                cell_frame.mousePressEvent = partial(mousePressEvent,obj=cell_frame)
+                QtCore.QObject.connect(cell_frame, QtCore.SIGNAL("itemClicked(QString)"), self.itemClicked)
+                
                 widgets_row.append(cell_frame)
             widgets_matrix.append(widgets_row)
         return widgets_matrix
+        
+    def itemClicked(self,item_name):
+        self.trace('In TaurusGrid.itemClicked(%s)'%item_name)
+        self.setItemSelected(item_name)
+        self.emit(QtCore.SIGNAL("itemClicked(QString)"),str(item_name))
+        
+    def setItemSelected(self,item_name='',selected=True):
+        """ it adds a blue frame around a clicked item. """
+        if isinstance(item_name,TaurusValue): 
+            self.trace('In TaurusGrid.setItemSelected(%s,%s)'%(str(item_name.getModel()),selected))
+            item = item_name
+        else: 
+            self.trace('In TaurusGrid.setItemSelected(%s,%s)'%(str(item_name),selected))
+            if item_name: item = self.getItemByModel(item_name)
+            else: item = self._last_selected
+        if item:
+            if selected:
+                item._labelWidget.setStyleSheet('border-style: solid ; border-width: 1px; border-color: blue; color: blue; border-radius:4px;')
+                if self._last_selected and self._last_selected!=item:
+                    self.setItemSelected(self._last_selected,False)
+                self._last_selected = item
+            else:
+                item._labelWidget.setStyleSheet('border-style: solid; border-width: 1px; border-color: transparent; color: black;  border-radius:4px;')        
+                self._last_selected = None
+        else: return None        
     
     def getItemByModel(self, model, index=0):
         #a particular model can be many times, index means which one of them
+        model = str(model).lower()
         for widget in self._widgets_list:
-            #@todo taurus model comparision?
-            if widget.getModel().lower() == model.lower():
+            if str(widget.getModel()).lower() == model:
                 if index <= 0:
                     return widget
                 else:
@@ -748,78 +859,6 @@ class TaurusGrid(QtGui.QFrame, TaurusBaseWidget):
         ret['icon'] = ":/designer/grid.png"
         return ret
 
-import Queue,traceback
-class TaurusEmitterThread(QtCore.QThread):
-    """
-    This object get items from a python Queue and performs a thread safe operation on them.
-    It is useful to delay signals in a background thread.
-    :param parent: a Qt/Taurus object
-    :param queue: if None parent.getQueue() is used, if not then the queue passed as argument is used
-    :param target: the method to be executed using each queue item as argument
-    """
-    def __init__(self, parent=None,queue=None,method=None):
-        """Parent most not be None and must be a TaurusGraphicsScene!"""
-        #if not isinstance(parent, TaurusGraphicsScene):
-            #raise RuntimeError("Illegal parent for TaurusGraphicsUpdateThread")
-        QtCore.QThread.__init__(self, parent)
-        self.log = Logger('TaurusEmitterThread')
-        self.queue = queue
-        self.todo = Queue.Queue()
-        self.method = method
-        self.emitter = QtCore.QObject(QtGui.QApplication.instance())
-        self._done = 0
-        QtCore.QObject.connect(self.emitter, QtCore.SIGNAL("doSomething"), self._doSomething)
-        QtCore.QObject.connect(self.emitter, QtCore.SIGNAL("somethingDone"), self._next)
-        self.emitter.moveToThread(QtGui.QApplication.instance().thread())        
-        
-    def getQueue(self):
-        if self.queue: return self.queue
-        elif hasattr(self.parent(),'getQueue'): self.parent().getQueue()
-        else: return None
-
-    def getDone(self):
-        """ Returns % of done tasks in 0-1 range """
-        return self._done/(self._done+self.getQueue().qsize()) if self._done else 0.
-
-    def _doSomething(self,args):
-        print '#'*80
-        self.log.debug('At TaurusEmitterThread._doSomething(%s)'%str(args))
-        if self.method: 
-            try:
-                self.method(args)
-            except:
-                self.log.error('At TaurusEmitterThread._doSomething(): %s' % traceback.format_exc())
-        self.emitter.emit(QtCore.SIGNAL("somethingDone"))
-        self._done += 1
-        return
-        
-    def _next(self):
-        queue = self.getQueue()
-        self.log.debug('At TaurusEmitterThread._next(), %d items remaining.' % queue.qsize())
-        if not queue.empty():
-            try:
-                item = queue.get(False) #A blocking get here would hang the GUIs!!!
-                self.todo.put(item)
-            except Queue.Empty,e:
-                pass
-        return
-        
-    def run(self):
-        print '#'*80
-        self.log.debug('At TaurusEmitterThread.run()')
-        self._next()
-        while True:
-            item = self.todo.get(True)
-            if type(item) in taurus.core.util.types.StringTypes:
-                if item == "exit":
-                    break
-                else:
-                    continue
-            self.emitter.emit(QtCore.SIGNAL("doSomething"), item)
-            #End of while
-        self.log.info('Out of TaurusEmitterThread.run()')
-        #End of Thread                    
-    
 class Delegate(QtGui.QItemDelegate):
     
     def __init__(self, parent=None):
@@ -839,7 +878,7 @@ def sysargs_to_dict(defaults=[]):
             bar = a.split('=')
             if bar[1] in ['True','False']:#convert string to boolean
                 bar[1] = eval(bar[1])
-            result[bar[0]] = bar[1]
+            result[bar[0].replace('--','')] = bar[1]
         else:
             result[defaults[i]] = a
             i+=1
