@@ -33,11 +33,12 @@ import numpy
 import sys
 
 import taurus.core
+from taurus.core.taurusbasetypes import DataFormat
 from taurus.qt.qtgui.base import TaurusBaseWidget, TaurusBaseWritableWidget
 from taurus.qt.qtgui.display import TaurusLabel
 from taurus.qt.qtgui.resource import getThemeIcon, getThemePixmap
 from taurus.qt.qtgui.container import TaurusWidget
-from taurus.core.util import Enumeration
+from taurus.core.util.enumeration import Enumeration
 
 TableRWState = Enumeration("TableRWState", ("Read", "Write"))
 
@@ -95,7 +96,8 @@ class TaurusValuesIOTableModel(Qt.QAbstractTableModel):
         elif role == Qt.Qt.DecorationRole:
             status = self.getStatus(index)
             if (self._modifiedDict.has_key((index.row(), index.column()))) and (self._writeMode):
-                if self.inAlarmRange(self._modifiedDict[(index.row(), index.column())].toDouble()[0]):
+                float_data = Qt.from_qvariant(self._modifiedDict[(index.row(), index.column())], float)
+                if self.inAlarmRange(float_data):
                     icon = getThemeIcon('document-save')
                     #return Qt.QVariant(Qt.QColor('blue'))
                 else:
@@ -118,7 +120,8 @@ class TaurusValuesIOTableModel(Qt.QAbstractTableModel):
                 return Qt.QVariant(Qt.QColor('white'))
         elif role == Qt.Qt.ForegroundRole:
             if self._modifiedDict.has_key((index.row(), index.column())) and (self._writeMode):
-                if self.inAlarmRange(self._modifiedDict[(index.row(), index.column())].toDouble()[0]):
+                float_data = Qt.from_qvariant(self._modifiedDict[(index.row(), index.column())], float)
+                if self.inAlarmRange(float_data):
                     return Qt.QVariant(Qt.QColor('blue'))
                 else:
                     return Qt.QVariant(Qt.QColor('orange'))
@@ -128,8 +131,9 @@ class TaurusValuesIOTableModel(Qt.QAbstractTableModel):
                 return Qt.QVariant(Qt.QFont("Arial", 10, Qt.QFont.Bold))
         elif role == Qt.Qt.ToolTipRole:
             if self._modifiedDict.has_key((index.row(), index.column())) and (self._writeMode):
+                float_data = Qt.from_qvariant(self._modifiedDict[index.row(), index.column()], float)
                 return Qt.QVariant('Original value: %d.\nNew value that will be saved: %d' 
-                                   %(tabledata[index.row(), index.column()],self._modifiedDict[index.row(), index.column()].toDouble()[0]))
+                                   %(tabledata[index.row(), index.column()], float_data))
         return Qt.QVariant()
     
     def setAttr(self, attr):
@@ -142,9 +146,9 @@ class TaurusValuesIOTableModel(Qt.QAbstractTableModel):
         values = numpy.array(attr.value)
         wvalues = numpy.array(attr.w_value)
         #reshape the table
-        if attr.data_format == taurus.core.DataFormat._1D:
+        if attr.data_format == DataFormat._1D:
             rows, columns = values.size, 1
-        elif attr.data_format == taurus.core.DataFormat._2D:
+        elif attr.data_format == DataFormat._2D:
             rows, columns = values.shape
         else:
             raise TypeError('Unsupported data format "%s"'%repr(attr.data_format))
@@ -171,7 +175,7 @@ class TaurusValuesIOTableModel(Qt.QAbstractTableModel):
         '''
         Returns the configuration object for the data
         
-        :returns:  (TaurusConfiguration)
+        :returns:  (taurus.core.taurusconfiguration.TaurusConfiguration)
         '''
         return self._attrConfig
     
@@ -179,7 +183,7 @@ class TaurusValuesIOTableModel(Qt.QAbstractTableModel):
         '''
         Returns Status of the variable
         
-        :returns:  (taurus.core.AttrQuality) 
+        :returns:  (taurus.core.taurusbasetypes.AttrQuality) 
         '''
         return self._attr.quality
     
@@ -228,20 +232,20 @@ class TaurusValuesIOTableModel(Qt.QAbstractTableModel):
         if kind in 'SU':
             table = table.tolist() #we want to allow the strings to be larger than the original ones
             for (r,c),v in self._modifiedDict.items():
-                table[r][c] = str(v.toString())
+                table[r][c] = Qt.from_qvariant(v, str)
             table = numpy.array(table)
         else:        
             for k,v in self._modifiedDict.items():
                 if kind == 'f':
-                    table[k],ok = v.toDouble()
+                    table[k] = Qt.from_qvariant(v, float)
                 elif kind in 'iu':
-                    table[k],ok = v.toInt()
+                    table[k] = Qt.from_qvariant(v, int)
                 elif kind == 'b':
-                    table[k] = v.toBool()
+                    table[k] = Qt.from_qvariant(v, bool)
                 else:
                     raise TypeError('Unknown data type "%s"'%kind)
         #reshape if needed
-        if self._attr.data_format == taurus.core.DataFormat._1D:
+        if self._attr.data_format == DataFormat._1D:
             table = table.flatten()
         return table    
      
@@ -292,9 +296,9 @@ class TaurusValuesIOTableModel(Qt.QAbstractTableModel):
             #refresh the write data (unless it is dirty)
             wvalues=numpy.array(self._attr.w_value)
             #reshape the table
-            if self._attr.data_format == taurus.core.DataFormat._1D:
+            if self._attr.data_format == DataFormat._1D:
                 rows, columns = wvalues.size, 1
-            elif self._attr.data_format == taurus.core.DataFormat._2D:
+            elif self._attr.data_format == DataFormat._2D:
                 rows, columns = wvalues.shape
             else:
                 self.warning('unsupported data format %s'%str(val.data_format))
@@ -410,13 +414,13 @@ class TaurusValuesIOTableDelegate(Qt.QStyledItemDelegate):
         self._initialText = None
         if index.model().getType() == bool:
             editor.addItems(['true', 'false'])
-            a = str(index.data().toBool()).lower()
+            a = str(Qt.from_qvariant(index.data(), bool)).lower()
             self._initialText = a
             editor.setCurrentIndex(editor.findText(a))
         else:
             data = index.model().data(index, Qt.Qt.EditRole)
-            self._initialText = data.toString()
-            editor.setText(data.toString())
+            self._initialText = Qt.from_qvariant(data, str)
+            editor.setText(self._initialText)
     
     def setModelData(self, editor, model,index):
         '''
@@ -461,7 +465,10 @@ class TableInlineEdit(Qt.QLineEdit):
         see :meth:`Qt.QLineEdit.textEdited`
         '''
         color, weight = 'gray', 'normal' #default case: the value is in normal range with no pending changes
-        v, b = self.displayText().toDouble()
+        try:
+            v = float(self.displayText())
+        except:
+            v = 0.0
         try:
             if float(self._attrConfig.getMinAlarm()) <= v <= float(self._attrConfig.getMaxAlarm()): #the value is invalid and can't be applied
                 color = 'blue'
@@ -565,15 +572,26 @@ class TaurusValuesTable(TaurusWidget):
     
     def getModelClass(self):
         '''see :meth:`TaurusWidget.getModelClass`'''
-        return taurus.core.TaurusAttribute    
+        return taurus.core.taurusattribute.TaurusAttribute    
     
     def setModel(self, model):
         '''Reimplemented from :meth:`TaurusWidget.setModel`'''
         TaurusWidget.setModel(self, model)
         value = self.getModelValueObj()
-        if value is not None: 
-            self._tableView.setModel([value.dim_x, value.dim_y])
-        self._label.setModel(value)
+        if value is not None:
+            try: 
+                dim_x,dim_y = value.dim_x, value.dim_y #@this is tango-centric. dim_x and dim_y attribute is not present in TaurusConfiguration
+            except:
+                v = numpy.array(value.value)
+                if v.ndim == 1:
+                    dim_x, dim_y = v.shape[0], 1
+                elif v.ndim == 2:
+                    dim_x,dim_y = v.shape
+                else:
+                    self.error('Cannot display %i-dimensional data', v.ndim)
+                    return
+            self._tableView.setModel([dim_x, dim_y]) 
+        self._label.setModel(model)
 
     def handleEvent(self, evt_src, evt_type, evt_value):
         '''see :meth:`TaurusWidget.handleEvent`'''
@@ -581,10 +599,10 @@ class TaurusValuesTable(TaurusWidget):
         model = self._tableView.model()
         if model is None:
             return
-        if evt_type in (taurus.core.TaurusEventType.Change, taurus.core.TaurusEventType.Periodic) and evt_value is not None:            
+        if evt_type in (taurus.core.taurusbasetypes.TaurusEventType.Change, taurus.core.taurusbasetypes.TaurusEventType.Periodic) and evt_value is not None:            
             model.setAttr(evt_value)
             self._tableView.resizeColumnsToContents()
-        elif evt_type == taurus.core.TaurusEventType.Config:
+        elif evt_type == taurus.core.taurusbasetypes.TaurusEventType.Config:
             #force a read to set an attr
             model.setAttr(self.getModelValueObj())
             model.setConfig(evt_src)
@@ -704,7 +722,7 @@ class TaurusValuesTable(TaurusWidget):
     def chooseModel(self):
         '''shows a model chooser'''
         from taurus.qt.qtgui.panel import  TaurusModelChooser
-        selectables=[taurus.core.TaurusElementType.Attribute]
+        selectables=[taurus.core.taurusbasetypes.TaurusElementType.Attribute]
         models, ok = TaurusModelChooser.modelChooserDlg(selectables=selectables, singleModel=True)
         if ok and len(models)==1:
             self.setModel(models[0])
@@ -764,4 +782,4 @@ if __name__ == '__main__':
     taurusTableMain()
     
     
-    
\ No newline at end of file
+    
