@@ -44,7 +44,7 @@ import taurus.qt.qtgui.panel
 import taurus.qt.qtgui.taurusgui.paneldescriptionwizard
 import taurus.qt.qtgui.input
 import copy
-from taurus.core.util import etree
+from lxml import etree
 from taurus.core.util import Enumeration
 from taurus.qt.qtgui.util import ExternalAppAction
         
@@ -223,8 +223,8 @@ class ProjectPage(BasePage):
         Qt.QObject.connect(self._projectDirBT, Qt.SIGNAL("clicked()"), self.onSelectDir)       
         
     def onSelectDir(self):
-        dirname = Qt.QFileDialog.getExistingDirectory(self, 'Choose the project directory', self._projectDirLE.text())
-        if dirname.isNull(): return
+        dirname = unicode(Qt.QFileDialog.getExistingDirectory(self, 'Choose the project directory', self._projectDirLE.text()))
+        if not dirname: return
         self._projectDirLE.setText(dirname)
         
     def validatePage(self):
