@@ -39,18 +39,10 @@ from taurus.qt.qtcore.communication import SharedDataManager
 from taurus.qt.qtgui.util import TaurusWidgetFactory
 from taurus.qt.qtgui.base import TaurusBaseWidget, TaurusBaseComponent
 from taurus.qt.qtgui.container import TaurusMainWindow
-from taurus.qt.qtgui.input import GraphicalChoiceDlg
-
-try:
-    from taurus.qt.qtgui.extra_pool import PoolMotorSlim, IORegisterTV, PoolChannelTV
-    HAS_EXTRA_POOL = True
-except ImportError:
-    HAS_EXTRA_POOL = False
-
 from taurus.qt.qtgui.taurusgui.utils import ExternalApp, PanelDescription, ToolBarDescription, AppletDescription
-from PermanentCustomPanelsDlg import PermanentCustomPanelsDlg
+from taurus.qt.qtgui.panel import QDoubleListDlg
 import taurus.qt.qtgui.resource
-from taurus.core.util import etree
+from lxml import etree
 
 
 class AssociationDialog(Qt.QDialog):
@@ -70,7 +62,6 @@ class AssociationDialog(Qt.QDialog):
         
     def refresh(self):
         currentinstrument = self.ui.instrumentCB.currentText()
-        currentinstrumentIdx= self.ui.instrumentCB.currentIndex()
         mainwindow = self.parent()
         
         self.associations = mainwindow.getAllInstrumentAssociations()
@@ -144,10 +135,21 @@ class DockWidgetPanel(Qt.QDockWidget, TaurusBaseWidget):
     def setPermanent(self, permanent):
         self._permanent = permanent
     
-    def setWidgetFromClassName(self, classname):
+    def setWidgetFromClassName(self, classname, modulename=None):
         if self.getWidgetClassName() != classname:
-            klass = TaurusWidgetFactory().getWidgetClass(classname)
-            w = klass()
+            try:
+                klass = TaurusWidgetFactory().getWidgetClass(classname)
+                w = klass()
+            except:
+                try:
+                    if classname is not None and '.' in classname:
+                        mn,classname = classname.rsplit('.',1)
+                        modulename = ("%s.%s"%(modulename or '',mn)).strip('. ')
+                    module = __import__(modulename, fromlist=[''])
+                    klass = getattr(module, classname)
+                    w = klass()
+                except Exception, e:
+                    raise RuntimeError('Cannot create widget from classname "%s". Reason: %s'%(classname, repr(e)))
             #set customwidgetmap if necessary
             if hasattr(w,'setCustomWidgetMap'):
                 w.setCustomWidgetMap(self._mainwindow.getCustomWidgetMap())
@@ -155,6 +157,12 @@ class DockWidgetPanel(Qt.QDockWidget, TaurusBaseWidget):
             wname="%s-%s"%(str(self.objectName()), str(classname))
             w.setObjectName(wname)
             
+    def getWidgetModuleName(self):
+        w = self.widget()
+        if w is None:
+            return ''
+        return w.__module__
+            
     def getWidgetClassName(self):
         w = self.widget()
         if w is None:
@@ -163,18 +171,20 @@ class DockWidgetPanel(Qt.QDockWidget, TaurusBaseWidget):
         
     def applyConfig(self, configdict, depth=-1):
         #create the widget
-        try: 
-            self.setWidgetFromClassName(configdict.get('widgetClassName'))
+        try:
+            self.setWidgetFromClassName(configdict.get('widgetClassName'), modulename=configdict.get('widgetModuleName',None))
             if isinstance(self.widget(),BaseConfigurableClass):
                 self.widget().applyConfig(configdict['widget'])
         except Exception,e:
-            self.info('Failed to set the widget for this panel. Reason: %s'%repr(e)) 
+            self.info('Failed to set the widget for this panel. Reason: %s'%repr(e))
+            self.traceback(self.Debug)
             return
         TaurusBaseWidget.applyConfig(self, configdict, depth)
         
     def createConfig(self, *args, **kwargs):
         configdict = TaurusBaseWidget.createConfig(self, *args, **kwargs)
         configdict['widgetClassName'] = self.getWidgetClassName()
+        configdict['widgetModuleName'] = self.getWidgetModuleName()
         if isinstance(self.widget(),BaseConfigurableClass):
             configdict['widget'] = self.widget().createConfig()
         return configdict
@@ -197,21 +207,23 @@ class TaurusGui(TaurusMainWindow):
     
     IMPLICIT_ASSOCIATION = '__[IMPLICIT]__'
     
-    def __init__(self, parent=None, confname=None):
+    def __init__(self, parent=None, confname=None, configRecursionDepth=None):
         TaurusMainWindow.__init__(self, parent, False, True)
         
+        if configRecursionDepth is not None:
+            self.defaultConfigRecursionDepth = configRecursionDepth
+            
         self.__panels = {}   
         self.__synoptics = []
         self.__instrumentToPanelMap = {}
         self.__panelToInstrumentMap = {}
-        
         self.setDockNestingEnabled(True)
         
         self.registerConfigProperty(self._getPermanentCustomPanels, self._setPermanentCustomPanels, 'permanentCustomPanels')
         self.registerConfigProperty(self.getAllInstrumentAssociations, self.setAllInstrumentAssociations, 'instrumentAssociation')
-        
-        from taurus.TaurusCustomSettings import T_FORM_CUSTOM_WIDGET_MAP
-        self.setCustomWidgetMap(T_FORM_CUSTOM_WIDGET_MAP)
+                
+        from taurus import tauruscustomsettings
+        self.setCustomWidgetMap(getattr(tauruscustomsettings,'T_FORM_CUSTOM_WIDGET_MAP',{}))
         
         #Create a global SharedDataManager
         Qt.qApp.SDM =  SharedDataManager(self)
@@ -223,11 +235,9 @@ class TaurusGui(TaurusMainWindow):
         self.__initJorgBar()  
         self.__initSharedDataConnections()
         self.__initToolsMenu()
-                
+                    
         self.loadConfiguration(confname)
-        
-        self.splashScreen().finish(self)
-        
+                
         #connect the main window itself as a reader/writer of "short messages"
         Qt.qApp.SDM.connectReader("shortMessage", self.onShortMessage) 
         Qt.qApp.SDM.connectWriter("shortMessage", self, 'newShortMessage')
@@ -236,6 +246,14 @@ class TaurusGui(TaurusMainWindow):
         msg = '%s is ready'%Qt.qApp.applicationName()
         self.emit(Qt.SIGNAL('newShortMessage'), msg)
         
+        if self.defaultConfigRecursionDepth >=0:
+            Qt.QMessageBox.information(self, "Fail-proof mode", 
+                                   ('Running in fail-proof mode.'+
+                                    '\nLoading of potentially problematic settings is disabled.'+ 
+                                    '\nSome panels may not be loaded or may ignore previous user configuration'+
+                                    '\nThis will also apply when loading perspectives'), 
+                                   Qt.QMessageBox.Ok, Qt.QMessageBox.NoButton)
+        
     def closeEvent(self, event):
         try:
             self.__macroBroker.removeTemporaryPanels()
@@ -273,7 +291,6 @@ class TaurusGui(TaurusMainWindow):
         
     def __initViewMenu(self):
         self.viewMenu.addSeparator() #the superclass may already have added stuff to the viewMenu
-        self.viewMenu.addMenu(self.__panelsMenu)
         #view locking
         self.viewMenu.addSeparator()
         self._lockviewAction = Qt.QAction(taurus.qt.qtgui.resource.getThemeIcon("system-lock-screen"),"Lock View", self)
@@ -305,17 +322,23 @@ class TaurusGui(TaurusMainWindow):
         
     def __initSharedDataConnections(self):        
         #register the TAURUSGUI itself as a writer/reader for several shared data items
-        self.splashScreen().showMessage("setting up shared data connections")
+        splashScreen = self.splashScreen()
+        if splashScreen is not None:
+            self.splashScreen().showMessage("setting up shared data connections")
         Qt.qApp.SDM.connectWriter("macroserverName", self, 'macroserverNameChanged')
         Qt.qApp.SDM.connectWriter("doorName", self, 'doorNameChanged')
         Qt.qApp.SDM.connectReader("SelectedInstrument", self.onSelectedInstrument)
         Qt.qApp.SDM.connectWriter("SelectedInstrument", self, 'SelectedInstrument')
         Qt.qApp.SDM.connectReader("executionStarted", self.setFocusToPanel)
-
+        Qt.qApp.SDM.connectReader("selectedPerspective", self.loadPerspective)
+        Qt.qApp.SDM.connectWriter("perspectiveChanged", self, 'perspectiveChanged')
+        
     def __initToolsMenu(self):
         if self.toolsMenu is None:
             self.toolsMenu = Qt.QMenu("Tools")
         self.toolsMenu.addAction(taurus.qt.qtgui.resource.getIcon(":/apps/preferences-system-session.svg"),"manage instrument-panel associations", self.onShowAssociationDialog)
+        self.toolsMenu.addAction(taurus.qt.qtgui.resource.getThemeIcon("document-save"),"Export current Panel configuration to XML", self.onExportCurrentPanelConfiguration)
+        self.toolsMenu.addAction(taurus.qt.qtgui.resource.getIcon(":/actions/data-transfer.svg"),"Show Shared Data Manager connections", self.showSDMInfo)
         
     def setCustomWidgetMap(self, map):
         '''
@@ -346,17 +369,22 @@ class TaurusGui(TaurusMainWindow):
     def createConfig(self, *args, **kwargs):
         '''reimplemented from TaurusMainWindow.createConfig'''
         self.updatePermanentCustomPanels(showAlways=False)
-        return TaurusMainWindow.createConfig(self, *args, **kwargs)        
- 
+        return TaurusMainWindow.createConfig(self, *args, **kwargs)   
+    
     def removePanel(self, name=None):
-        ''' remove the given panel from the GUI
+        ''' remove the given panel from the GUI.
+        
+        .. note:: The panel; is actually removed from the current perspective.
+                  If the panel is saved in other perspectives, it should be removed from
+                  them as well.
         
         :param name: (str or None) the name of the panel to be removed
                      If None given, the user will be prompted
         '''
         if name is None:
-            items = sorted(self.getPanelNames())
-            name,ok = Qt.QInputDialog.getItem (self, "Remove Panel", "Panel to be removed", items, 0, False)
+            items = sorted([n for n,p in self.__panels.iteritems() if p.isCustom()])
+            name,ok = Qt.QInputDialog.getItem (self, "Remove Panel", 
+                                               "Panel to be removed (only custom panels can be removed).\n Important: you may want to save the perspective afterwards,\n and maybe remove the panel from other perspectives as well", items, 0, False)
             if not ok:
                 return
         name = unicode(name)
@@ -364,11 +392,17 @@ class TaurusGui(TaurusMainWindow):
             self.debug('Cannot remove panel "%s" (not found)'%name)
             return
         panel = self.__panels.pop(name)
+        try:
+            panel.widget().setModel(None) #in case the widget is a Taurus one and does some cleaning when setting model to None
+        except:
+            pass
+        
         self.unregisterConfigurableItem(name, raiseOnError=False)
         self.removeDockWidget(panel)
         panel.setParent(None)
         panel.setAttribute(Qt.Qt.WA_DeleteOnClose)
         panel.close()
+        self.debug('Panel "%s" removed'%name)
         
     def createPanel(self, widget, name, floating=False, registerconfig=True, custom=False, 
                     permanent=False, icon=None, instrumentkey=None):
@@ -488,18 +522,20 @@ class TaurusGui(TaurusMainWindow):
         perm = self._getPermanentCustomPanels()
         temp = [n for n,p in self.__panels.iteritems() if (p.isCustom() and not p.isPermanent())]
         if len(temp)>0 or showAlways:
-            dlg = PermanentCustomPanelsDlg(temporaryList=temp, permanentList=perm)
+            dlg = QDoubleListDlg(winTitle='Stored panels', 
+                     mainLabel='Select which of the panels should be stored',
+                     label1='Temporary (to be discarded)', label2='Permanent (to be stored)', 
+                     list1=temp, list2=perm )
             result = dlg.exec_()
             if result == Qt.QDialog.Accepted:
                 #update the permanent Custom Panels
                 registered = self.getConfigurableItemNames()
-                perm = dlg.getPermanentPanels()
-                for name in perm:
+                for name in dlg.getAll2():
                     if name not in registered:
                         self.__panels[name].setPermanent(True)
                         self.registerConfigDelegate(self.__panels[name], name)
                 #unregister any panel that is temporary
-                for name in dlg.getTemporaryPanels():
+                for name in dlg.getAll1():
                     self.__panels[name].setPermanent(False)
                     self.unregisterConfigurableItem(name, raiseOnError=False)
             
@@ -514,7 +550,7 @@ class TaurusGui(TaurusMainWindow):
 
         if paneldesc is None:
             from taurus.qt.qtgui.taurusgui import PanelDescriptionWizard
-            paneldesc,ok = PanelDescriptionWizard.getDialog(self)
+            paneldesc,ok = PanelDescriptionWizard.getDialog(self, extraWidgets=self._extraCatalogWidgets)
             if not ok: 
                 return
         w = paneldesc.getWidget(sdm=Qt.qApp.SDM, setModel=False)
@@ -544,7 +580,7 @@ class TaurusGui(TaurusMainWindow):
             synoptic.setModel(jdwFileName)
             self.__synoptics.append(synoptic)
         except Exception,e:
-            print repr(e)
+            #print repr(e)
             msg='Error loading synoptic file "%s".\nSynoptic won\'t be available'%jdwFileName
             self.error(msg)
             self.traceback(level=taurus.Info)
@@ -569,13 +605,26 @@ class TaurusGui(TaurusMainWindow):
                                          icon=taurus.qt.qtgui.resource.getThemeIcon('image-x-generic'))
         toggleSynopticAction = synopticpanel.toggleViewAction()
         self.quickAccessToolBar.addAction(toggleSynopticAction)
-                
+    
+    def createConsole(self, kernels):
+        try:
+            from taurus.qt.qtgui.console import TaurusConsole
+        except ImportError:
+            self.warning('Cannot import taurus.qt.qtgui.console. The Console Panel will not be available')
+            return
+        console = TaurusConsole(kernels=kernels)
+        consolePanel = self.createPanel(console, "Console", permanent=True,
+            icon=taurus.qt.qtgui.resource.getThemeIcon('utilities-terminal'))
+        toggleConsoleAction = consolePanel.toggleViewAction()
+        self.quickAccessToolBar.addAction(toggleConsoleAction)
+    
     def createInstrumentsFromPool(self, macroservername):
-        '''creates a list of instrument objects
+        '''
+        Creates a list of instrument panel descriptions by gathering the info
+        from the Pool. Each panel is a TaurusForm grouping together all those
+        elements that belong to the same instrument according to the Pool info
         
-        :return: (list<object>) A list of instrument objects. Each instrument
-                 object has "name" (str) and "model" (list<str>) members
-                 
+        :return: (list<PanelDescription>)                 
         '''      
         instrument_dict = {}
         try:
@@ -595,28 +644,19 @@ class TaurusGui(TaurusMainWindow):
             i_view = PanelDescription(i_name,classname='TaurusForm', floating=False, model=[])
             instrument_dict[i_name] = i_view
         
-#        motors = sorted(ms.getElementNamesWithInterface('Moveable'))
-#        channels = sorted(ms.getElementNamesWithInterface('ExpChannel'))
-#        ioregisters = sorted(ms.getElementNamesWithInterface('IORegister'))
-        
-#        pool_elements = motors + channels + ioregisters
-#        for e_name in pool_elements:
-#            e = taurus.Device(e_name)
-#            instrument = e['Instrument'].value
-#            if instrument != '':
-#                i_name = instrument
-#                e_name = e.alias()
-#                instrument_dict[i_name].model.append(e_name)
-
-        pool_elements = ms.getElementsWithInterfaces(('Moveable', 'ExpChannel', 'IORegister'))
-        for elem_name, elem in pool_elements.items():
+        from operator import attrgetter
+        pool_elements =  sorted(ms.getElementsWithInterface('Moveable').values(), key=attrgetter('name'))
+        pool_elements += sorted(ms.getElementsWithInterface('ExpChannel').values(), key=attrgetter('name'))
+        pool_elements += sorted(ms.getElementsWithInterface('IORegister').values(), key=attrgetter('name'))
+        for elem in pool_elements:
             instrument = elem.instrument
             if instrument:
                 i_name = instrument
                 e_name = elem.full_name
                 instrument_dict[i_name].model.append(e_name)
-
-        return instrument_dict.values()
+        #filter out empty panels
+        ret =  [instrument for instrument in instrument_dict.values() if len(instrument.model)>0]
+        return ret
 
     def __getVarFromXML(self, root, nodename, default=None):
         name = root.find(nodename)
@@ -643,8 +683,8 @@ class TaurusGui(TaurusMainWindow):
                 import imp
                 path, name = os.path.split(confname)
                 name, ext = os.path.splitext(name) 
-                file, filename, data = imp.find_module(name, [path])
-                conf = imp.load_module(name, file, filename, data)
+                f, filename, data = imp.find_module(name, [path])
+                conf = imp.load_module(name, f, filename, data)
             else: #if confname is not a dir name, we assume it is a module name in the python path
                 confsubdir = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'conf') #the path to a conf subdirectory of the place where taurusgui.py is
                 oldpath = sys.path
@@ -662,7 +702,8 @@ class TaurusGui(TaurusMainWindow):
                         sys.exit()
                 sys.path = oldpath #restore the previous sys.path
         except Exception, e:
-            msg = 'Error loading configuration: %s'%repr(e)
+            import traceback
+            msg = 'Error loading configuration: %s'%traceback.format_exc() #repr(e)
             self.error(msg)
             Qt.QMessageBox.critical(self,'Initialization error', msg, Qt.QMessageBox.Abort)
             sys.exit()
@@ -672,6 +713,10 @@ class TaurusGui(TaurusMainWindow):
         
         #Get the xml root node from the xml configuration file 
         XML_CONFIG = getattr(conf,'XML_CONFIG', None)
+        if XML_CONFIG is None:
+            self._xmlConfigFileName = None
+        else:
+            self._xmlConfigFileName = os.path.join(self._confDirectory, XML_CONFIG)
         xmlroot = etree.fromstring('<root></root>') #default fallback (in case of I/O or parse errors)
         if XML_CONFIG is not None:
             try:
@@ -699,6 +744,13 @@ class TaurusGui(TaurusMainWindow):
         Qt.qApp.setApplicationName(APPNAME)
         Qt.qApp.setOrganizationName(ORGNAME)
         Qt.QApplication.instance().basicConfig()
+
+        ORGANIZATIONLOGO =  getattr(conf, 'ORGANIZATION_LOGO', self.__getVarFromXML(xmlroot,"ORGANIZATION_LOGO", ':/logo.png'))
+        ##
+        if ORGANIZATIONLOGO.startswith(':'):
+            organizationIcon = taurus.qt.qtgui.resource.getIcon(ORGANIZATIONLOGO)
+        else:
+            organizationIcon = Qt.QIcon(os.path.join(self._confDirectory, ORGANIZATIONLOGO))
         
         #if required, enforce that only one instance of this GUI can be run
         SINGLEINSTANCE = getattr(conf,'SINGLE_INSTANCE', (self.__getVarFromXML(xmlroot,"SINGLE_INSTANCE", 'True').lower() == 'true') )
@@ -713,9 +765,18 @@ class TaurusGui(TaurusMainWindow):
         self.resetQSettings() 
         self.setWindowTitle(APPNAME)
         self.setWindowIcon(customIcon)
-        self.jorgsBar.addAction(taurus.qt.qtgui.resource.getIcon(":/logo.png"),ORGNAME)
+        self.jorgsBar.addAction(organizationIcon,ORGNAME)
         self.jorgsBar.addAction(customIcon,APPNAME)
         
+        #get custom widget catalog entries
+        EXTRA_CATALOG_WIDGETS = getattr(conf,'EXTRA_CATALOG_WIDGETS', []) #@todo: support also loading from xml
+        self._extraCatalogWidgets = []
+        for classname,pixmapname in EXTRA_CATALOG_WIDGETS:
+            if pixmapname and not pixmapname.startswith(":"): # If a relative file name is given, the conf directory will be used as base path
+                pixmapname = os.path.join(self._confDirectory,pixmapname)
+            self._extraCatalogWidgets.append((classname, pixmapname))
+                
+        
         #manual panel
         MANUAL_URI = getattr(conf,'MANUAL_URI', self.__getVarFromXML(xmlroot,"MANUAL_URI", None))
         if MANUAL_URI is not None:
@@ -725,7 +786,8 @@ class TaurusGui(TaurusMainWindow):
                     
         #configure the macro infrastructure       
         MACROSERVER_NAME = getattr(conf,'MACROSERVER_NAME', self.__getVarFromXML(xmlroot,"MACROSERVER_NAME", None))
-        if MACROSERVER_NAME is not None:
+        MACRO_PANELS = getattr(conf,'MACRO_PANELS', self.__getVarFromXML(xmlroot,"MACRO_PANELS", True))
+        if MACRO_PANELS and MACROSERVER_NAME is not None:# macro infrastructure will only be created if MACROSERVER_NAME is set
             from taurus.qt.qtgui.taurusgui import MacroBroker
             self.__macroBroker =  MacroBroker(self)
         if MACROSERVER_NAME: 
@@ -740,7 +802,7 @@ class TaurusGui(TaurusMainWindow):
             from taurus.qt.qtgui.extra_macroexecutor.macroparameterseditor.macroparameterseditor import ParamEditorManager
             ParamEditorManager().parsePaths(MACROEDITORS_PATH)
             ParamEditorManager().browsePaths()
-            
+        
         #Synoptics          
         SYNOPTIC = getattr(conf, 'SYNOPTIC', None)
         if isinstance(SYNOPTIC, basestring): #old config file style
@@ -751,19 +813,25 @@ class TaurusGui(TaurusMainWindow):
             node = xmlroot.find("SYNOPTIC")
             if (node is not None) and (node.text is not None):
                 for child in node:
-                    str = child.get("str")
-                    if str is not None and len(str): #we do not append empty strings
-                        SYNOPTIC.append(str)
+                    s = child.get("str")
+                    if s is not None and len(s): #we do not append empty strings
+                        SYNOPTIC.append(s)
         for s in SYNOPTIC:
             self.createMainSynoptic(s)
             
-        #Get panel descriptions from pool if required                         
+        #Get panel descriptions from pool if required            
         INSTRUMENTS_FROM_POOL = getattr(conf,'INSTRUMENTS_FROM_POOL', (self.__getVarFromXML(xmlroot,"INSTRUMENTS_FROM_POOL", 'False').lower() == 'true') )                   
         if INSTRUMENTS_FROM_POOL:
+            try: self.splashScreen().showMessage("Gathering Instrument info from Pool")
+            except AttributeError: pass            
             POOLINSTRUMENTS = self.createInstrumentsFromPool(MACROSERVER_NAME) #auto create instruments from pool 
         else:
             POOLINSTRUMENTS = []
-        
+
+        CONSOLE = getattr(conf,'CONSOLE', self.__getVarFromXML(xmlroot,"CONSOLE", ['ipython']))
+        if CONSOLE:
+            self.createConsole(CONSOLE)
+       
         #get custom panel descriptions from the python config file      
         CUSTOM_PANELS = [obj for name,obj in inspect.getmembers(conf) if isinstance(obj, PanelDescription)]
         
@@ -779,16 +847,18 @@ class TaurusGui(TaurusMainWindow):
         #create panels based on the panel descriptions gathered before
         for p in CUSTOM_PANELS + POOLINSTRUMENTS:
             try:
+                try: self.splashScreen().showMessage("Creating panel %s"%p.name)
+                except AttributeError: pass 
                 w = p.getWidget(sdm=Qt.qApp.SDM, setModel=False)
                 if hasattr(w,'setCustomWidgetMap'):
                     w.setCustomWidgetMap(self.getCustomWidgetMap())
                 if p.model is not None:
                     w.setModel(p.model)
                 if p.instrumentkey is None:
-                    instrumentkey = self.IMPLICIT_ASSOCIATION
+                    instrumentkey = self.IMPLICIT_ASSOCIATION 
+                registerconfig = p not in POOLINSTRUMENTS #the pool instruments may change when the pool config changes, so we do not store their config
                 #create a panel
-                self.createPanel(w, p.name, floating=p.floating, instrumentkey=instrumentkey, permanent=True)
-                
+                self.createPanel(w, p.name, floating=p.floating, registerconfig=registerconfig, instrumentkey=instrumentkey, permanent=True)
             except Exception,e:
                 msg='Cannot create panel %s'%getattr(p,'name','__Unknown__')
                 self.error(msg)
@@ -812,6 +882,8 @@ class TaurusGui(TaurusMainWindow):
         #create toolbars based on the descriptions gathered before
         for d in CUSTOM_TOOLBARS:
             try:
+                try: self.splashScreen().showMessage("Creating Toolbar %s"%d.name)
+                except AttributeError: pass 
                 w = d.getWidget(sdm=Qt.qApp.SDM, setModel=False)
                 if d.model is not None:
                     w.setModel(d.model)
@@ -837,7 +909,7 @@ class TaurusGui(TaurusMainWindow):
         MONITOR = getattr(conf, 'MONITOR', self.__getVarFromXML(xmlroot,"MONITOR", []))
         if MONITOR:
             CUSTOM_APPLETS.append(AppletDescription('monitor', classname='TaurusMonitorTiny', model=MONITOR) )
-
+        
         #get custom applet descriptions from the python config file      
         CUSTOM_APPLETS += [obj for name,obj in inspect.getmembers(conf) if isinstance(obj, AppletDescription)]
         
@@ -853,6 +925,8 @@ class TaurusGui(TaurusMainWindow):
         #create applet based on the descriptions gathered before
         for d in CUSTOM_APPLETS:
             try:
+                try: self.splashScreen().showMessage("Creating applet %s"%d.name)
+                except AttributeError: pass 
                 w = d.getWidget(sdm=Qt.qApp.SDM, setModel=False)
                 if d.model is not None:
                     w.setModel(d.model)
@@ -883,11 +957,18 @@ class TaurusGui(TaurusMainWindow):
         for a in EXTERNAL_APPS:
             self.addExternalAppLauncher(a.getAction())
         
+        
+        
         #get the "factory settings" filename. By default, it is called "default.ini" and resides in the configuration dir
         INIFILE = getattr(conf, 'INIFILE', self.__getVarFromXML(xmlroot,"INIFILE", "default.ini")) 
         iniFileName = os.path.join(self._confDirectory, INIFILE) #if a relative name is given, the conf dir is used as the root path
         
         #read the settings (or the factory settings if the regular file is not found)
+        msg = "Loading previous state"
+        if self.defaultConfigRecursionDepth >=0:
+            msg += " in Fail Proof mode"
+        try: self.splashScreen().showMessage(msg)
+        except AttributeError: pass
         self.loadSettings(factorySettingsFileName=iniFileName)
             
     def setLockView(self, locked):
@@ -915,54 +996,6 @@ class TaurusGui(TaurusMainWindow):
         '''
         self.statusBar().showMessage(msg)
     
-#    def onDoorNameChanged(self, doorname):
-#        ''' Slot to be called when the door name has changed
-#        
-#        :param doorname: (str) the tango name of the door device 
-#        '''
-#        if getattr(self, '__qdoor',None) is not None: #disconnect it from *all* shared data providing
-#            Qt.qApp.SDM.disconnectWriter("macroStatus", self.__qdoor, "macroStatusUpdated")
-#            Qt.qApp.SDM.disconnectWriter("doorOutputChanged", self.__qdoor, "outputUpdated")
-#            Qt.qApp.SDM.disconnectWriter("doorInfoChanged", self.__qdoor, "infoUpdated")
-#            Qt.qApp.SDM.disconnectWriter("doorWarningChanged", self.__qdoor, "warningUpdated")
-#            Qt.qApp.SDM.disconnectWriter("doorErrorChanged", self.__qdoor, "errorUpdated")
-#            Qt.qApp.SDM.disconnectWriter("doorDebugChanged", self.__qdoor, "debugUpdated")
-#            Qt.qApp.SDM.disconnectWriter("doorResultChanged", self.__qdoor, "resultUpdated")
-#             
-#        if doorname == "": return #@todo send signal with doorName to macroExecutorWidget in case of "" also send it to disconnect doorstateled
-#        door = taurus.Device(doorname)
-#        if not isinstance(door, Qt.QObject):
-#            msg= "cannot connect to door %s"%doorname
-#            Qt.QMessageBox.critical(self,'Door connection error', msg)
-#            return
-#        self.__qdoor = door
-#        Qt.qApp.SDM.connectWriter("macroStatus", self.__qdoor, "macroStatusUpdated")
-#        Qt.qApp.SDM.connectWriter("doorOutputChanged", self.__qdoor, "outputUpdated")
-#        Qt.qApp.SDM.connectWriter("doorInfoChanged", self.__qdoor, "infoUpdated")
-#        Qt.qApp.SDM.connectWriter("doorWarningChanged", self.__qdoor, "warningUpdated")
-#        Qt.qApp.SDM.connectWriter("doorErrorChanged", self.__qdoor, "errorUpdated")
-#        Qt.qApp.SDM.connectWriter("doorDebugChanged", self.__qdoor, "debugUpdated")
-#        Qt.qApp.SDM.connectWriter("doorResultChanged", self.__qdoor, "resultUpdated")
-#        #@todo: connect as a writer of other data as well
-    
-#    def onSelectedInstrumentChanged(self, instrumentname):
-#        ''' Slot to be called when the selected instrument has changed (e.g. by user
-#        clicking in the synoptic)
-#        
-#        :param instrumentname: (str) The name that identifies the instrument.
-#                               This name must be unique within the panels in the
-#                               GUI. This is also the name to be used for the
-#                               corresponding object in the jdraw synoptic
-#        '''
-#        instrumentname=unicode(instrumentname)
-#        try:
-#            inst = self.__panels[instrumentname]
-#            inst.show()
-#            inst.setFocus()
-#            inst.raise_()
-#        except KeyError:
-#            pass
-    
     def hideAllPanels(self):
         '''hides all current panels'''
         for panel in self.__panels.itervalues():
@@ -1018,14 +1051,21 @@ class TaurusGui(TaurusMainWindow):
         '''
         return copy.deepcopy(self.__instrumentToPanelMap) 
     
-    def setAllInstrumentAssociations(self, associationsdict):
+    def setAllInstrumentAssociations(self, associationsdict, clearExisting=False):
         '''
-        Sets the dictionary of instrument-panel associations
-                
-        :return: (dict<str,str>) a dict whose keys are the instruments names
-                 and whose values are the corresponding associated panels (or None).
+        Sets the dictionary of instrument-panel associations. 
+        By default, it keeps any existing association not present in the associationsdict.
+        
+        :param associationsdict: (dict<str,str>) a dict whose keys are the instruments names
+                                 and whose values are the corresponding associated panels (or None)
+        :param clearExisting: (bool) if True, the the existing asociations are cleared. 
+                              If False (default) existing associations are 
+                              updated with those in associationsdict
         '''
-        self.__instrumentToPanelMap = copy.deepcopy(associationsdict)
+        if clearExisting:
+            self.__instrumentToPanelMap=copy.deepcopy(associationsdict)
+        else:
+            self.__instrumentToPanelMap.update(copy.deepcopy(associationsdict))
         self.__panelToInstrumentMap = {}
         for k,v in self.__instrumentToPanelMap.iteritems():
             self.__panelToInstrumentMap[v]=k
@@ -1101,7 +1141,97 @@ class TaurusGui(TaurusMainWindow):
         '''reimplemented from :class:`TaurusMainWindow` to show the manual in a panel (not just a dockwidget)'''
         self.setFocusToPanel('Manual')
 
-
+    def onExportCurrentPanelConfiguration(self, fname=None):
+        
+        if fname is None:
+            fname = self._xmlConfigFileName
+                
+        if self._xmlConfigFileName is None:
+            xmlroot = etree.Element("taurusgui_config")
+        else:
+            try:
+                f = open(self._xmlConfigFileName,'r')
+                xmlroot = etree.fromstring(f.read())
+                f.close()
+            except Exception,e:
+                self.error('Cannot parse file "%s": %s',self._xmlConfigFileName, str(e))
+                return
+        
+        #retrieve/create the PanelDescriptions node
+        panelDescriptionsNode = xmlroot.find("PanelDescriptions")
+        if panelDescriptionsNode is None:
+            panelDescriptionsNode = etree.SubElement(xmlroot, "PanelDescriptions")
+        
+        #Get all custom panels
+        dlg = QDoubleListDlg(winTitle='Export Panels to XML', 
+                             mainLabel='Select which of the custom panels you want to export as xml configuration',
+                             label1='Not Exported', label2='Exported', 
+                             list1=[n for n,p in self.__panels.iteritems() if p.isCustom()], list2=[] )
+        result = dlg.exec_()
+        if result != Qt.QDialog.Accepted:
+            return
+        exportlist = dlg.getAll2()
+        
+        #create xml for those to be exported
+        registered = self.getConfigurableItemNames()
+        for name in exportlist:
+            panel = self.__panels[name]
+            if name not in registered:
+                panel.setPermanent(True)
+                self.registerConfigDelegate(panel, name)
+            panelxml = PanelDescription.fromPanel(panel).toXml()
+            panelDescriptionsNode.append(etree.fromstring(panelxml))
+        xml = etree.tostring(xmlroot, pretty_print=True)
+        
+        #write to file
+        while True: 
+            if fname is None:
+                fname = Qt.QFileDialog.getOpenFileName(self, "Open File", fname or self._confDirectory, self.tr("XML files (*.xml)"))
+                if not fname:
+                    return
+            fname = str(fname)
+            #backup the file   
+            if os.path.exists(fname):
+                import shutil
+                try:
+                    bckname = "%s.orig"%fname
+                    shutil.copy(fname,bckname)
+                except:
+                    self.warning("%s will be overwritten but I could not create a backup in %s", fname,bckname)
+            #write the data
+            try:     
+                f = open(fname,'w')
+                f.write(xml)
+                f.close()
+                break
+            except Exception,e:
+                msg = 'Cannot write to %s: %s'%(fname,str(e))
+                self.error(msg)
+                Qt.QMessageBox.warning(self, "I/O problem", msg+'\nChoose a different location.', Qt.QMessageBox.Ok, Qt.QMessageBox.NoButton)
+                fname = None
+        
+        
+        hint = "XML_CONFIG = %s"%os.path.relpath(fname, self._confDirectory)
+        msg = 'Configuration written in %s'%fname 
+        self.info(msg)
+        Qt.QMessageBox.information(self, "Configuration updated", 
+                                   msg+'\nMake sure that the .py configuration file in %s contains\n%s'%(self._confDirectory, hint), 
+                                   Qt.QMessageBox.Ok, Qt.QMessageBox.NoButton)
+        
+        return
+    
+    def showSDMInfo(self):
+        '''pops up a dialog showing the current information from the Shared Data Manager'''
+        #w = Qt.QMessageBox( self)
+        text = 'Currently managing %i shared data objects:\n%s'%(len(Qt.qApp.SDM.activeDataUIDs()),', '.join(Qt.qApp.SDM.activeDataUIDs()) )
+        nfo = Qt.qApp.SDM.info()
+        w = Qt.QMessageBox ( Qt.QMessageBox.Information, 'Shared Data Manager Information', text, 
+                             buttons = Qt.QMessageBox.Close, parent = self )
+        w.setDetailedText(nfo)
+        w.show()
+        self.info(nfo)
+        
+        
 #------------------------------------------------------------------------------ 
 def main():
     import sys
@@ -1118,29 +1248,34 @@ def main():
                   help="use the given configuration directory for initialization")
     parser.add_option("", "--new-gui", action="store_true", dest="new_gui", default=None,
                   help="launch a wizard for creating a new TaurusGUI application")
+    parser.add_option("", "--fail-proof", action="store_true", dest="fail_proof", default=None,
+                  help="launch in fail proof mode (it prevents potentially problematic configs from being loaded)")
 
     app = TaurusApplication(cmd_line_parser=parser, app_name="taurusgui",
                             app_version=taurus.Release.version)
     args = app.get_command_line_args()
     options = app.get_command_line_options()
-    
+
     if options.new_gui: #launch app settings wizard instead of taurusgui
         from taurus.qt.qtgui.taurusgui import AppSettingsWizard
-        Qt.QMessageBox.information(None, 'Alpha-quality warning', 
-                                   'The Applications settings wizard is still under heavy development.\n Use it at your own risk and report any problems',
-                                   Qt.QMessageBox.Ok)
         wizard = AppSettingsWizard()
         wizard.show()
         sys.exit(app.exec_())
         
     confname = options.config_dir
-    if confname is None and len(args) == 1: #for backwards compat, we allow to specify the confname without the "--config-dir" parameter
-        confname = args[0]
+    if confname is None:
+        if len(args) == 1: #for backwards compat, we allow to specify the confname without the "--config-dir" parameter
+            confname = args[0]
+        else:
+            parser.print_help(sys.stderr)
+            sys.exit(1)
+    
+    if options.fail_proof:
+        configRecursionDepth = 0
     else:
-        parser.print_help(sys.stderr)
-        sys.exit(1)
+        configRecursionDepth = None
         
-    gui = TaurusGui(None, confname=confname)
+    gui = TaurusGui(None, confname=confname, configRecursionDepth=configRecursionDepth)
     
     gui.show()
     ret = app.exec_()
