@@ -3,21 +3,21 @@
 #############################################################################
 ##
 ## This file is part of Taurus, a Tango User Interface Library
-## 
+##
 ## http://www.tango-controls.org/static/taurus/latest/doc/html/index.html
 ##
 ## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
-## 
+##
 ## Taurus is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU Lesser General Public License as published by
 ## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
-## 
+##
 ## Taurus is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU Lesser General Public License for more details.
-## 
+##
 ## You should have received a copy of the GNU Lesser General Public License
 ## along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
 ##
@@ -35,43 +35,20 @@ from taurus.qt.qtgui.util import ActionFactory
 from taurus.qt.qtgui.resource import getIcon, getThemeIcon
 
 
-class _NavigationLabel(Qt.QWidget):
-    """Internal widget providing a navigation label & icon"""
-    
-    def __init__(self, pixmap, text, cont, index, parent=None):
-        super(_NavigationLabel, self).__init__(parent)
-        self._index = index
-        
-        l = Qt.QHBoxLayout()
-        l.setContentsMargins(0,0,0,0)
-        self.setLayout(l)
-        if pixmap is not None:
-            p = Qt.QLabel()
-            p.setPixmap(pixmap)
-            l.addWidget(p)
-        self._label = Qt.QLabel(text)
-        l.addWidget(self._label)
-        if cont:
-            l.addWidget(Qt.QLabel(u" \u00bb "))
-    
-    def label(self):
-        return self._label
-    
-    def index(self):
-        return self._index
-
-
 class _NavigationWidget(Qt.QFrame):
     """Internal widget that provides a navigation path to be placed in a toolbar"""
-     
+
     def __init__(self, treeWidget, toolBarWidget, parent=None):
         super(_NavigationWidget, self).__init__(parent)
         self._tree = treeWidget
         self._toolbar = toolBarWidget
+        self._label = Qt.QLabel()
         l = Qt.QHBoxLayout()
         l.setContentsMargins(4, 0, 4, 0)
         self.setLayout(l)
-   
+        l.addWidget(self._label)
+        
+
     def treeWidget(self):
         return self._tree
 
@@ -80,55 +57,21 @@ class _NavigationWidget(Qt.QFrame):
 
     def toolBarWidget(self):
         return self._toolbar
-    
-    def clean(self):
-        l = self.layout()
-        w = l.takeAt(0)
-        while w is not None:
-            w = w.widget()
-            Qt.QObject.disconnect(w, Qt.SIGNAL("clicked()"), self.onGotoNode)
-            w.setParent(None)
-            w = l.takeAt(0)
 
     def updateSelection(self, index):
-        self.clean()
         treeWidget = self.treeWidget()
-        viewWidget = treeWidget.viewWidget()
-        model = viewWidget.model()
         src_model = treeWidget.getBaseQModel()
         toolbar = self.toolBarWidget()
-        size = None
-        rootPixmap = getThemeIcon("go-home").pixmap(toolbar.iconSize())
-        rootText = u"Top"
+        txt = u""
         
-        l = self.layout()
-        n = 0
         while index.isValid():
             src_index = treeWidget._mapToSource(index)
             name = src_model.pyData(src_index, Qt.Qt.DisplayRole)
-            font = src_model.pyData(src_index, Qt.Qt.FontRole)
-            tooltip = src_model.pyData(src_index, Qt.Qt.ToolTipRole)
-            decoration = src_model.pyData(src_index, Qt.Qt.DecorationRole) 
-            pixmap = None
-            if isinstance(decoration, Qt.QIcon):
-                pixmap = decoration.pixmap(toolbar.iconSize())
-            elif isinstance(decoration, Qt.QPixmap):
-                pixmap = decoration
-            button = _NavigationLabel(pixmap, name, n>0, Qt.QPersistentModelIndex(index))
-            font = font or model.DftFont
-            button.setFont(font)
-            button.setToolTip(tooltip)
-            Qt.QObject.connect(button.label(), Qt.SIGNAL("linkActivated(const QString &)"), self.onGotoNode)
-            l.insertWidget(0, button)
+            txt = u" \u00bb " + name + txt
             index = index.parent()
-            n += 1
-        rootButton = _NavigationLabel(rootPixmap, rootText, n>0, Qt.QPersistentModelIndex(index))
-        rootButton.setFont(model.DftFont)
-        #Qt.QObject.connect(rootButton.label(), Qt.SIGNAL("linkActivated(const QString &)"), self.onGotoNode)
-        l.insertWidget(0, rootButton)
-        
-        lastLabel = l.itemAt(l.count()-1).widget()
-        
+        txt = u"Root" + txt
+        self._label.setText(txt)
+
     def onGotoNode(self, *args):
         label = self.sender()
         persistent_index = label.parent().index()
@@ -139,11 +82,11 @@ class _NavigationWidget(Qt.QFrame):
 
 
 class NavigationToolBar(BaseToolBar):
-    
+
     def __init__(self, view=None, parent=None, designMode=False):
         BaseToolBar.__init__(self, name="Taurus selection toolbar", view=view,
                              parent=parent, designMode=designMode)
-        
+
         af = ActionFactory()
         self._goIntoAction = af.createAction(self, "Go Into",
                                              icon=getThemeIcon("go-down"),
@@ -165,13 +108,13 @@ class NavigationToolBar(BaseToolBar):
 
     def goIntoAction(self):
         return self._goIntoAction
-    
+
     def goTopAction(self):
         return self._goTopAction
-    
+
     def goUpAction(self):
         return self._goUpAction
-    
+
     def goInto(self):
         self.emit(Qt.SIGNAL("goIntoTriggered"))
 
@@ -183,7 +126,7 @@ class NavigationToolBar(BaseToolBar):
 
 
 class ExpansionBar(BaseToolBar):
-    
+
     def __init__(self, view=None, parent=None, designMode=False):
         BaseToolBar.__init__(self, name="Taurus selection toolbar", view=view,
                              parent=parent, designMode=designMode)
@@ -209,40 +152,53 @@ class ExpansionBar(BaseToolBar):
         self.addAction(self._collapseAllAction)
         self.addAction(self._expandSelectionAction)
         self.addAction(self._collapseSelectionAction)
-    
+
     def onExpandAll(self):
         self.emit(Qt.SIGNAL("expandTriggered"))
-    
+
     def onCollapseAll(self):
         self.emit(Qt.SIGNAL("collapseTriggered"))
-    
+
     def onExpandSelection(self):
         self.emit(Qt.SIGNAL("expandSelectionTriggered"))
-    
+
     def onCollapseSelection(self):
         self.emit(Qt.SIGNAL("collapseSelectionTriggered"))
 
 
 class QBaseTreeWidget(QBaseModelWidget):
     """A pure Qt tree widget implementing a tree with a navigation toolbar"""
-    
+
     def __init__(self, parent=None, designMode=False, with_navigation_bar=True,
-                 with_filter_widget=True):
-        self._with_navigation_bar = with_navigation_bar
-        QBaseModelWidget.__init__(self, parent, with_filter_widget=with_filter_widget)
-    
+                 with_filter_widget=True, with_selection_widget=True,
+                 with_refresh_widget=True, perspective=None, proxy=None):
+
+        if with_navigation_bar:
+            if isinstance(with_navigation_bar, (bool, int)):
+                self._with_navigation_bar = NavigationToolBar
+            else:
+                self._with_navigation_bar = with_navigation_bar
+        else:
+            self._with_navigation_bar = None
+
+        QBaseModelWidget.__init__(self, parent,
+            with_filter_widget=with_filter_widget,
+            with_selection_widget=with_selection_widget,
+            with_refresh_widget=with_refresh_widget,
+            perspective=perspective, proxy=proxy)
+
     def createToolArea(self):
         ta = QBaseModelWidget.createToolArea(self)
-        
+
         e_bar = self._expandBar = ExpansionBar(view=self, parent=self)
         self.connect(e_bar, Qt.SIGNAL("expandTriggered"), self.expandAllTree)
         self.connect(e_bar, Qt.SIGNAL("collapseTriggered"), self.collapseAllTree)
         self.connect(e_bar, Qt.SIGNAL("expandSelectionTriggered"), self.expandSelectionTree)
         self.connect(e_bar, Qt.SIGNAL("collapseSelectionTriggered"), self.collapseSelectionTree)
         ta.append(e_bar)
-        
+
         if self._with_navigation_bar:
-            n_bar = self._navigationToolBar = NavigationToolBar(view=self, parent=self)
+            n_bar = self._navigationToolBar = self._with_navigation_bar(view=self, parent=self)
             self.connect(n_bar, Qt.SIGNAL("goIntoTriggered"), self.goIntoTree)
             self.connect(n_bar, Qt.SIGNAL("goTopTriggered"), self.goTopTree)
             self.connect(n_bar, Qt.SIGNAL("goUpTriggered"), self.goUpTree)
@@ -250,9 +206,11 @@ class QBaseTreeWidget(QBaseModelWidget):
         else:
             self._navigationToolBar = None
         return ta
-    
-    def createViewWidget(self):
-        tree = Qt.QTreeView()
+
+    def createViewWidget(self, klass=None):
+        if klass is None:
+            klass = Qt.QTreeView
+        tree = klass()
         tree.setSortingEnabled(True)
         tree.setUniformRowHeights(True)
         tree.setAlternatingRowColors(True)
@@ -267,47 +225,46 @@ class QBaseTreeWidget(QBaseModelWidget):
         h = tree.header()
         h.setResizeMode(0, Qt.QHeaderView.Stretch)
         return tree
-    
+
     def treeView(self):
         return self.viewWidget()
 
     def goIntoAction(self):
         return self._navigationToolBar.goIntoAction()
-    
+
     def goTopAction(self):
         return self._navigationToolBar.goTopAction()
-    
+
     def goUpAction(self):
         return self._navigationToolBar.goUpAction()
-    
+
     def expandAllTree(self):
         self.statusBar().showMessage("Expanding all items... (it may take a few seconds)")
-        tree = self.viewWidget()
         Qt.QTimer.singleShot(0, self._expandTree)
-    
+
     def _expandTree(self):
         tree = self.viewWidget()
         tree.expandAll()
         self.statusBar().showMessage("All items expanded!", 3000)
-    
+
     def onExpanded(self):
         self.resizeColumns()
-    
+
     def collapseAllTree(self):
         self.viewWidget().collapseAll()
-    
+
     def expandSelectionTree(self):
         tree = self.viewWidget()
         index = tree.currentIndex()
         if index.isValid():
             tree.expand(index)
-        
+
     def collapseSelectionTree(self):
         tree = self.viewWidget()
         index = tree.currentIndex()
         if index.isValid():
             tree.collapse(index)
-    
+
     def resizeColumns(self):
         tree = self.viewWidget()
         model = tree.model()
@@ -315,34 +272,34 @@ class QBaseTreeWidget(QBaseModelWidget):
             return
         for c in range(model.columnCount()):
             tree.resizeColumnToContents(c)
-    
+
     def goIntoTree(self):
         tree = self.viewWidget()
         index = tree.currentIndex()
         base_index = self._mapToSource(index)
-        
+
         if not index.isValid():
             return
-        
+
         # do not enter if the item doesn't have any children
         if base_index.internalPointer().childCount() == 0:
             return
-    
+
         tree.setRootIndex(index)
         tree.setCurrentIndex(index.child(0, 0))
         self._updateToolBar()
-    
+
     def goUpTree(self):
         tree = self.viewWidget()
         index = tree.rootIndex()
         if not index.isValid():
             return
         index_parent = index.parent()
-        
+
         tree.setRootIndex(index_parent)
         tree.setCurrentIndex(index)
         self._updateToolBar()
-    
+
     def goTopTree(self):
         tree = self.viewWidget()
         current_root = tree.rootIndex()
@@ -352,7 +309,7 @@ class QBaseTreeWidget(QBaseModelWidget):
         tree.setRootIndex(p)
         tree.setCurrentIndex(p)
         self._updateToolBar()
-    
+
     def _updateToolBar(self, current=None, previous=None):
         if not self._with_navigation_bar:
             return
@@ -360,7 +317,7 @@ class QBaseTreeWidget(QBaseModelWidget):
         if current is None:
             current = tree.currentIndex()
         goInto = False
-        
+
         base_current = self._mapToSource(current)
         if current.isValid():
             ip = base_current.internalPointer()
@@ -369,11 +326,11 @@ class QBaseTreeWidget(QBaseModelWidget):
         self._navigationToolBar._goIntoAction.setEnabled(goInto)
         self._expandBar._expandSelectionAction.setEnabled(goInto)
         self._expandBar._collapseSelectionAction.setEnabled(goInto)
-        
+
         goUp = tree.rootIndex().isValid()
         self._navigationToolBar._goUpAction.setEnabled(goUp)
         self._navigationToolBar._goTopAction.setEnabled(goUp)
-        
+
         index = tree.rootIndex()
         self._navigationToolBar._navigationWidget.updateSelection(index)
-    
\ No newline at end of file
+
